package WHATAgent;

use strict;
use base 'Agent';

sub isSelectable {"WHATAMI"} # displayed by the GUI in front of a checkbox that allows

# admin to turn this Agent off.
sub revision {'$Revision: 1.1 $'}

sub new { # standard to all Agent packages.
	my($self) = Agent->new();
	bless ($self, 'WHATAgent');
	return $self;
}
sub RUN {
	my($agent) = @_;

	if (!$agent->category_selected) {
		Debug->print2("Skipping " . ref($agent) . ".");
		return;
	}

	my($pdm) = $agent->get_pdm(); # get handle to the PDM
	my($renv) = System->get_renv(); # get environment data-struct
	Debug->print1("-> WHATAgent: Reading host configuration");
	my $key = System->hostid(); # get hostid as the key for this report
	my $name = System->hostname(); # name of report
	my $command = System->find_executable("whatami"); # find this executable by trying the usual directories.
	my($err,$com) = Util->run_command( $command, "whatami.txt",20); # run the command, timeout after 20 secs.
	if ($key) {
		Catalog->deviceWrite($key, ['what', $name, $key,undef,undef,undef, undef, undef]);
		# store name and key in Catalog. (appendix B)
	} else {
		Debug->err(TEXT => "Cannot identify this host, Skipping report"); # report an error when hostid fails.
		return; # get-out
	}
	# parse $com and generate data structure.
	my %Report;
	foreach my $l (@$com) { # $com is a pointer to an array, thus @$ is needed.
		my($name, $val) = split(/: /, $l); # separate
		$name = Util->ltrim($name); # remove leading spaces
		$name =~ s/ /_/g; # join
		$Report{$name} = $val;
	}
	my $report = \%Report; # $report must be a hash pointer.
	$agent->addIdentification($report); # add device location to the report

	my $id = { accessName => $name,
		deviceName => $key,
		active => "Y",
		logFile => "",
		name => $name,
		category => Report::CAT_WHAT, # valid category
		ip => "",
	};
	if ($err) {
		$pdm->saveReport(
		Report->new($id, $report , undef, Report::STATUS_CANNOT_CONNECT));
	} else {
		$pdm->saveReport(Report->new($id, $report , undef)); # normal status.
	}
}
sub REPORT {
	my($class, $rep) = @_;
	my($name) = $rep->name(); # rep is the Report object,
	my($report) = $rep->value();# $rep->value is the actual hash created previously.
	print "<b>WHATAMI Report for $name</b><p>";
	print "<table border=1 cellspacing=0 width=90% bgcolor=white><tr><th>Name<th>Value</tr>";
	foreach my $k (keys %$report) {
		print "<tr><td>$k <td> $report->{$k}</td>";
	}
	print "</table>";
}
sub FCCounters {
	my($class, $arg) = @_;
	return undef;
}
1;
