
package Agent;
#  $Name:  $ 
#  $Id: Agent.pm,v 1.19 2001/09/28 16:00:20 ccadieux Exp $

use Net::Ping;
use strict;
use Carp;
use vars qw($AUTOLOAD);
use System;
use Timelapse;
use Debug;
use PDM;
use Data::Dumper;
use PDM::Thresholds;
use Report;
use Catalog;
use Debug;


use vars qw(%_FIELDS %LOGFILES %DEVICES %CACHES);

%_FIELDS = ('can_be_turned_off', undef);

#
# $agent = Agent->new({data => $DATA, today => $today, opt_T => $opt_T,
#                   });
# Usage:
# $agent->last_run
#       ->new
#       ->read_log_file 
#       ->format_dev_info 
#       ->ping 
#       ->telnet
#       ->diff 
#       ->read_file2string 
#       ->read_file2array 
#       ->write_array2cache 
#       ->load_cache
#       ->save_cache
#

sub new {
  my($this) = {};
  bless($this, 'Agent');
  my $hostid   = `/bin/hostid`; chomp($hostid);

  $this->{hostid} = $hostid;
  return $this;
}

sub init {
  my($agent,$vals) = @_;
  my($x);

  foreach $x (keys %$vals) {
    $agent->{$x} = $vals->{$x};
  }
}



#############################################################
#       READK_LOG_FILE
#############################################################

%LOGFILES = ();

# flag = NEW : new lines since last run
#        24  : lines written in the ast 24 hours.
#
# << Oct092000_16:31:20 >>
# Oct 11 14:52:05
#
  
sub read_log_file {
  my($agent, $file, $flag, $is24) = @_;
  my(@lines, $startdate, $seek, $size, $tell, $size0, $clear_seek);
  return ('',undef) if (!$file);

  my($cache) = PDM->getCacheHandle("read_log_file");

  if ($LOGFILES{$file}) {
    return ('', $LOGFILES{$file});
  }

  if (!(-r $file && -f $file)) {
    $LOGFILES{$file} = [];
    return ("CANNOT_READ $file", undef);
  }

  my $today = Util->today;
  if ($cache->{START}{$file}) {
      $startdate = $cache->{START}{$file};
  } else {
      $startdate= substr($today,0,5) . " 00:00:00";
  }
  
  Debug->print('READING_MESSAGE_FILE', $file);

  open(FILE_F1, $file); 
  my $cnt=0;
  $seek = $cache->{SEEK}{$file};

  if ($flag eq "NEW") {    # find new stuff.
    $size = (stat($file))[7];
    if ($seek) {
      if ($seek < $size) {
         seek(FILE_F1, $seek, 0);

      } elsif ($seek == $size) {
         close(FILE_F1);
         $LOGFILES{$file} = [];
         return ('',undef);     # found nothing

      } elsif (-f "$file.0") {
         $size0 = (stat("$file.0"))[7];
         if ($size0 > $seek) {  # do this one also
            open(FILE_F1, "$file.0"); 
            seek(FILE_F1, $seek, 0);
            $clear_seek = 1;
         }
      }
    }         
  }
  my($line, $current);
  my $count;
  for ($tell = tell(FILE_F1); $line = <FILE_F1>; $tell = tell(FILE_F1)) { 
      chop($line);
      last if ($line eq "exit"); # for tests only

      my @a = split(/ +/, $line,6);
      if ($a[0] eq "<<") {
        $current = sprintf("%2.2d-%2.2d %s", Util->MTH(substr($a[1],0,3)), 
                       substr($a[1],3,2), substr($a[1],10) );
      } else {
        $current = sprintf("%2.2d-%2.2d %s", Util->MTH($a[0]), $a[1], $a[2]);
      }

      next if (substr($current,0,5) gt substr($today,0,5)); # if too big, probably the wrong year.
      if ($flag eq "NEW" &&  $current ge $startdate) {
           push(@lines, $line);
      }
      if ($flag eq "24") {
        if ($is24 && $current ge $is24 ) { # find 24 hours for thresholds
          push(@lines , $line);
        }
      }
      last if ($count++ > 7000);
  }
  if ($clear_seek) {
    $cache->{SEEK}{$file} = 0;
  } elsif ($tell && $tell != $seek) {
    $cache->{SEEK}{$file} = $tell;
  }
  close(FILE_F1);
  $LOGFILES{$file} = \@lines;  # save for next request.
  $cache->{START}{$file} = $current; # move pointer to end of file.

  return ('', \@lines);
}

#
# select devices to monitor

sub deviceList {
  my($class) = @_;
  my(@LIST);
  my(@a) = split(/=/, $class);
  
  my($prefix) = lc(substr($a[0], 0, -5));

  my($devs) = System->get_configDevices();
  foreach my $d (@$devs) {
     next if ($d->{type} ne $prefix);
     next if (!Util->isMineToMonitor($d));
     push(@LIST, $d);
  }
  return @LIST;
}


sub get_pdm {
  my($class) = @_;

  return $class->{pdm};
}
  

sub format_dev_info {
  my($device) = @_;
  my($info);
  if ($device->{serial}) {
     $info = "S" . $device->{serial};
  } else {
     $info = "N" . $device->{type} . "-" . $device->{_name} . "-" . $device->{name} . "-" . $device->{ip};
  }
  $info;
}

sub addIdentification {
  my($agent, $report) = @_;

  my($env) = $agent->{renv};
  $report->{'location.customerNo'}   = $env->{cust_no};
  $report->{'location.customerName'} = $env->{customer};
  $report->{'location.contractNo'}   = $env->{contract};
  $report->{'location.hostId'}        = System->hostid;

  $report->{'location.siteName'}      = $env->{site_name};
  $report->{'location.siteAddress'}   = $env->{site_address};
  $report->{'location.siteCity'}      = $env->{site_city};
  $report->{'location.siteState'}     = $env->{site_state};
  $report->{'location.siteCountry'}   = $env->{site_country};
  $report->{'location.siteZip'}       = $env->{site_zip};
  $report->{'location.siteContact'}   = $env->{site_contact};
  $report->{'location.siteEmail'}     = $env->{site_email};
  $report->{'location.ticker'}        = $env->{ticker};
  $report->{'location.production'}    = $env->{production};

}


#######################################################
#                 DISK_ INQUIRY STORE
#######################################################
%DEVICES = ();

sub di_register {
  my($agent, $key_type, $key, $a) = @_;

  $key_type = uc($key_type);

  $DEVICES{$key_type}{$key} = $a; 
}

sub di_list {
  my($agent, $key_type) = @_;
  my($x, @list, $p);
  $key_type = uc($key_type);
  $p = $DEVICES{$key_type};
  foreach $x (keys %$p ) {
    push(@list, $x);
  }
  return @list;
}

sub di_lookup {
  my($agent, $key_type, $key) = @_;
  
  $key_type = uc($key_type);

  if ($DEVICES{$key_type}{$key}) {
     return $DEVICES{$key_type}{$key};
  } else {
     return undef;
  }
}

########################################################
#                  ERRORS
########################################################

sub system_error {
   my($agent, $device, $object, $type) = @_;
   my($o, $now, $serial, %V);
   
   %V = ();
   $now = Util->get_today;
   $o =<<EOF;
<elem elem_type=\"system_error\" system_name="$device->{name}" id="$device->{_name}" date="$now" error_type="$type" device_type=\"$device->{type}\" />
EOF
  return $o;
}

sub reconnect_info {
   my($agent, $device, $object, $kt, $k) = @_;
   my($o, $now, $serial, %V);
   
   %V = ();
   $now = Util->get_today;
   $o =<<EOF;
<elem elem_type=\"connect_info\" system_name="$device->{name}" key_type="$kt" key="$k" ip="$device->{ip}" date="$now" error_type="RECONNECTED" device_type=\"$device->{type}\"/>
EOF
  return $o;
}




########################################################
#                  SOCKETS
########################################################
#  This ping will remember what happened last time it tryed to ping so that
#  only newly-up and newly-down are reported, repeats fail silently.
#  $mode : D : newly down
#  $mode : U : newly up

sub smart_ping {
  my($agent, $ip, $PROMPT, $mode) = @_;
  my($con_errs) = $agent->load_cache('connect_errors.state');

  if ($agent->ping($ip , $PROMPT)) {
     Debug->print('SUCCESSFUL_PING', $ip);
     if ($con_errs->{$ip}) {
          Debug->print('TELNET_IS_BACK');
          $con_errs->{$ip} = 0;   # reset if it was down
          $$mode = "U";
     }
     return 1;
  } else {
     Debug->err(CANNOT_TELNET =>  $ip);
     if (!$con_errs->{$ip}) {
        Debug->print('TELNET_IS_NOW_DOWN');
        $con_errs->{$ip} = 1;
        $$mode = "D";
     }
  }
  return 0;
}

sub pingXX {
  my($agent, $host_ip, $prompt, $timeout) = @_;
  my($t);
  $timeout = 10 if (!$timeout);
  $t = new Net::Telnet (
                  errmode   => "return",
                  Timeout   => $timeout,
                  Prompt    => $prompt,
           );
  if (!defined($t->open($host_ip))) {
     return 0;
  }
  $t->close;
  return 1;
}

sub telnetXX {
  my($agent, $host_ip, $cmd,  $timeout, $prompt, $login,$static) = @_;
  my($t, @lines, $i);
  my($pass) = "";

  if (System->get_testMode() && $static) {
     return Util->as_array(Util->readf(System->get_home() . "/Test/$static"));
  }
  if (($i = index($login,"/")) > 0) {
     $pass = substr($login,$i+1);
     $login = substr($login,0, $i);
  }
  $timeout = 60 if (!$timeout);

#'/' . $prompt . ':\/:\<\d+\>/',

  $t = new Net::Telnet (
                  errmode => "return",
                  Timeout   => $timeout,
                  Prompt    => $prompt,
           );
  if (!defined($t->open($host_ip))) {
     return undef;
  }
  $t->login($login, $pass);
  @lines = $t->cmd($cmd);
  $t->close;
# my($x) = join(/\n/, @lines); 
# open(O, ">" . System->get_home()  . "/Test/$static"); print O $x; close(O);
  return \@lines;

}

####################################################
#             DIFF
####################################################
sub get_category {
  my($agent) = @_;
  my($cat) = ref($agent);
  $cat =~ s/Agent//;
  return $cat;
}

sub connect_error {
   my($agent, $device, $object, $kt, $k) = @_;
   my($o, $now, $serial);
   
   my %V = ();
   $now = Util->get_today;
   $o =<<EOF;
<elem elem_type=\"connect_error\" system_name="$device->{name}" key_type="$kt" key="$k" ip="$device->{ip}" date="$now" error_type="CANNOT_CONNECT" device_type=\"$device->{type}\"/>
EOF
  return $o;
}

# $id = [name, ip, key_type, key, key_type2, key2 ]
#
sub xml_event_holder {
  my($agent, $event_type, $id, $class) = @_;
  my($head, $cat, $type, $today, $altkey, $key, $es);

  $cat = $agent->get_category;
  $class = "compare" if (!$class);

  $type = Event->get_xml_name($event_type);
  $today =  Util->today("YMDH");
  if ($id->[3]) {
    $key = " key_type=\"$id->[2]\" key=\"$id->[3]\" ";
  }
  if ($id->[5]) {
    $altkey = " key_type2=\"$id->[4]\" key2=\"$id->[5]\" ";
  }
  $head = "<event event_type=\"$type\" message_sequence=\"$agent->{sequence}\">
  <event_info category=\"$cat\" name=\"$id->[0]\" hostid=\"$agent->{hostid}\" ip=\"$id->[1]\" $key $altkey date=\"$today\" />";
  return ($head, "</event>");
}


sub save_current_run {
  my($agent, $event_type, $data, $id) = @_;
  my($file, $head, $tail);

  my($cat) = $agent->get_category;
  $file = "$agent->{data}/RUN/$cat:$event_type:$id->[0]";

  ($head,$tail) = $agent->xml_event_holder($event_type, $id);
  if (ref($data) eq "ARRAY") {
    if ($#$data >= 0) {
      $data = join("\n", @$data);
    }
  }
  return undef if (!$data);
  $data = $head . "\n" . $data . "\n" . $tail;
  if (open(O, ">$file")) {
    print O $data;
    close(O);
    chmod 0666, $file;
  } else {
    Debug->err(CANNOT_WRITE =>  "$file: $!");
  }
}

sub add_mode {
  my($agent, $p, $m) = @_;
  my($head);

  my($ix) = index($$p, ">\n");

  $head = substr($$p, 0, $ix) . " modes=\"$m\">\n" . substr($$p, $ix+2);
  $$p = $head;
}

sub xml_add_sequence {
  my($agent, $p) = @_;
  my($head);

  my($ix) = index($$p, ">\n");

  my($es) = "12";
  $head = substr($$p, 0, $ix) . " event_no=\"$es\">\n" . substr($$p, $ix+2);
  $$p = $head;
}

# new: a string with \n in it
# returns: a pointer to an array of differences
# n=new, i=insert, d=delete, u=update
# flags : {xml => 1, ignorePattern=> "Temp,other", hash => 1, rawdata => 1, block => 1, 
#          xml_key=>}
# block : the content is compare as ablack, not line by line
# xml_key: xml key to help comparing the right lines
# DIFF saves the current values to TMP_DIFF
# Hash : Only report differences to entry with same hash value
#     needs 3 values Key | value | 0/1   0=bad, 1=good

sub diff_entry {
  my($agent, $type, $key, $value, $info, $error) = @_;
  return "<elem elem_type=\"$type\" key=\"$key\" value=\"$value\" info=\"$info\" error=\"$error\" />\n";
}




#################################################
#      CACHE FILE OPERATIONS
#################################################

sub read_file2string {
  my($agent, $file) = @_;
  my($l, $out);

  if (open(O3, "$agent->{data}/$file")) {
    while ($l = <O3>) {
       $out .= $l;
    }
    close(O3);
    return $out;
  } else {
    return undef;
  }
}


sub read_file2array {
  my($agent, $file) = @_;
  my(@last_config, $l, @a);

  @last_config = ();

  if (open(O, "$file.conf")) {
    while ($l = <O>) {
      push(@last_config,$l);
    }
    close(O);
  } else {
    Debug->print1("Cache $file.conf not found!");
  }
  \@last_config;
}

sub write_array2cache {
  my($agent, $file, $last_config) = @_;
  my($x, $p);

  if (open(O,">$file.conf")) {
    foreach $x (@$last_config) {
     print O $x;
    }
    close(O);
  } else {
    Debug->print1("Cannot write to $file: $!");
  }
}


#####################################################
#           CACHE DATABASE OPERATIONS
#####################################################
#
# Load values from a cache-file into a hash
# Cache file usually stores device specific data about each devices.

%CACHES = ();

sub load_cache {
   my($agent, $f)  = @_;
   my(%starts, $l, @a);

   return $CACHES{$f} if ($CACHES{$f});

   if (open(O, "$agent->{data}/$f")) {
      while ($l = <O>) {
        chop($l);
        @a = split(/=/, $l);
        $starts{$a[0]} = $a[1];
      }
      close(O);
   }
   $CACHES{$f} = \%starts;
   return \%starts;
}

# STORE data in TMP, save_cache will save permanently

sub clear_diff_cache {
  my($agent, $data) = @_;
  my(@f1, $file1, $l);

  opendir(O, "$data/TMP_DIFF");
  @f1 = readdir(O);
  closedir(O);
  foreach $file1 (@f1) {
     unlink("$data/TMP_DIFF/$file1");
  }
}


sub string2file {
   my($agent, $data, $file) = @_;

   if (open(T, ">$file")) {
     print T $data;
     close(T);
   } else {
     Debug->err(CANNOT_WRITE => "$file: $!");
   }
}

#
# Write the hash back in the cache-file
#
sub save_cache {
  my($agent, $starts, $f) = @_;
  my($x, $type);
  my($data) = $agent->{data};
#  return if ($agent->test_mode); save nothing on tests.

  if ($starts) {  # if 'starts' is specified, save one cache-file.
    if (open(O, ">$data/$f")) {
      foreach $x (keys %$starts) {
        print O "$x=$starts->{$x}\n";
      }
      close(O);
      chmod 0666, "$data/$f";
    } else {
      Debug->err(WRITE_HASH_FAILED => "$data/$f",  $! );
    }
    return;
  }
}

sub save_all_caches {
  my($agent,$util) = @_;

# SAVE ALL CACHES IF starts not specified.
  my($data) = $util->{data};

  foreach my $type (keys %CACHES) {
    my $starts = $CACHES{$type};
    if (open(O, ">$data/$type")) {
      foreach my $x (keys %$starts) {
        print O "$x=$starts->{$x}\n";
      }
      close(O);
      chmod 0666, "$data/$type";
    } else {
      Debug->err(WRITE_HASH_FAILED => "$data/$type",  $! );
    }
  }
  &copy($data, "TMP_DIFF", "DIFF");
}

sub copy {
  my($data ,$from, $to) = @_;

  opendir(O, "$data/$from");

  my(@f1, $file1, $l);
  @f1 = readdir(O);
  closedir(O);
  foreach $file1 (@f1) {
     next if (substr($file1,0,1) eq ".");
     next if (!-f "$data/$from/$file1");
     open(R, "$data/$from/$file1");
     open(W, ">$data/$to/$file1");
     while ($l = <R>) {
        print W $l;
     }
     close(R); close(W);
     chmod 0666 , "$data/$to/$file1";
  }

}

sub AUTOLOAD {
  my $self = shift;
  my $type = ref($self) || croak "$self is not an object";

  my $name = $AUTOLOAD;
  $name =~ s/.*://;   # strip fully-qualified portion

  unless (exists $_FIELDS{$name} ) {
   croak "Can't access `$name' field in class $type";
  }
  if (@_) {
     return $self->{$name} = shift;
  } else {
     return $self->{$name};
  }
}

#####################################################
#           XML operations
#####################################################
#
#   <old>\n  $old[$x]\n </old>\n <new>\n  $new[$x]\n </new
#
sub xml_attr_diff {
  my($agent, $in_old, $in_new, $ignore) = @_;
  my($o, $n, @f1, @f2, $ko, $kn);
  my($k, $old, $new, $root, $found);
  
  $o = XMLinput($in_old, (keeproot => 1)) || {};
  $n = XMLinput($in_new, (keeproot => 1)) || {};

  @f1 = (keys %$o);
  @f2 = (keys %$n);
# MUST HAVE ONE ROOT
  if ($#f1 > 0 || $#f2 > 0) {
    return " <old>$in_old</old>\n <new>$in_new</new>\n";
  }
  $root = $f1[0];
  $ko = $o->{$root};
  $kn = $n->{$root};
  @f1 = sort keys %$ko;
  @f2 = sort keys %$kn;
# MUST HAVE SAME ATTRIBUTES
  $found =0;
  foreach $k (keys %$ko) {
    if (index(",elem_type,id,fruId,name,", ",$k,") >= 0) {
       $old .= "$k=\"$ko->{$k}\" ";
       $new .= "$k=\"$kn->{$k}\" ";
    } elsif ($ko->{$k} ne $kn->{$k}) {
       next if ($ignore && &found_pattern($ignore, $k) );
       $old .= "$k=\"$ko->{$k}\" ";
       $new .= "$k=\"$kn->{$k}\" ";
       $found = 1;
    }
  }
  foreach $k (keys %$kn) {
     if (!defined($ko->{$k})) {
       $new .= "$k=\"$kn->{$k}\" ";
       $found = 1;
     }
  }
        
  if ($found) {
     return " <old><$root $old/></old>\n <new><$root $new/></new>\n";
  } else {
     return undef;
  }
}

sub getXML { 
  my($agent, $o) = @_;
  my($e, $l, $x);
  my($s) = XMLout($o, (rootname => ''));
  my(@a) = split(/\n/, $s);
  for ($x=0; $x <= $#a; $x++) {
    $l = $a[$x];
    if (substr($l,0,4) eq "  </") {
       $a[$x] = "  </elem>";
    } elsif ($l =~ /^  <(\w+)[ >]/) {
       $e = $1;
       $a[$x] = "  <elem elem_type=\"$e\" " . substr($l,length($e)+4);
    }
  }
  return join("\n", @a);
}
      
  

sub found_pattern {
  my($list, $el) = @_;
  my($v);
  my(@a) = split(/,/, $list);
  foreach $v (@a) {
    if ($el =~ /$v/) {
print STDERR "     Ignoring $el in $list \n";
      return 1;
    }
  }
  return 0;
}


sub hash2xml {
  my($agent,$h) = @_;
  my($o);

  foreach my $k (sort keys %$h) {
     $o .= "<$k>$h->{$k}</$k>\n";
  }
  return $o;
}

#########################################################################
#    MEMORY FUNCTION: moved to PDM
#########################################################################

# Answer the question: Has it been X days since  a certain event occured:
# - If the question was never asked, the event is initialized and
#     the answer is 'NEW'.
# - If it's been that long, answer 'YES', reset the event.
#   else answer undef
#   Uses the cache system to store the memory for all events.
# Usage:
#   if (Util->isXdays('send_switch_counts')) {
#       'send_swith_stats in message...
#
sub isXdays {
 my($agent, $event_name, $days) = @_;
 my(@a,$year, $yday, $day);
 my($util) = $agent->{util};
 $days = 1 if (!$days);
 @a = localtime(time); $year = $a[5]; $yday = $a[7];
 $day = $year* 365 + $yday;
 my($events) =  $agent->load_cache('isXdays.cache');

 my($lastrun)  = $events->{$event_name};
 if (!$lastrun) {   # if new, save today to start
    $events->{$event_name} = $day;
    return "NEW";

 } elsif ( ($day - $lastrun) >= $days) {
   $events->{$event_name} = $day;
   return "YES";
 } else {
   return undef;
 }
}


#########################################################################
#    CONFIG FILE UTILITIES
#########################################################################

#
# find the right devices in Config for a certain Agent module
# modules are called a5k, a35k, t3, switch, switch.brocade etc..
# types from configfile are
#
sub get_config_devices {
  my($agent) = @_;
  my($d, @results, $type);
  my($ag) = lc(ref($agent));
  $ag = substr($ag, 0, -5) if (substr($ag, -5) eq "agent");

  foreach $d (@{$agent->{devices}} ) {
#     next if ($d->{active} eq "N"); # active is set by the GUI start/stop monitoring
#     and it should only impact the emailing of events, not the monitoring.

     push(@results, $d) if (lc($d->{type}) eq $ag);
  }
  return \@results;
}

sub category_selected {
  my($agent,$cat0) = @_;

  my($renv) = $agent->{renv};
  my($cat) = $cat0 || $agent->get_category;

  if (index(lc($renv->{categories}), lc($cat)) >= 0) {
     return 1;
  } else {
     return 0;
  }
}



#/usr/sbin/osa/lad
#c1t5d0 1T80703654 LUNS: 0 1 2


sub get_RM6Devices {
  my($q) = @_;
  my($type) = $q->{type};
  my(@out, $f, $l, $name, $hba, $in , $wwn, $x, $i, $path, $out, $out2, $err);
  my($details, $dev, $vendor, %A3);

  my($di) = "/usr/sbin/osa/lad";

  ($err,$out) = Util->run_command($di, "luxprobe.txt", 20);

  return "ERR $err" if ($err);
  return "ERR @$out" if ("@$out" =~ /No RAID devices/);

  for ($x=0; $x <= $#$out; $x++) {
    $l= $out->[$x];
    my(@a) = split(/\s+/, $l);
    $dev = $a[0]; $wwn = $a[1];

    my($di2) = "/usr/sbin/osa/raidutil -c $dev -i";
    ($err,$out2) = Util->run_command($di2, "luxprobe.txt", 20);
    if ("@$out2" !~ /$type/) {
       next;
    }

    my($err,$rdac) =
       Util->run_command("/usr/sbin/osa/rdacutil -i $dev", "luxprobe.txt", 20);
    my(@b) = split(/\:\s+/, $rdac->[1]);
    $A3{$b[0]}  .= "$dev|";
  }
  foreach my $k (keys %A3) {
     chop($A3{$k});
     $f .= "$k=$k=$A3{$k}\n";
  }
  if ($q->{print}) {
    print "OK $f";
  } else {
    return "OK $f";
  }
}






   

1;
