package BROCADEAgent;
#  $Name:  $ 
#  $Id: BROCADEAgent.pm,v 1.14 2001/09/21 20:50:34 ccadieux Exp $

use base 'Agent';

sub isSelectable {"BrocadeSwitch"}

sub revision {'$Revision: 1.14 $'}
sub new {
  my($self) = Agent->new();

  bless ($self, 'BROCADEAgent');
  return $self;
}

  
sub RUN {
  my($agent,$ras_flag) = @_;
  my(%DONE, $HBA, $logfile, $r, @results);
  my( @all_connect_errors);
  my($xml, @lux_dif, $dif, $xml, @all_dif);
  my($ev, @stats, $id, $log_lines, $sdt, $pdm);
  my($pdm) = $agent->{pdm};
  my($renv) = System->get_renv();

  if (!$agent->category_selected) {
     Debug->print2("Skipping " . ref($agent)  . ".");
     return;
  }
  $DB::single=1;
  Timelapse->start(ref($agent));

  my($today) = Util->today("YMDH");
  my($dc) = 0;

  foreach $device ( $agent->deviceList()) {

    $dc++;
    if (($sdt = Util->sdt($device)) ) {
        Debug->print('SDT', $sdt);
        next;
    }
    Debug->print1("-> Brocade: Reading device $device->{name}: $device->{ip}");

    if (!$device->{ip}) {
        Debug->err( 'IP_NOT_FOUND',"Switch at $device->{name}" );
        next;
    }

    my($connect_errs, $wwn, $report) = $agent->read_brocade($device);
    $wwn = lc($wwn);
    Catalog->deviceWrite( $wwn , ['brocade',$device->{name}, $wwn, undef, $device->{ip}, $device->{ipno}]);

    $id = {   accessName  => $device->{ip},
              deviceName  => $wwn,
              active      => $device->{active},
              display     => "$device->{name} (ip=$device->{ip})",
              name        => $device->{name},
              category    => Report::CAT_BROCADE,
              ip          => $device->{ip},
           };

    $device->{ipno} = Util->name2ip($device->{ip});

    $report->{"id.name"} = $device->{name};
    $report->{"id.ip"}   = $device->{ip};
    $report->{"id.ipno"} = $device->{ipno};

    if ($connect_errs || !$wwn) {
        $pdm->saveReport(
          Report->new($id, $report , [], Report::STATUS_CANNOT_CONNECT));

    } else {
        $pdm->saveReport(Report->new($id, $report , $log_lines));
    }
  }
  Debug->print2("  No devices found") if (!$dc);

  Timelapse->stop(ref($agent));
}




sub read_brocade {
  my($agent, $device) = @_;
  my(@s, %dev, $in, %info);
  my($connect_err, %C);
  my($port,  $config);
  my(%R, $wwn);
  my($ip)    = $device->{ip};
  my(@counts, @stats, $stat, $x);
  my($DIR) = System->get_home() . "/bin";
  my($TO) = $agent->{renv}{'timeout.snmp'};

  my($MIBS) = System->get_home() . "/snmp/mibs";

  $ENV{LD_LIBRARY_PATH} = System->get_home() . "/bin/lib";

  my($OPT) = "-r 3 -m all -Os -M $MIBS ";

  my($err0,$system) = Util->run_command("$DIR/snmpwalk $OPT $ip public system", 
                      "sanbox_version.txt", $TO);

  if ("@$system" =~ /Timeout:/) {
     return ("@$system", undef,undef);
  }

  my($err1,$interface) = Util->run_command("$DIR/snmpwalk $OPT $ip public interfaces", 
                      "sanbox_version.txt", $TO);

  if ("@$interface" =~ /Timeout:/) {
     return ("@$system", undef,undef);
  }

  my($err3,$swsystem) = Util->run_command("$DIR/snmpwalk $OPT $ip public fcSwitch.sw.swSystem",
                       "sanbox_version.txt", $TO);

  my($err4,$ports) = Util->run_command("$DIR/snmpwalk $OPT $ip public fcSwitch.sw.swFCport", 
                       "sanbox_version.txt", $TO);

  my($err5,$errors) = Util->run_command("$DIR/snmpwalk $OPT $ip public fibreChannel.fcFabric.fcFe.fcFeError", 
                       "sanbox_version.txt", $TO);

  if ($err0 || $err1 || $err3 || $err4) {
    Debug->err(SNMP => "Cannot run snmp on $ip:$err0$err1$err3$err4");
    return ("$err0$err1$err3$err4","", undef) 
  }
  foreach my $x (@$system) {
     my(@a) = split(/ +\= */, $x);
     $a[1] = substr($a[1],1,-1) if (substr($a[1],0,1) eq "\"");
     $a[0] = substr($a[0],0,-2) if (substr($a[0], -2) eq ".0");
     $a[1] =~ s/Counter\d\d\://;
     $R{"system.$a[0]"} =  $a[1];
  }
  foreach my $x (@$interface) {
     my(@a) = split(/ +\= */, $x);
     if ($x =~ /ifPhysAddress.1 =/) {
       $wwn = lc($a[1]);
       $wwn =~ s/hex: //;
       $wwn =~ s/ //g;
     } elsif ($x =~ /ifLastChange.1/) {
       last;
     } else {
       $a[0] =~ s/\.(\d+)$/_$1/;
       $a[1] =~ s/Counter\d\d\://;
       $R{"interfaces." . $a[0]} = $a[1];
     }
  }
  my($sensor, $port, $name);
  my($max) = 0;
  my(%SENS);
  foreach my $x (@$swsystem) {
     my(@a) = split(/ +\= */, $x);
     $a[1] = substr($a[1],1,-1) if (substr($a[1],0,1) eq "\"");
     $a[1] =~ s/Counter\d\d\://;
     if ($a[0] =~ /^swSensor/) {
        my($ix) = rindex($a[0], ".");
        $sensor = sprintf("%2.2d", substr($a[0],$ix+1));
        $max = $sensor if ($sensor > $max);
        $name = substr($a[0], 0, $ix);
        $name = substr($name, 8);
        if ($name ne "Index") {
          $SENS{$sensor}{$name} =  $a[1];
        }
       
     } else {
       $a[0] = substr($a[0],0,-2) if (substr($a[0], -2) eq ".0");
       $R{"swSystem.$a[0]"} =  $a[1];
     }
  }
  $wwn = $R{'swSystem.swSsn'};
  $wwn =~ s/://g;
  $wwn = lc($wwn);
  $R{"sensor.count"} = $max;
  foreach $x (keys %SENS) {
      my($key);
      if ($SENS{$x}{Info}) {
        $key = $SENS{$x}{Info};
        $key =~  s/\W+/_/g;
      } else {
        $key = $x;
      }
      my($p) = $SENS{$x};
      foreach my $y (keys %$p) {
          next if ($y eq "Info");
          $R{"sensor.$key.$y"} = $p->{$y};
      }
  }
        

  my($l)  = $ports->[0];
  my(@a) = split(/ +\= */, $l);
  my($size) = $a[1];
  $R{"port.count"} = $size;
  my($ERR_LIST) = ",RxTooLongs,LipOuts,RxBadEofs,RxBadOs,McastTimedOuts,LipIns,C3Discards,RxEncInFrs,RxCrcs,RxTruncs";

  for ($x=1; $x <= $#$ports; $x++) {
     my(@a) = split(/ +\= */, $ports->[$x]);
     $a[1] = substr($a[1],1,-1) if (substr($a[1],0,1) eq "\"");
     my($ix) = rindex($a[0], "-");
     $port = substr($a[0],$ix+1) + 1;
     $name = substr($a[0], 8);
     $ix = index($name, ".");
     $name = substr($name, 0, $ix);
     $a[1] =~ s/Counter\d\d\://;
     if (index(",Type,Wwn,Index,", ",$name,") >= 0) {
        $R{"port.$port.$name"} =  $a[1];

     } elsif (index(",AdmStatus,LinkState,PhyState,OpStatus,", ",$name,") >= 0) {
        $R{"port.$port.state.$name"} =  $a[1];

     } elsif (index($ERR_LIST, ",$name,") >= 0) {
        $R{"port.$port.error.$name"} =  $a[1];

     } else {
        $R{"port.$port.stats.$name"} =  $a[1];
     }
  }

  for ($x=1; $x <= $#$errors; $x++) {
     my(@a) = split(/ +\= */, $errors->[$x]);
     my $ix = rindex($a[0],".");
     $port = substr($a[0],$ix+1);
     $a[1] =~ s/Counter\d\d\://;
     my $a0 = substr($a[0],0,$ix);
     $a0 =~ s/\./-/g;
     $R{"port.$port.fcerror.$a0"} =  $a[1];
  }

  $agent->addIdentification(\%R);
  $R{"id.wwn"} = $wwn;

  return (undef, $wwn, \%R);
}

sub REPORT {
  my($class, $host, $r, $arg) = @_;

  my($out);
  my($v) = $r->{_value};

  print "<table border=1 cellspacing=0 cellpadding=1 width=95% bgcolor=white>
  <tr><td bgcolor=#666699 colspan=2>
   <table border=0><tr><td>
    <font color=white><b>Report for Brocade $r->{_id}{name} $r->{_id}{accessName}
   </table>
  ";
  my $last;
  foreach my $el (sort keys %$v) {
     my($a, $b) = split(/\./, $el);
     if ($a ne $last) {
        print "<tr bgcolor=#CCCCFF><td colspan=2>&nbsp;<b>$a" if ($a);
        $last = $a;
     }
        
     print "<tr><td>&nbsp;$el<td>&nbsp;$v->{$el}</td>";
  }
  print "</table>";


}


1;
