package MESSAGEAgent;
#  $Name:  $ 
#  $Id: MESSAGEAgent.pm,v 1.13 2001/08/03 14:03:56 ccadieux Exp $

use Agent;
use Debug;
use Catalog;
use Carp;
use vars qw($AUTOLOAD);
use Report;
use strict;
use PDM::Thresholds;
use base 'Agent';

sub isSelectable {0}
sub revision {'$Revision: 1.13 $'}

use vars qw(%DB);

sub new {
  my($self) = Agent->new();

  bless ($self, 'MESSAGEAgent');
  return $self;
}
sub DESTROY {}

sub RUN {
  my($agent,$ras_flag) = @_;

  my($report, $log_err, $lines, $x, $id);
  $DB::single=1;
  my($renv) = $agent->{renv}; 
  my($pdm) = $agent->{pdm};

  my($today) = Util->today("YMDH");

  my($lux) = System->find_luxadm;

  my($logfile)   = $agent->{renv}->{logfile};
  my($log_lines) ;

  Timelapse->start(ref($agent));
  %DB = ();
  if (-r $logfile) {
    ($log_err, $lines) = $agent->read_log_file($logfile,'NEW');
    Debug->err('CANNOT_READ', $logfile) if ($log_err);

    for ($x=0; $x <= $#$lines; $x++) {
       $log_lines .= $agent->read_thresholds(\$x, $lines);
    }
    $log_lines .= $agent->log_thresholds();
    if ($log_lines) {

       my($name)   = $renv->{hostname} || System->hostname();
       my($hostid) = System->hostid();
       $hostid     = Catalog->deviceWrite( $hostid, 
                         [ 'message', $name, undef, undef]);


       $id = {accessName  => $name,
              deviceName  => $hostid,
              active      => "Y",
              display     => "$name (id=$hostid)",
              logFile     => $logfile,
              name        => $name,
              category    => Report::CAT_MESSAGE,
              ip          => "",
           };
       $report->{"id.name"} = $name;
       $report->{"id.ipno"} = System->get_ipno();
       $pdm->saveReport(
          Report->new($id, $report , $log_lines, Report::STATUS_OK));
    }
  }
  Timelapse->stop(ref($agent));
}


############################################################################
#
# FIND THRESHOLD INFO FROM LOG FILE.
#
sub read_thresholds {
  my($agent,$x, $array) = @_;
  my(@a, $d1, $line, $next, $mysfwarn, $lookahead, $mywarn, $nextline);
  my($myoff, $found);

  $line = $array->[$$x];
  my($out);

  while (1) {
    if ($line =~ /SCSI transport failed: reason/ || $line =~ /ASC: 0x/ ) {
       $out .= $agent->writeEvent("W", "SCSI", $line);
       last;

    } elsif ($line =~ /The recommended firmware version for this application/) {
       $out .= $agent->writeEvent("W", "Firmware", $line);
       last;

    } elsif ($line =~ /PWWN=([0-9a-f]+) .+appeared /) {
       my $wwn = $1;
       $out .= $agent->writeEvent("W", "FABRIC", $line, $wwn);
       last;

    } elsif ($line =~ /Copyright .+ Sun Microsystems/) {
       $out .= $agent->writeEvent("W", "POWER-CYCLE", $line);
       last;

    } elsif ($line =~ /[Ff]ailover/) {
       my $wwn;
       if ($line =~ /GUID ([0-9a-f]+)/) {
          $wwn = uc(substr($1, -18));
       }

       $out .= $agent->writeEvent("W", "MPXIO", $line, $wwn);
       last;

    } elsif ($line =~ /mpxio: .*target address: ([0-9a-fA-F]+).*offline/) {
      
       $out .= $agent->writeEvent("W", "MPXIO", $line, $1);
       last;

    } elsif ($line =~ /failure prediction threshold exceeded/) {
       my(@m) = split(/ +/, $line);
       my($date) = "$m[0] $m[1] $m[2]";
       my($info) = "";
       foreach my $y  (5,4,3,2,1,0) {
          if ($$x - $y >= 0) {
            my($l) = $array->[$$x - $y];
            next if ($l =~ /Informational/ || $l =~ /Requested Block:/);
            my($t) = substr($l,16); $t =~ s/ +/ /g;
            $info .= " $t\n";
          }
       }
       $agent->log_inc('PFA', 0, $line);
       last;

    } elsif ($line =~ /NOTICE: .* OFFLINE/) {
       $agent->log_inc('QOFFLINE', 0, $line);
       last;

    } elsif ($line =~ /NOTICE: .* ONLINE/) {
       $agent->log_inc('QONLINE', 0, $line);
       last;
    }

    if (System->os_version ge "5.7") {
            # Loop Offlines
            if ($line =~ /(socal\d+:\s+port\s*\d+):.*OFFLINE$/) {  
                                                           # port\s+\d+
                $agent->log_inc('SFOFFL', $1, $line);
                $agent->log_inc('SFOFFLALERT', $1, $line);
                last;
            }

            if ($line =~ /Error for Command: read\(\d+\)\s+Error Level: Retryable/) {
                $agent->log_inc('SCSI_TR_READ', $1, $line);
                last;
            }

            # FC transient write
            if ($line =~ /Error for Command: write\(\d+\)\s+Error Level: Retryable/) {
                $agent->log_inc('SCSI_TR_WRITE', $1, $line);
                last;
            }

            # SSD Warnings
            if ($line =~ /WARNING.*\((ssd\d+)\)/) {
                $agent->log_inc('WARNSSD', $1, $line);
                $agent->log_inc('ALERTSSD', $1, $line);
                last;
            }
            # Loop CRC, DMA and Timeout Warnings
            if ($line =~ /WARNING.*\((sf\d+)\)/) {
               $mysfwarn = $1;
               $$x++;
               $next = $array->[$$x];
               if ($next =~ /CRC/) {
                  $agent->log_inc('SFCRCWARN', $mysfwarn, $line);
                  $agent->log_inc('SFCRCALERT', $mysfwarn, $line);
                  last;
               }
               if ($next =~ /Offline Timeout/) {
                  $agent->log_inc('SFOFFTOWARN', $mysfwarn, $line);
                  last;
               }
               if ($next =~ /INCOMPLETE DMA XFER/) {
                  $agent->log_inc('SFDMAWARN', $mysfwarn, $line);
                  last;
  	           }
               # SF Resets
               if ($next =~ /(sf\d+):.*sf_reset/) {
                  $agent->log_inc('SFRESET', $mysfwarn, $line);
               }
               # Target and Loop ELS Retry
               if ($next =~ /(sf\d+):\s+ELS\s+.*(target\s+0x[0-9a-f]+)\s+retrying/) {
                  $agent->log_inc('RETRYELS', "$mysfwarn $2", $line);
                  $agent->log_inc('RETRYSF', $mysfwarn, $line);
                  last;
               }
  
               # Target and Loop ELS Time Outs 
               if ($next =~ /(sf\d+):\s+ELS\s+.*(target\s+0x[0-9a-f]+)\s+timed out/) {
                  $agent->log_inc('TOELS', "$mysfwarn $2", $line);
                  $agent->log_inc('SFTOELS', $2, $line);
                  last; 
               }
            } 
  
            # Target Offlines
            if ($line =~ /(sf\d+):\s+(target 0x[0-9a-f]+)\s+offline/) {
                $agent->log_inc('DDOFFL', "$1 $2", $line);
                last;
            }
     } else {
            if ($line =~ /(socal\d+:\s+port\s+\d+):.*OFFLINE$/) {
                $agent->log_inc('SFOFFL', $1, $line);
                $agent->log_inc('SFOFFLALERT', $1, $line);
                last;
            }
            # SSD Warnings
            if ($line =~ /WARNING.*\((ssd\d+)\)/) {
                $agent->log_inc('WARNSSD', $1, $line);
                $agent->log_inc('ALERTSSD', $1, $line);
                last;
            }
            # Loop CRC, DMA and Timeout Warnings
            if ($line =~ /WARNING.*\((sf\d+)\)/) {
                $mysfwarn = $1;
                $$x++;
                $next = $array->[$$x];
                if ($next =~ /CRC/) {
                    $agent->log_inc('SFCRCWARN', $mysfwarn, $line);
                    $agent->log_inc('SFCRCALERT', $mysfwarn, $line);
                    last;
                }
                if ($next =~ /Offline Timeout/) {
                    $agent->log_inc('SFOFFTOWARN', $mysfwarn, $line);
                    last;
                }
                if ($next =~ /INCOMPLETE DMA XFER/) {
                    $agent->log_inc('SFDMAWARN', $mysfwarn, $line);
                    last;
  	            }
            }
            # SF Resets
            if ($line =~ /(sf\d+):.*sf_reset/) {
                $agent->log_inc('SFRESET', $1, $line);
            }
            # Target and Loop ELS Retry
            if ($line =~ /(sf\d+):\s+ELS\s+.*(target\s+0x[0-9a-f]+)\s+retrying/) {
                $agent->log_inc('RETRYELS', "$1 $2", $line);
                $agent->log_inc('RETRYSF', $1, $line);
                last;
            }
            # Target and Loop ELS Time Outs 
            if ($line =~ /(sf\d+):\s+ELS\s+.*(target\s+0x[0-9a-f]+)\s+timed out/) {
                $agent->log_inc('TOELS', "$1 $2", $line);
                $agent->log_inc('SFTOELS', $2, $line);
                last;
            }
            # Target Offlines
            if ($line =~ /(sf\d+):\s+(target 0x[0-9a-f]+)\s+offline/) {
                $agent->log_inc('DDOFFL', "$1 $2", $line);
                last;
            }
    }
    if (1) {
        $lookahead = $array->[$$x + 1];
        if ($line =~ /WARNING/ || $line =~ /LIP reset occured/ || 
                  $lookahead =~ /LIP reset occured/ ) {
             if ( $line =~ /\((ssd\d+)\)/ ) {
                ($mywarn) = $1;
                $agent->log_inc('WARNSSD', $1, $line);
                $agent->log_inc('ALERTSSD', $1, $line);
             } elsif ( $line =~ /ifp/ ) {
                 $$x++;
                 $nextline = $array->[$$x];
                 if ( $line =~ /LIP reset occured/ ) {
                     ($myoff) = $line =~ /(ifp\d+)/;
                     $agent->log_inc('SFOFFLALERT', $myoff, $line);
                     $$x--;
                 } elsif ($nextline =~ /LIP reset occured/) {       
                     ($myoff) = $line =~ /(ifp\d+)/;
                     $agent->log_inc('SFOFFLALERT', $myoff, $line);
                 } elsif ($nextline =~ /CRC/) {       
                     # count it against the ifp
                     $agent->log_inc('SFCRCWARN', $mysfwarn, $line);
                     $agent->log_inc('SFCRCALERT', $mysfwarn, $line);
                     my ($tmpt) = $line =~ /target=(0x[0-9a-f]+):/;
                     my $mytarg = "${mysfwarn} target ${tmpt}";

                     # count it against the ifp targ
                     $agent->log_inc('DDCRCWARN', $mytarg, $line);
                 } elsif ($nextline =~ /DMA Failure/) {
                     $agent->log_inc('SFDMAWARN', $mysfwarn, $line);
                 } else {
                     $$x--;
                 }
  				
             } elsif ($line =~ /offline/) {
                 my($mysf, $mytarg, $myidx);
                 ($mysf) = $line =~ /(ifp\d+)/;
                 ($mytarg) = $line =~ /(target 0x[0-9a-f]+)/;
                 ($myidx) = "$mysf $mytarg";
                 $agent->log_inc('DDOFFL', $myidx, $line);
             }
        }
    }
    last;
  }
  return $out;
}

#
#  $out .= $agent->writeEvent("E", "Error xyz", $line);
#
sub writeEvent {
  my($agent, $level, $category,  $mline, $wwn) = @_;

  my(@m) = split(/ +/, $mline, 6);
  my($date) = "$m[0] $m[1] $m[2]";
  my($info) = "$m[3] $m[4] $m[5] $m[6]";
  $info =~ s/"/ /g;
  my($enc_type, $enc, $details) = Util->wwnInfo(substr($wwn,2)) if ($wwn);
  my($i) = "($details in $enc_type:$enc)" if ($enc);

  if ($level eq "W") {
     return "Warning\t$date\t$category\t$info $i\n";
  } elsif ($level eq "E") {
     return "Error\t$date\t$category\t$info $i\n";
  }
}

%DB = ();

sub log_inc {
  my($agent, $type, $idx, $mline) = @_;
  my($key) = "fibre|$type|$idx";
  my($wwn);
  my(@m) = split(/ +/, $mline, 6);

  my($p) =  $DB{$key};
  my($date) = "$m[0] $m[1] $m[2]";
  my($info) = "$m[3] $m[4] $m[5] $m[6]";

  if ($idx && $mline =~ /\@w([0-9a-fA-F]+)/) {
     $wwn = $1;
  }

  if (defined($p)) {
    $p->[0] = $date;
    $p->[1] = $info;
    $p->[2]++;
    $p->[3] = $wwn;
  } else {
    $DB{$key} = [$date, $info, 1, $wwn]; 
  }

}

#ERR     Mar 6 10:38:25  socal5: port 1: Received 21 fibre.SFOFFLALERT messages (threshold=15)    diag239.Central.Sun.COM socal: [ID 403145 kern.info] ID[SUNWssa.socal.link.5010] socal5: port 1: Fibre Channel is OFFLINE 

sub log_thresholds {
  my($agent) = @_;
  my(%MAP) = (W => 'Threshold-Warning', E => 'Threshold-Error');
  my($pdm) = $agent->{pdm};
  my($enc, $details, $enc_type);

  my($out);
  foreach my $p (keys %DB) {
     my($cat, $type, $idx) = split(/\|/, $p);

     my($val)  = $DB{$p};
     my($cnt)  = $val->[2];
     my($date) = $val->[0];
     my($info) = $val->[1];
     my($wwn)  = $val->[3];
     if (length($wwn) > 8 ) {
        ($enc_type, $enc, $details) = Util->wwnInfo(substr($wwn,2));
        $details =~ s/\.wwn$//;
     }
     my($i) = "($details in $enc_type:$enc)" if ($enc);

     my($level, $cnt, $desc, $mins, $thr, $thr_time) = 
              $pdm->threshold()->test($cat, $type, $idx,$cnt);
     next if (!$level);
     
     my($tx) = "Received $cnt '$desc' message(s)";
     $tx .= " on '$idx'" if ($idx);
     $tx .= " $i";
     $tx .= " in $mins mins" if ($mins);
     $thr_time .= ($thr_time =~ /h$/) ? "ours":"ins";
     $tx .= " [threshold is $thr in $thr_time]";
     $info =~ s/"/ /g;
     $out .= $MAP{$level} . "\t$date\t$tx\tLast-Message: '$info'\n";
  }
  return $out;

}


sub log_incNU {
  my($agent, $type, $idx, $mline) = @_;
  my($data, $thr, $th, $val, $t1, $today, $hrs);

  my($pdm) = $agent->{pdm};
  my(@m) = split(/ +/, $mline, 6);
  my($info, $date );
  $info =~ s/"/ /g;
  my($level, $cnt, $desc, $mins) = $pdm->threshold()->test('fibre',$type, $idx,1);

  my($tx) = "$idx: Received $cnt $desc messages";
  $tx .= " in $mins mins" if ($mins);
  if ($level eq "W") {
     return "WARN\t$date\t$tx\t$info\n";
  } elsif ($level eq "E") {
     return "ERR\t$date\t$tx\t$info\n";
  }
}



1;

