package BK2;  # Berkeley 2 keys

use strict;
use BerkeleyDB ;
use Carp;

use vars qw($Error);

#  Support 2 indexex.
#  Readonly users look for the lock file but do not create one.
#  Writers create the lockfile.
#  Both users wait for the lockfile before creating the DB, delete the lockfile if too old.

#  $bk = BK2->new({ file => "data", len => [10,20], oldlock => 5
#                  clear => 1, readonly => 1, create => 1, debug => 1});
#  readonly and create and exclusive
#  
#  $bk->put($idx1,$idx2,$val) || print "Error: $BK2::Error";
#  $bk->del([1/2], $kvalue) || print "Err: $BK2::Error";
#  $val = $bk->get([1/2], $key);

#  CURSOR BASED
#  $csr = $bk->cursor([1/2]);
#  $el = $csr->getGE($key);    $el->[0] = newkey1, $el->[1] = newkey2, $el->[2] = value
#  $el = $csr->getGT($key);

#  $el = $csr->getLE($key) 
#  $el = $csr->getLT($key);

#  $el = $csr->next();
#  $el = $csr->prev();

#  $csr = $bk->cursor(1);
#  $csr->getGE("key1", "key2");
#  for ($el = $csr->getGE("k1","k2")) ; $el && $el->[0] < 'XXX'; $el = $csr->next() ) {
#     use $el->[2];

$Error ;

sub new {
  my($class, $arg)  = @_;
  my($o);
  $o->{file} = $arg->{file};
  $o->{len1} = $arg->{len}[0];
  $o->{len2} = $arg->{len}[1];
  my($flags);
  $flags |= DB_TRUNCATE if ($arg->{clear}    );
  $flags |= DB_RDONLY   if ($arg->{readonly} );
  $flags |= DB_CREATE   if ($arg->{create} && !$arg->{readonly} );

  $o->{readonly} = $arg->{readonly};
  $o->{debug}   = $arg->{debug};
  $arg->{clear} = 1 if (!-f "$o->{file}.idx1");
  $o->{oldlock} = $arg->{oldlock} || 6;

  my($db1) =  BerkeleyDB::Btree->new(
                -Filename => "$o->{file}.idx1",
                -Flags    => $flags,    # | DB_INIT_LOCK | DB_RDONLY
               );

  my($db2) =  BerkeleyDB::Btree->new(
                -Filename => "$o->{file}.idx2",
                -Flags    => $flags,    # | DB_INIT_LOCK | DB_RDONLY
               );

  my($db3) =  BerkeleyDB::Recno->new(
                -Filename => "$o->{file}.idx3",
                -Flags    => $flags,    # | DB_INIT_LOCK | DB_RDONLY
               );

  if (!$db1 || !$db2 || !$db3) {
     $Error = "Cannot open DB file $o->{file}: $! $BerkeleyDB::Error\n" ;
     return undef;
  }

  my($tries);
  my($lf) =  "$o->{file}.lock";
  if (-f $lf) {
     my($age) = (stat($lf))[9];
     if (time - $age > $o->{oldlock}) {
        print "Removing old lock file\n";
        unlink($lf);   # remove if old, a writer probably crashed
     }
  }
  while (-f $lf && ++$tries < 4) {
     sleep(1);
  }
  if ($tries == 4) {
     print "waited 3 seconds on $lf, quitting";
     return undef;
  }
  if (!$arg->{readonly}) {  # create lock file for writing
     open(O,">$lf"); close(O);
  }

  $o->{db}[1] = $db1;
  $o->{db}[2] = $db2;
  $o->{recdb} = $db3;
  if ($arg->{clear}) {
    $db3->db_put(0,"1");
    $o->{recno} = 1;
  } else {
    my($r);
    $db3->db_get(0,$r);
    $o->{recno} = $r;
  }
  bless($o , "BK2");
  return $o;
}

sub DESTROY {
  my($o) = shift;
  $o->close();
  print "destroying\n";
}
  

sub close {
  my($o) = @_;
  my($status);
  my($db1) = $o->{db}[1];
  my($db2) = $o->{db}[2];
  my($rec) = $o->{recdb};

  $rec->db_put(0,$o->{recno});
  $rec->db_sync();
  $db1->db_sync();
  $db2->db_sync();

  $rec->db_close();
  $db1->db_close();
  $db2->db_close();
  unlink "$o->{file}.lock" if (!$o->{readonly}); # remove if you created it

}

sub cursor {
  my($bk, $key) = @_;

  return BK2C->new($bk, $key);
}


sub truncate {
  my($class, $file) = @_;
  my($tries) = 0;

  unlink "$file.idx1";
  unlink "$file.idx2";
  unlink "$file.idx3";
  unlink "$file.lock";
  return 1;
}


#  $bk->buildKey
sub buildKey {
  my($o, $kno, $k1, $k2) = @_;

  my($a)  = $o->{len1};
  my($b)  = $o->{len2};

  if ($kno == 1) {
     my($s) = "%-$a.${a}s%-$b.${b}s";
     return sprintf($s, $k1, $k2);
  } else {
     my($s) = "%-$b.${b}s%-$a.${a}s";
     return sprintf($s, $k2, $k1);
  }
}

#  undef = failed to delete
sub del {
  my($o , @p) = @_;
  my($db, $recno, $value, $status, $kvalue);

  croak "->del needs 3 args" if ($#p < 2);
  my($kno, $kv1, $kv2) = @p;

  $db = $o->{db}[$kno];
  $kvalue = $o->buildKey($kno, $kv1, $kv2);

  if ($db->db_get($kvalue, $recno) == 0) {
    my($rec) = $o->{recdb};
    $status = $rec->db_get($recno , $value);
    if ($status) {
      $o->debug($status);
      return undef;
    }
    $status = $rec->db_del($recno);
    if ($status) {
      $o->debug($status);
      return undef ;
    }

    my(@a) = split(/\002/, $value);
    my($db) = $o->{db}[1];
    $status = $db->db_del($a[0]);
    if ($status) {
      $o->debug($status);
      return undef;
    }

    my($db) = $o->{db}[2];
    $status = $db->db_del($a[1]);
    if ($status) {
      $o->debug($status);
      return undef;
    }
    return 1;
  }
  return undef;
}

sub debug {
  my($o, $status) = @_;
  $Error = $status;
  print "$status\n" if ($o->{debug});
}

#  $bk->put() || print $BK2::Error;
sub put {
  my($o, @p) = @_;
  my($status, $cnt, $recno);
  croak "->put needs 3 args" if ($#p < 2);
  my($idx1, $idx2, $val) = @p;

  my($db1) = $o->{db}[1];
  my($db2) = $o->{db}[2];
  my($db3) = $o->{recdb};
  my($key1) = $o->buildKey(1, $idx1, $idx2);
  my($key2) = $o->buildKey(2, $idx1, $idx2);

  if ($db1->db_get($key1, $recno) == 0) {
     $cnt = $recno;
  } else {
     $cnt = $o->{recno};
     $o->{recno}++;
  }

#  my($k) = 0; 
#  $status = $db3->db_get($k, $cnt);
#  my($new) = $cnt++;
#  $db3->db_put(0,$new);
#  if ($status) {
#    $o->debug($status);
#    return undef;
#  }

#  my($flag) = DB_NOOVERWRITE if ($noover);

  if (($status = $db1->db_put($key1, $cnt)) || 
      ($status = $db2->db_put($key2, $cnt)) ||  
      ($status = $db3->db_put($cnt, $val)) ) {
      $o->debug($status);
      return undef;
  }
  return 1;
}

# return value or undef
sub get {
  my($o, @p) = @_;
  my($recno, $value, $db, $status);
  croak "->get needs 3 args" if ($#p < 2);
  my($kno, $k1, $k2) = @p;

  my($kvalue) = $o->buildKey($kno, $k1, $k2);
  $db = $o->{db}[$kno];
  if (($status = $db->db_get($kvalue, $recno)) == 0) {
    my($rec) = $o->{recdb};
    $status = $rec->db_get($recno , $value);
    if ($status) {
      $o->debug($status);
      return undef;
    }
    return $value;
  } else {
    $o->debug($status);
    return undef;
  }
}

package BK2C;
use strict;
use BerkeleyDB ;
use Carp;

sub new {
  my($class, $bk, $key) = @_;
  croak "Must specify key# in Cursor->new()" if (!$key);
  $BK2::Error = undef;
  my($csr) = {key => $key, db => $bk, cursor => 0};
  bless($csr, "BK2C");
  return $csr;
}
sub close {
  my($o) = @_;
  my($cursor) = $o->{cursor};
  $cursor->c_close() if ($cursor);
}

sub getGE {
  my($class, $k1, $k2) = @_;
  croak "getGE need 3 args" if (!defined($k2));
  return $class->_getG( $k1, $k2, ">=");
}

sub getFIRST {
  my($class) = @_;
  return $class->_getG( "","", "F");
}

sub getLAST {
  my($class) = @_;
  return $class->_getG( "","", "L");
}


sub getGT {
  my($class, $k1, $k2) = @_;
  croak "getGT need 3 args" if (!defined($k2));
  return $class->_getG( $k1, $k2, ">");
}

sub getLE {
  my($class, $k1, $k2) = @_;
  croak "getLE need 3 args" if (!defined($k2));
  return $class->_getG( $k1, $k2, "<=");
}

sub getLT {
  my($class, $k1, $k2) = @_;
  croak "getLT need 3 args" if (!defined($k2));
  return $class->_getG($k1, $k2, "<");
}


# return $el or undef
sub _getG {
  my($o, $k1, $k2, $mode) = @_;
  my($db,$cursor, $status, $recno, $value , $start, $kvalue, $kno, $bk);
  $bk    = $o->{db};
  $kno   = $o->{key};
  $start = $kvalue = $bk->buildKey($kno, $k1, $k2);

  $db = $bk->{db}[$kno];
  $cursor = $db->db_cursor() ;
  if ($mode eq "F") {
    $status = $cursor->c_get($kvalue, $recno, DB_FIRST);
  } elsif ($mode eq "L") {
    $status = $cursor->c_get($kvalue, $recno, DB_LAST);
  } else {
    $status = $cursor->c_get($kvalue, $recno, DB_SET_RANGE);
  }

  if ($status) {
     $o->{db}->debug($status);
     return undef;
  }
  my($flag) = 0;
  if ($mode eq ">" && $start eq $kvalue) {  # next one if GT
     $status = $cursor->c_get($kvalue, $recno, DB_NEXT);
     $flag =1;
  } elsif ($mode eq "<=" && $kvalue gt $start) {
     $status = $cursor->c_get($kvalue, $recno, DB_PREV);
     $flag =1;
  } elsif ($mode eq "<" && $kvalue ge $start) {
     $status = $cursor->c_get($kvalue, $recno, DB_PREV);
     $flag =1;
  }
  if ($flag && $status) {
     $o->{db}->debug($status);
     return undef;
  }
#  $o->{current} = $kvalue;
  $o->{cursor}  = $cursor;
  my($rec)      = $bk->{recdb};
  $status       = $rec->db_get($recno , $value);
  if ($status) {
    $o->{db}->debug($status);
    return undef;
  }
  my($k1, $k2) = $o->breakKey($kno, $kvalue);
  my(@a) = ($k1, $k2, $value);
  return \@a;
}
  
sub next {
  my($class) = @_;
  return $class->nextprev(DB_NEXT);
}

sub prev {
  my($class) = @_;
  return $class->nextprev(DB_PREV);
}

# return $el or undef
sub nextprev {
  my($o, $mode) = @_;
  my($db, $cursor, $recno, $value, $k0, $kno);
  my($k0) ; # = $o->{current};
  $kno = $o->{key};
  my($bk) = $o->{db};

#  $DB::single = 1;
#  if (!defined($k0) ) {
#    $o->debug("Current not defined");
#    $o->{cursor} = undef;
#    return undef;
# }

  $db = $bk->{db}[$kno];
  if (!$o->{cursor}) {
     croak "No cursor defined! ($k0)";
  } else {
     $cursor = $o->{cursor};
  }    
  my $status = $cursor->c_get($k0, $recno, $mode);

  if ($status) {
     $o->{cursor} = undef;
     if (substr($status,0,11) eq "DB_NOTFOUND") {
       return 0;
     }
     $o->{db}->debug($status);
     return undef;
  }
  my($rec) = $bk->{recdb};
  $status = $rec->db_get($recno , $value);
  if ($status) {
    $o->{cursor} = undef;
    $o->{db}->debug($status);
    return undef;
  }
#  $o->{current} = $k0;
  my($k1, $k2) = $o->breakKey($kno, $k0);
  my(@a) = ($k1, $k2, $value);
  return \@a;
}

sub breakKey {
  my($csr, $kno, $v) = @_;
  my($k1, $k2);
  my($o) = $csr->{db};
  my($a)  = $o->{len1};
  my($b)  = $o->{len2};
  if ($kno == 1) {
    $k1 = &rtrim(substr($v,0,$a));
    $k2 = &rtrim(substr($v,$a));
    return ($k1, $k2);
  } else {
    $k2 = &rtrim(substr($v,0,$b));
    $k1 = &rtrim(substr($v,$b));
    return ($k1, $k2);
  }
}

sub rtrim {
  my($s) = @_;
  $s =~ /(\s*)$/;
  if ($1) {
    substr($s,0,0 - length($1));
  } else {
    $s;
  }
}


1;
