package CIM::GenInstance;
#  $Name:  $ 
#  $Id: GenInstance.pm,v 1.3 2001/09/28 16:00:23 ccadieux Exp $

use CIM;
use CIM::Property;
use System;
use Carp;
use base 'CIM::Base';

#
#  Type: E: Event, C:Component, P: Product
#   $ass =  CIM::GenInstance->new('ClassName','A', [
#                           [ ref1     => <some objectName> , 'R'  ],
#                           [ ref2     => <some ObjectName> , 'R'  ],
#                           [ attrib1  => "value", 'type' ],
#                   ]);

#   $ass =  CIM::GenInstance->new('ClassName','C', [
#                           [ KEY_field1 => "value", 'type'],   # a key
#                           [ attrib1  => "value", 'type' ],
#                           [ attrib1  => "value"],             # a string
#                   ]);
#
#    $ins = CIM::GenInstance(<classname>, <type>, $property_list);

sub new {
  my($class, $className, $ins_type, $arg, $quals) = @_;
  my(@props, $self);
  my($cnt) = 0;

  if (ref($arg->[0]) eq "CIM::Property") {
     $self = [ $ins_type, $className, undef, $arg];
     bless($self, "CIM::Instance");
     return $self;
  }

  foreach $p (@$arg) {
     $name = $p->[0];
     $value = $p->[1];
     $type  = $p->[2] || 'string';
     $key = 0;
     if (substr($name,0,4) eq "KEY_") {
        $name = substr($name,4);
        $key = 1;
     } 
     $value = $value->key if (ref($value) eq "CIM::Instance");
     if (ref($value) eq "CIM::Key") {  #  $type eq "R") {
        if ($ins_type ne "A") {
           croak("Invalid: References can only be in associations: $className");
        }   
        $cnt++;
        push(@props, CIM::Property->new( [$name, $value, "reference", 1]) );
     } else {
        $type = $CIM::MAP{$type} || $type;
        if ($type eq "boolean") {
             $value = ($value eq "TRUE" || $value == 1)? "TRUE":"FALSE";
        }
        if (CIM->check_type($type)) {
           push(@props,    CIM::Property->new( [$name, $value, $type, $key] ) );
        } else {
           croak("Invalid Type: $type for attribute $p0");
        }
     }
  }

  carp("Associations need at least two references!") if ($cnt < 2 && $ins_type eq "A");
  
  $self = [ $ins_type, $className, undef, \@props ];

  bless($self, "CIM::Instance");
  return $self;

}




1;


# 1;
#


__END__

=head1 NAME

GenInstance.pm - Generate a Generic Instance with no validation against a schema.
Inherits from Instance.pm


=head3 SYNOPSIS

 use GenInstance;

 $inst =  CIM::GenInstance->new('ClassName','C', [
                       [ KEY_field1 => "value", 'type'],   # a key
                       [ attrib1  => "value", 'type' ],
                       [ attrib1  => "value"],             # a string
               ]);



=head3 DESCRIPTION

This module creates CIM instance object based on a data structured that is 
passed at creation.



=head3 CONSTRUCTOR

=over 4

=item new( <ClassName> <Type> <Properties> )

 Type: C=Component, E=Event, A=Association, P=Product.

=back 4

=head3 SEE ALSO

 CIM::Instance.


=head3 AUTHOR

 Christian Cadieux (ccadieux@central.sun)



=head3 COPYRIGHT

Copyright (c) 2000 Sun Microsystems

=cut
