package FCRules::Rule1;
use TO;
use Data::Dumper;
use FCRules;
use strict;

# WRITE
sub description {
  return "iPort error in switch";
}


sub category {
  return "CRC";
}

sub run {
  my($class, $To, $hm, $report, $th, $count, $key, $info, $reads, $writes,
   $fc_type ) = @_;

# |wwn|port.1|switch
  my($hba, $name, $port0, $type) = split(/\|/, $key);
  return(undef,undef) if ($type ne "switch");

  my($port) = substr($port0,5) + 0;

  my($sw) = $To->nodeByName("switch:$name"); 
  if (!$sw) {
    Debug->err(TEXT => "FCRule1: Cannot find switch $name in Topology");
    return ();
  }
  
  my($sw_info) = $sw->{info};

  my $port_info  = $sw->{portInfo};
  my $port_type;
  my $rn = $port_info->[$port]{RegisterName};

# qlc4-sw2-f6
  if ($rn  =~ /-ip\d+\_/ || $rn =~ /sw\d+-i\d/ || 
            $port_info->[$port]{sw_PortType} eq "F_Port") {
     $port_type = "i";
  } else {
     return (undef, undef);
  }
  
  my $links  = $sw->{port};
  my $link  = $links->[$port];

  my($ltype, $host_name, $pno) = split(/:/, $link);

  if ($ltype ne "host") { # not pointing to a host, error
     return ("Error: iport is not pointing to a host", undef);
  }
  my($level, $cnt, $desc, $mins) = $th->test("${fc_type}counters","rule1","$name.$port" , $count);
  $mins = 1 if (!$mins);

  if ( $level eq "E") { # error level threshold fired
    Debug->print2("Rule 1 fired");

    my $host = $To->nodeByName("host:$host_name");
    if (!$host) {
       return("Cannot find host $host_name");
    }

    my $hbas = $host->{portInfo};
  
    my $hba = $hbas->[$pno];
 
    my $short = $host->{info}{name};
    my $ix2 = index($short,".");
    $short = substr($short,0,$ix2) if ($ix2 > 0);
    my($desc) = "$fc_type-ERROR ($cnt in $mins mins): Origin: Hba $pno on '$short. Destination: port $port on " . $sw->id()  . ":";

    my $pc  = Util->getInfo("Probable_Causes", "R1");

    my $ra  = Util->getInfo("Recommended_Actions","R1") . 
" Register-Origin : $hba->{RegisterName}
 Register-Destin.: $rn";

    my($CIMKey1, $CIMKey2, $key1, $key2);

    $CIMKey1 = $host->{info}{hostid} . ".hba.$pno";
    $key1    = $host->name() . ":$pno";

    $CIMKey2 = $sw->key() . ".port.$port";
    $key2    = $sw->name() . ":$port";

    $hm->linkEvent( $report, "host", $CIMKey1, $key1, "switch", $CIMKey2 , $key2,
                      $desc, $reads, $writes, $hba, $pc,$ra);

  }

  return (undef, "");
}


1;


