package FCRules::Rule2;
use TO;
use Data::Dumper;
use strict;

# WRITE
sub description {
  return "E_Port error in switch";
}


sub category {
  return "CRC";

}

sub run {
  my($class, $To, $hm, $report, $th, $count, $key, $info, $reads, $writes,
     $fc_type) = @_;

  my($hba, $name, $port0, $type) = split(/\|/, $key);

  return(undef,undef) if ($type ne "switch"); # Switch

  my $port = substr($port0,5) + 0;
 
  my $sw = $To->nodeByName("switch:$name");
  if (!$sw) {
    Debug->err(TEXT => "FCRule2: Cannot find switch $name in Topology");
    return ();
  }
  
  my $sw_info = $sw->{info};

  my $port_info = $sw->{portInfo};
  
   my $port_type; #  = ($port_info->[$port]{RegisterName} =~ /-sp\d+\_/ ) ? "s" : "X";

  if ($port_info->[$port]{sw_PortType} ne 'E_Port') {
      return; # ISL port is 'sp',
  }

  my $links = $sw->{port};
  my $link  = $links->[$port];

  my($ltype, $sw_name2, $port2) = split(/:/, $link);

  if ($ltype ne "switch") { # not a switch to switch 
     return ;
  }

  my($level, $cnt, $desc, $mins) = $th->test("${fc_type}counters","rule2","$name.$port" , $count);
  $mins = 1 if (!$mins);

  if ( $level eq "E") {
    Debug->print2("Rule 2 fired");

    my $sw2    = $To->nodeByName("switch:$sw_name2");
    if (!$sw2) {
       return ("Cannot find switch $sw_name2");
    }
    my $info2  = $sw2->{info};
    my $pinfo2 = $sw2->{portInfo};
    
    my($desc) = "$fc_type-ERROR ($cnt in $mins mins): Origin: Port $port2 on " . $sw2->id() . 
    ". Destination: Port $port on " . $sw->id() . ":";
    my $pc  = Util->getInfo("Probable_Causes", "R2");
    my $ra  = Util->getInfo("Recommended_Actions","R2") . 
" Register-Origin : $pinfo2->[$port2]{RegisterName}
 Register-Destin.: $port_info->[$port]{RegisterName}";

    my($CIMKey1, $CIMKey2, $key1, $key2) ;

    $CIMKey1 = $sw2->key() . ".port.$port2";
    $key1    = $sw2->name() . ":$port2";

    $CIMKey2 = $sw->key()  . ".port.$port";
    $key2    = $sw->name()  . ":$port";

    $hm->linkEvent( $report, "switch", $CIMKey1, $key1, "switch", $CIMKey2 , $key2,
                      $desc, $reads, $writes, undef, $pc,$ra);

  }

  return (undef, "");

}


1;


