package FCRules::Rule3;
use TO;
use Data::Dumper;
use Catalog;
use strict;

# WRITE
sub description {
  return "Single target device error (tape or T3 or A3500FC device)";
}

sub category {
  return "CRC";
}
# key= ' |100000c0dd00885d|port.002|switch'
# key= '0|100000000c0d0|port.1|t3'
# key= '0|10000000000de0|front.0|a5k'

sub run {
  my($class, $To, $hm, $report, $th, $count, $key, $info, $reads, $writes,
     $fc_type) = @_;

  my($CIMKey1, $CIMKey2, $key1, $key2, $ctrl_port, $port_desc);
  my($hba, $wwn, $port0, $type) = split(/\|/, $key);

  return(undef,undef) if ($type eq "switch" || $type eq "host");
  my $st;

  if ($type eq "t3") {
     substr($wwn,7,1) = "3" if (length($wwn) > 8);
     my($ci, $port) = Catalog->deviceRead('t3',Catalog::WWN, $wwn);
     $port--;
     return if (!$ci);
     $wwn = $ci->[Catalog::Key];
     $port0 = "port.$port";
  }
  $st = $To->nodeByName("$type:$wwn");

  if (!$st) {
      return (undef,undef);
  }
  my($portI) = $st->portInfo();

  my($head, $port) = split(/\./, $port0);

  ($CIMKey2, $key2, $ctrl_port, $port_desc) = 
                     FCRules->findPortKeys("$hba|$wwn|$port|$type", $info, $st);
    

  my $st_info = $st->{info};

  my $links = $st->port();      # ports of array
  my $link  = $links->[$port];  # could be 0 or 1

  my($ltype, $t_key, $pno) = split(/:/, $link);
  my($host, $hbas, $sw, $sw_info, $ip);

  if ($ltype eq "host" || $ltype eq "switch") {   # connected to a host, dport
      my($level, $cnt, $desc, $mins) =
            $th->test("${fc_type}counters","rule3","$wwn-$port" , $count);

      $mins = 1 if (!$mins);

      if ( $level eq "E") { # error level threshold fired
         Debug->print2("Rule 3 fired");

         if ($ltype eq "host") {        # host
            $host = $To->nodeByName("host:$t_key");
            $hbas = $host->{portInfo};
            $hba  = $hbas->[$pno];

            my($desc) = "$fc_type-ERROR ($cnt in $mins mins): Origin: Hba $pno on '$host->{info}{name}. Destination: $port_desc on " . $st->id() . " ($port0):";
            my $code = ($type eq "a5k") ? "Ra5k":"R3h";
            my $pc  = Util->getInfo("Probable_Causes", $code);
            my $ra  = Util->getInfo("Recommended_Actions",$code) .
" Register-Origin : $hba->{RegisterName}
 Register-Destin.: $portI->[0]{RegisterName}";

            $CIMKey1 = $host->{info}{hostid} . ".hba.$pno";
            $key1    = $host->name() . ":$pno";

            $hm->linkEvent($report, "host", $CIMKey1, $key1, $type, $CIMKey2 , $key2, 
                    $desc, $reads, $writes, $hba, $pc,$ra);

         } else {
            $sw = $To->nodeByName("switch:$t_key");
            return ("Cannot find switch:$t_key") if (!$sw);
            $sw_info = $sw->{info};
            my $swI   = $sw->{portInfo};
            my($desc) = "$fc_type-ERROR ($cnt in $mins mins): Origin: Dport $pno on " . $sw->id() . 
            ", Destination: $port_desc on " . $st->id() ." ($port0):";

            my $code = ($type eq "a5k") ? "Ra5k":"R3s";
            my $pc  = Util->getInfo("Probable_Causes", $code);
            my $ra  = Util->getInfo("Recommended_Actions", $code) .
" Register-Origin : $swI->[$pno]{RegisterName}
 Register-Destin.: $portI->[0]{RegisterName}";


            $CIMKey1 = $sw->key() . ".port.$pno";
            $key1    = $sw->name() . ":$pno";

            $hm->linkEvent($report, "switch",$CIMKey1, $key1, $type, $CIMKey2 , $key2, 
                    $desc, $reads, $writes, undef, $pc,$ra);
          
         }
      }
 
   } else {
      return ("$type should be connected to a host or switch : ltype=$ltype",undef);
   }

   return (undef, "");
}


1;


