package FCRules::Rule8;
use TO;
use Data::Dumper;
use Debug;
use FCRules;
use strict;

# READ
sub description {
  return "HBA port error with target device directly attached or Iport";
}

sub category {
  return "CRC";
}

# '0|5080020000067560|hba|a5k' => '0      1       0'

sub run {
  my($class, $To, $hm, $report, $th, $count, $key, $info, $reads, $writes,
     $fc_type) = @_;

  my($hba, $wwn, $port, $type) = split(/\|/, $key);
  my($renv) = System->get_renv();
  my($current_hba);

  return if ($port ne "hba");

  my $host = $To->nodeByName("host:" . $renv->{hostname});
  if (!$host) {
    return ("FCRule8: Cannot find host Topology", undef);
  }

 
  my $hbas =  $host->{port};
  
  my $hb = $info->{hba}{$hba};

  my $port_info = $host->{portInfo};
  my $hba_no = -1;

  my($x, $ip, $link, $p);

  for ($x=0 ; $x <= $#$port_info; $x++) {
     $p = $port_info->[$x];
     if ($p->{path} =~ /$hb/ ) {
        $current_hba = $p;
        $hba_no = $x;
        last;
     }
  }

  if ($hba_no < 0) {
     return ();
  } 
  $link = $hbas->[$hba_no]; # where is this hba going?

  my($ltype, $dev_name, $pno) = split(/:/, $link);

  my $st = $To->nodeByName("$ltype:$dev_name");

  if (!$st) {
     Debug->print2("FCRule8: cannot find $ltype:$dev_name in TO");
     return ();
  }

  my($level, $cnt, $desc, $mins) = $th->test("${fc_type}counters","rule8","$wwn-$port", $count);
  $mins = 1 if (!$mins);

  return () if ($level ne "E");

  my $CIMKey2 = $host->{info}{hostid} . ".hba.$hba_no";
  my $key2    = $host->name() . ":$hba_no";

  my($CIMKey1, $key1, $desc);

  if ($ltype eq "switch") { # switch -> hba
     my $sw = $st;
     my $swI = $sw->{portInfo};

     $CIMKey1 = $sw->key() . ".port.$pno";
     $key1    = $sw->name() . ":$pno";

     $desc = "$fc_type-ERROR ($cnt in $mins mins): Origin: port $pno on " . $sw->id() . 
       ", Destination: Hba $hba_no on $host->{info}{name}:";

     my $pc  = Util->getInfo("Probable_Causes", "R8s");
     my $ra  = Util->getInfo("Recommended_Actions","R8s") .
" Register-Origin : $swI->[$pno]{RegisterName}
 Register-Destin.: $port_info->[$hba_no]{RegisterName}";

     $hm->linkEvent($report, 
            "switch", $CIMKey1 , $key1,
            "host", $CIMKey2, $key2, 
            $desc, $reads, $writes, $current_hba, $pc,$ra);
     return (undef, "");

  } else {  # iport -> hba
    my($ctrl, $port_desc);
    my $link0 = "|$dev_name|$pno|$ltype";
    my ($stI) = $st->{portInfo};

    ($CIMKey1, $key1, $ctrl, $port_desc)  = FCRules->findPortKeys($link0, $info, $st);

    $desc = "$fc_type-ERROR ($cnt in $mins mins): Origin: $port_desc path on '" . $st->id() . " , Destination:".
      "Hba $hba_no on '$host->{info}{name}:";

     my $pc  = Util->getInfo("Probable_Causes", "R8");
     my $ra  = Util->getInfo("Recommended_Actions","R8") .
" Register-Origin : $stI->[$pno]{RegisterName}
 Register-Destin.: $port_info->[$port]{RegisterName}";

     $hm->linkEvent($report, 
            $ltype, $CIMKey1 , $key1,
            "host", $CIMKey2, $key2, 
            $desc, $reads, $writes, $current_hba, $pc,$ra);
     return (undef, "");

  }
}

1;
