package Fibre::Rules::Rule8;
use TObjects;
use Data::Dumper;
use Util;

# READ
sub description {
  return "HBA port CRC error with target device directly attached or Iport";
}

sub category {
  return "CRC";
}

sub run {
  my($class, $th, $count, $key, $info, $reads, $writes) = @_;

  my($hba, $name, $port, $type) = split(/\|/, $key);

  return if (!($name eq "hba" && $port eq "-"));
  print "Rule8: $hba, $name, $port, $type<br>";

  $host_no =  $TO::NamesH->{"h:". System->hostname()};
 
  $host =  $TO::Hosts->[$host_no];
  $hbas =  $host->{port};
  
  $hb = $info->{"hba_a" . $hba};
  print "hb=$hb <br>";

  $port_info = $host->{port_info};
  $hba_no = -1;
  for ($x=0 ; $x <= $#$port_info; $x++) {
     $p = $port_info->[$x];
     my($p1) = Util->ltrim($p->{path});
     $p1 = substr($p1, 0, -7) if (substr($p1,-7) eq ":devctl");
     if ($p1 eq $hb) {
        $hba_no = $x;
        last;
     }
  }
  if ($hba_no < 0) {
     return ("Error: Cannot find hba at $hb", undef)
  } 
  print "hba_no = $hba_no<br>";
  $link = $hbas->[$hba_no];

  my($ltype, $dev_no, $pno) = split(/:/, $link);

  print "link=$ltype, $hno, $pno<br>";

  if ($ltype eq "t") { # t3
     print "   <b><font color=red>Rule 8 fired</font></b>\n";
     $t3 = $TO::Storages->[$dev_no];
     $info = $t3->{info};
     print "   <b>Origin Point</b>: T3 $info->{name} \n";
     print "   <b>Destination Point</b>: Hba $hb on $host->{info}{name}</b>\n ";
    
     Fibre::Rules->log(8,
                      "t:" . $info->{name} . ":0",
                      "h:" . $host->{info}{name} .":hb",
                      "T3 or Hba", $reads, $writes);
  }

  if ($ltype eq "a") { # a5K
     print "   <b><font color=red>Rule 8 fired</font></b>\n";
     $a5 = $TO::Storages->[$dev_no];
     $target_disk = Fibre::Rules->a5kTargetIsLast($reads, $writes, $a5, $pno);
     return (undef, "") if (!defined($target_disk));
     $a5_info = $a5->{info};
     print "   <b>Origin Point</b>: Disk $target_disk on $a5_info->{name} \n";
     print "   <b>Destination Point</b>: Hba $hba_no on $host->{info}{name}</b>\n ";

     Fibre::Rules->log(8,
           "$a5_info->{name}:$pno",
           $host->{info}{name} . ":" . $hba_no,
           "A5K or Hba", $reads, $writes);
  }

  if ($ltype eq "s") { # Switch
     print "   <b><font color=red>Rule 9 fired</font></b>\n";
     $sw = $TO::Switches->[$dev_no];
     $sw_info = $sw->{info};
     $name = $sw_info->{name};

     Fibre::Rules->log(9,
           "$name:$pno",
           $host->{info}{name} . ":" . $hba_no,
           "Iport or Hba", $reads, $writes);


    print "   <b>Origin Point</b>: Iport $pno on switch: $name at $sw_info->{ip}\n";
    print "   <b>Destination Point</b>: Hba $hba_no on $host->{info}{name} ($hb)</b>\n";

  }
  return (undef, "");
}

1;
