package T3Health;
#  $Name:  $ 
#  $Id: T3Health.pm,v 1.28 2001/09/28 16:00:25 ccadieux Exp $


use base 'Health';
use NWS::T3;
use Ilist;
use Message;
use Catalog;
use Report;
use System;
use Events;
use strict;
use Debug;

sub revision {'$Revision: 1.28 $'}

sub new {
  my($hm, $pdm) = @_;

  my($self) = {pdm => $pdm};
  bless ($self, 'T3Health');

# Specify each callback
#                            HM  Callback,  Filters
  $pdm->reportRequest($self, 'all_logic' , {category => Report::CAT_T3});
  return $self;
}

#####################################################

sub all_logic {
  my($hm, $report) = @_;
  my($pdm) = $hm->{pdm};
  my($ev, $ed, $pertains, $x ,$y, $contain, $sd);
  my($orep, $key);
  my($rep)     = $report->content;
  my($oreport) = $pdm->getLastReport($report->fileKey);
  my($ip)      = $report->accessName;
  $DB::single = 1;
  my $wwn      = $report->id('deviceName');
  my($id)      = $report->id('display');
  my($shortid) = $report->name();
  my($log)     = $report->log();
  my($renv)    = System->get_renv();
  if (!$wwn) {
     Debug->err(ERROR => "This T3 ($ip) cannot be identified");
     return;
  }
  TO->setIncomplete() if (!$oreport);

  CIM->version("1.1");

  my $invalidate_catalog = 0;

  if ($hm->connectionEvent($wwn, $report, {method => 3} )) {  # IB / OOB
     return;
  }
  my($audit); 
  my($freq) = $renv->{audit_freq} || 7;
  if (!$oreport || ($audit = $pdm->isXdays("Audit$wwn", $freq)) eq "YES" ) {  # new t300 or audit time.
     Catalog->newCatalog();
     my($etype) = $audit ? "t3.AuditEvent" : "t3.DiscoveryEvent";
     my($title) = $audit ? "Auditing":"Discovered";
     my($dt)    = "A" if (!$audit);

     $invalidate_catalog = 1;
     $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => $etype  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "$title a new T3 called $id"],
                         ]);

     $sd = Events->sourceDetector({ event => $ev , host => 1, rep => $rep});
      
     my($p) = NWS::T3->newSystem($rep);

     $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $p->[0] ],
                  [ DiscoveryType => $dt],
                    ]);
     my($monitors) = CIM::Instance->new('NWS_AgentMonitors', [
                  [ Agent       => $sd->[0]  ],
                  [ Element     => $p->[0] ],
                    ]);

     $ed = Message->new( {  id        => $report->id,
                            instances => [$ev, @$sd, @$p, $monitors, $pertains ],
                            severity  => Message::SEVERITY_NORMAL });

     $pdm->saveMessage($ed);

     if ($audit) {  # dump stats.

        $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => 't3.Statistics'  ],
                  [ EventId     => $pdm->getEventSequence  ],
                  [ Description => "Statistics about T3 $id :"],
                  [ Data        => Util->nnl($rep->toString('system.'))],
                         ]);

        $sd = Events->sourceDetector({ event => $ev });

        my($system_key) = CIM::Key->new( ['NWS_System', 
                    'Name'            =>$rep->get('id.wwn'),  
                    CreationClassName => 'NWS_System']);

        $pertains = CIM::Instance->new('NWS_EventPertainsToSystem', [
                  [ Event       => $ev  ],
                  [ Element     => $system_key ],
                    ]);

        $ed = Message->new( { id        => $report->id,
                            instances => [$ev, @$sd, $pertains ],
                            severity  => Message::SEVERITY_NORMAL });

        $pdm->saveMessage($ed);

     }
  }

  $orep = ($oreport)? $oreport->content() : $rep; # same report if new disco.

  $hm->locationChangeEvent($report, $rep, $orep, $wwn);

  my($physicalFrame) = CIM::Key->new( ['CIM_PhysicalFrame',   # key of the frame
                        Tag               => $rep->get("id.wwn") ,
                        CreationClassName => 'CIM_PhysicalFrame' ]);

  my($CTRL) = $rep->get('controller.count');
# check if disks were replaced or their state has changed.
  my($comp);
  for ($x=1; $x <= $CTRL; $x++) {
  my($oserial, $nserial);
# CONTROLLER (1)
     $comp = "controller.u${x}ctr";
     $oserial = NWS::T3->getKey($orep, $comp); 
     $nserial = NWS::T3->getKey($rep, $comp);  
     if (!$nserial && defined($nserial)) {# move the key from old to new
         NWS::T3->copyKey($comp, $orep, $rep);
         $nserial = NWS::T3->getKey($rep, $comp);  
     }

     if ($oserial eq $nserial) {
         my $S = $nserial || "$wwn.$comp";
         $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_Card', $S, $wwn);
     } else {
         if ($oserial) {
           $hm->removeCompEvent($comp,$physicalFrame, $report, 'CIM_Card',$oserial, $wwn);
         }
         if ($nserial) {
           my($ctrl) = NWS::T3->newController($rep, $comp);
           $hm->insertCompEvent($comp,$physicalFrame,$report, $ctrl, $nserial, $wwn);
           $invalidate_catalog = 1;
         }
     }

# POWER SUPPLY (2)
     for ($y=1; $y <= 2; $y++) {
        $comp = "power.u${x}pcu$y";
        $oserial = NWS::T3->getKey($orep, $comp); 
        $nserial = NWS::T3->getKey($rep, $comp);  

        if (!$nserial && defined($nserial)) {# move the key from old to new
           NWS::T3->copyKey($comp, $orep, $rep);
           $nserial = NWS::T3->getKey($rep, $comp);  
        }
        if ($oserial eq $nserial) {
           $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_PhysicalPackage', $nserial, $wwn);
           foreach my $el ('BatState','Fan1State','Fan2State','PowOutput','PowTemp') {
              my($s1) = $rep->{"$comp.fruPower$el"};
              my($s2) = $orep->{"$comp.fruPower$el"};
              if ($s1 ne $s2) {
                my($desc) = 
                   "The state of $comp.$el on $shortid changed from '$s2' to '$s1'";
                $hm->alarmEvent("", $report, $wwn, $desc);
              }
           }

        } else {
           if ($oserial) {
             $hm->removeCompEvent($comp, $physicalFrame, $report, 'CIM_PhysicalPackage',$oserial, $wwn);
           }
           if ($nserial) {
             my($part) = NWS::T3->newPowerUnit($rep, $comp);
             $hm->insertCompEvent($comp,$physicalFrame,$report, $part, $nserial, $wwn);
             $invalidate_catalog = 1;
           }
        }
     }
# LOOP CARDS (2)
     for ($y=1; $y <= 2; $y++) {
        $comp = "loopcard.u${x}l$y";

        $oserial = NWS::T3->getKey($orep, $comp); 
        $nserial = NWS::T3->getKey($rep, $comp);  

        if (!$nserial && defined($nserial)) {# move the key from old to new
           NWS::T3->copyKey($comp, $orep, $rep);
           $nserial = NWS::T3->getKey($rep, $comp);  
        }

        if ($oserial eq $nserial) {
           $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_Card', $nserial, $wwn);
           my($cable);
           foreach $cable ('1','2') {
             my($s1) = $rep->{"$comp.fruLoopCable${cable}State"};
             my($s2) = $orep->{"$comp.fruLoopCable${cable}State"};
             if ($s1 ne $s2) {
                my($desc) = "The state of $comp.Cable$cable changed from '$s2' to '
$s1'";
                $hm->alarmEvent("", $report, $wwn, $desc);
             }
           }

        } else {
           if ($oserial) {
             $hm->removeCompEvent($comp,$physicalFrame,$report, 'CIM_Card',$oserial, $wwn);
           }
           if ($nserial) {
             my($part) = NWS::T3->newLoopCard($rep, $comp);
             $hm->insertCompEvent($comp,$physicalFrame,$report, $part, $nserial, $wwn);
             $invalidate_catalog = 1;
           }
        }
     }
# DISKS (9)
     for ($y=1; $y <= 9; $y++) {
        $comp = "disk.u${x}d$y";

        $oserial = NWS::T3->getKey($orep, $comp); 
        $nserial = NWS::T3->getKey($rep, $comp);  

        if (!$nserial && defined($nserial)) {# move the key from old to new
           NWS::T3->copyKey($comp, $orep, $rep);
           $nserial = NWS::T3->getKey($rep, $comp);  
        }

        if ($oserial eq $nserial) {
           $hm->stateEvent($comp, $report, $rep, $orep, 'CIM_PhysicalPackage', $nserial, $wwn);
           foreach my $el ('Port1State','Port2State') {
              my($s1) = $rep->{"$comp.fruDisk$el"};
              my($s2) = $orep->{"$comp.fruDisk$el"};
              if ($s1 ne $s2) {
                my($desc) =
                   "The state of $comp.$el on $shortid changed from '$s2' to '$s1'";
                $hm->alarmEvent("", $report, $wwn, $desc);
              }
           }

        } else {
           if ($oserial) {
             $hm->removeCompEvent($comp,$physicalFrame,$report,'CIM_PhysicalPackage',$oserial, $wwn);
           }
           if ($nserial) {
             my($part) = NWS::T3->newDiskDrive($rep, $comp);
             $hm->insertCompEvent($comp,$physicalFrame, $report, $part, $nserial, $wwn);
             $invalidate_catalog = 1;
           }
        }
     }

# Volumes (3)
     for ($y=1; $y <= 3; $y++) {
        
        $comp = "volume.u${x}vol$y";
        my($ostatus) = $orep->{"$comp.volStatus"};
        my($nstatus) = $rep->{"$comp.volStatus"};
        if ($ostatus && $ostatus ne $nstatus) {
           #   my($volkey) = CIM::Key->new( ["NWS_StorageVolume",
           #       SystemName        => "Volume",
           #       CreationClassName => 'NWS_StorageVolume',
           #      DeviceID    => "$wwn.u${x}vol$y"] );

           $hm->stateEvent($comp, $report, $rep, $orep, 
                           'CIM_PhysicalPackage', "$wwn.u${x}vol$y",  $wwn);
        }
     }
  }
  if (0) {
  if ($invalidate_catalog) { # try to invalidate catalog on data-path hosts
     my($devs) = System->get_configDevices();
     my(%hosts);
     foreach my $d (@$devs) {
        if ($d->{type} eq "t3" && ($d->{name} eq $report->name()) && 
            $d->{datahost} && ($d->{datahost} ne $renv->{hostname}) ) {
           next if (substr($d->{datahost},0,1) eq "*");
           PDM->setStaleCatalog($d->{datahost});
        }
     }
  }
  }
}

  
sub status {
   my($hm, $rep, $orep, $comp) = @_;
   my($pdm) = $hm->{pdm};
   my($map) = $pdm->getDeviceStateMap("t3.availability");

   if ($comp =~ /volume/) {
     my($status) = $rep->getState("$comp.volStatus");
     my($ostatus) = $orep->getState("$comp.volStatus");
     my($old) = $map->get("volStatus.$ostatus");
     my($new) = $map->get("volStatus.$status");
     return ("$ostatus", "$status", $old, $new);
   }
      
   my($state) = $rep->getState("$comp.fruState");
   my($ostate) = $orep->getState("$comp.fruState");
   my($status) = $rep->getState("$comp.fruStatus");
   my($ostatus) = $orep->getState("$comp.fruStatus");

   my($old) = $map->get("fruStatus.$ostatus") . $map->get("fruState.$ostate");
   my($new) = $map->get("fruStatus.$status") . $map->get("fruState.$state");

   $old = ($old eq "11")? 1:0;
   $new = ($new eq "11")? 1:0;
   return ("$ostatus-$ostate", "$status-$state", $old, $new);
}

# static method
# return status-state and 0/1
#  =controller.u1ctr
#  =power.u1pcu1
#  =power.u1pcu2
#  =loopcard.u1l1
#  =loopcard.u1l2
#  =disk.u1d1
#  =disk.u1d2
#  =disk.u1d3
#  =disk.u1d4
#  =disk.u1d5
#  =disk.u1d6
#  =disk.u1d7
#  =disk.u1d8
#  =disk.u1d9


sub compStatus {
   my($hm, $rep, $comp) = @_;

   my($map) = PDM->getDeviceStateMap("t3.availability");

   my($state) = $rep->get("$comp.fruState");
   my($status) = $rep->get("$comp.fruStatus");

   my($new) = $map->get("fruStatus.$status") . $map->get("fruState.$state");

   $new = ($new eq "11")? 1:0;
   return ("$status-$state", $new);
}





sub NUhealth1 {
  my($report, $pdm, $cat, $comp0, $rep, $orep, $cim1, $physicalFrame, $type, $wwn) = @_;
  my($sd, $pertains, $ed , $ev, $key, $contain);

   my($comp) = "$cat.$comp0";
   my($oserial) = $orep->get("$comp.fruSerialNo") ;
   my($nserial) = $rep->get("$comp.fruSerialNo") ;

   if ($oserial eq $nserial) {
      my($map) = $pdm->getDeviceStateMap("t3.availability");

      my($state) = $rep->get("$comp.fruState");
      my($ostate) = $orep->get("$comp.fruState");
      my($status) = $rep->get("$comp.fruStatus");
      my($ostatus) = $orep->get("$comp.fruStatus");

      my($old) = $map->{"fruStatus.$ostatus"} . $map->{"fruState.$ostate"};
      my($new) = $map->{"fruStatus.$status"} . $map->{"fruState.$state"};

      $old = ($old eq "11")? 1:0;
      $new = ($new eq "11")? 1:0;

      if ($old != $new) { # availability change.
         my($data) = "State=[o:$ostate,n:$state],Status=[o:$ostatus,n:$status]";

         $ev = CIM::Instance->new('NWS_StateChangeEvent', [
             [ EventType    => 't3.StateChangeEvent'   ],
             [ EventId      => $pdm->getEventSequence  ],
             [ CurrentState => $new],
             [ PriorState   => $old],
             [ Data         => $data],
                    ]);

         $sd = Events->sourceDetector({ event => $ev });

         $key = CIM::Key->new( [$cim1,   
                  Tag               => $nserial,
                  CreationClassName => $cim1]);

         $pertains = CIM::Instance->new('NWS_EventPertainsToPhysicalElement', [
             [ Event       => $ev  ],
             [ Element     => $key ],
               ]);

         $ed = Message->new( { id        => $report->id,
                       instances => [$ev, @$sd, $pertains ],
                       severity  => Message::SEVERITY_ERROR });

         $pdm->saveMessage($ed);
      }
   } else {  # new component
      if ($oserial) {  # remove old one
         $ev = CIM::Instance->new('NWS_Event', [
             [ EventType   => 't3.TopologyEvent'  ],
             [ Description => "$cat-remove" ],
             [ EventId     => $pdm->getEventSequence  ]]);

         $sd = Events->sourceDetector({ event => $ev });
         
         $key = CIM::Key->new( [$cim1,   # key of the disk
                  Tag               => $oserial,
                  CreationClassName => $cim1]);

         $contain = CIM::Instance->new( 'CIM_Container', [
                  [GroupComponent => $physicalFrame], # frame is a package
                  [PartComponent  => $key],           # package is an element
                  [LocationWithinContainer => $rep->get("$comp.fruId")],
                   ]);

         $pertains = CIM::Instance->new('NWS_EventPertainsToContainer', [
                  [Event          => $ev   ],
                  [Association    => $contain ],
                  [DiscoveryType => "D"   ],   # delete
               ]);

         $ed = Message->new( { id        => $report->id,
                       instances => [$ev, @$sd, $contain, $pertains ],
                       severity  => Message::SEVERITY_ERROR });

         $pdm->saveMessage($ed);
      } 
      if ($nserial) {   # add new disk
         $ev = CIM::Instance->new('NWS_Event', [
             [ EventType   => 't3.TopologyEvent'  ],
             [ Description => "$cat-add" ],
             [ EventId     => $pdm->getEventSequence  ]]);

         $sd = Events->sourceDetector({ event => $ev });
         my($part);
         if ($type eq "D") {
           $part = NWS::T300->newDiskDrive($rep, $comp0);
         } elsif ($type eq "C") {
           $part = NWS::T300->newController($rep, $comp0);
         } elsif ($type eq "L") {
           $part = NWS::T300->newLoopCard($rep, $comp0);
         } elsif ($type eq "P") {
           $part = NWS::T300->newPowerUnit($rep, $comp0);
         }
         $pertains = CIM::Instance->new('NWS_EventPertainsToPhysicalElement', [
             [ Event       => $ev  ],
             [ Element     => $part->[1]],
               ]);

         $contain = CIM::Instance->new('CIM_Container', [
              [GroupComponent           => $physicalFrame],
              [PartComponent            => $part->[1]],
              [LocationWithinContainer  => $rep->get("$comp.fruId")], 
               ]);
         
         $ed = Message->new( { id        => $report->id,
                       instances => [$ev, @$sd, @$part, $pertains , $contain],
                       severity  => Message::SEVERITY_ERROR });

         $pdm->saveMessage($ed);
      }
   }
}


1;
