package TOPOHealth;
#  $Name:  $ 
#  $Id: TOPOHealth.pm,v 1.11 2001/07/27 23:36:51 ccadieux Exp $


use base 'Health';
use NWS::SAN;
use Ilist;
use Message;
use Report;
use Events;
use FCRules;
use strict;
use Debug;
use TO;
use UNIVERSAL qw( isa ) ;

sub revision {'$Revision: 1.11 $'}

sub new {
  my($hm, $pdm) = @_;

  my($self) = {pdm => $pdm};
  bless ($self, 'TOPOHealth');

# Specify each callback
#                            HM  Callback,  Filters
  $pdm->reportRequest($self, 'all_logic' , {category => Report::CAT_TOPO});
  return $self;
}

#####################################################

sub all_logic {
  my($hm, $report) = @_;
  my($pdm) = $hm->{pdm};
  my($ev, $ed, $pertains, $x ,$y, $contain, $sd);
  my($key, $x);
  my($oreport) = $pdm->getLastReport($report->fileKey);
  my $ic = PDM->getCacheHandle("incomplete_topo");

  return if ($ic->{try});   # do not send incomplete topos.

  my $to      = $report->{_value};
  $DB::single = 1;

  if (!defined($to)) {
    Debug->err(TEXT => "Topology module returned empty topology");
    return;
  }
  if (!isa($to, 'TO')) {
    Debug->err(TEXT => "TOPOHealth received invalid TOPO object");
    return;
  }
  if ($oreport) {
    if (!TO->diff($oreport->{_value}, $to)) {   # skip if topo is the same.
        return;
    }
  }

  my($pdm) = $hm->{pdm};
  my($renv) = System->get_renv();

  my($rep) = {};
  Agent::addIdentification({renv => $renv}, $rep);
  $rep = ReportContent->new($rep);
  

  CIM->version("1.1");

  my($hosts) = $to->hostList();
  my($switches) = $to->switchList();
  my($storages) = $to->storageList();
  my($id) =  $pdm->getEventSequence ;

  my($ev, $sd, $pertains);
  my($zoom, $data) = $to->appletStream();
  
  my($desc) = "Topology event with " . ($#$hosts+1) . " host(s), " . 
                ($#$switches+1) . " switches and " .
                ($#$storages+1) . " Storage Array(s)";

  $ev = CIM::Instance->new('NWS_Event', [
                  [ EventType   => "topo.DiscoveryEvent"  ],
                  [ EventId     => $id ],
                  [ Caption     => "SAN_Topology" ],
                  [ Description => $desc ],
                  [ Data        => $data ],
                         ]);

  $sd = Events->sourceDetector({ event => $ev ,  host => 1, rep => $rep});
  my($hostInstance) = $sd->[3];
  my($node1, $node2, $hba, $p, $hba1, $link);
  my(@all);
  push(@all, $ev,@$sd);

  foreach my $h (@$hosts) {
      my($hbas) = $h->port();
      my($hba_infos) = $h->portInfo();

      for ($hba=0; $hba <= $#$hbas; $hba++) {
           my($hb) = $hba_infos->[$hba];
           next if (substr($hb->{RegisterName},0,6) eq "pseudo");
           $node1 = $h->hostid() . ".port.$hba";
           my($type, $wwn, $port) = split(/:/, $hbas->[$hba]);
           next if (!$type || $type eq "hub");

           $hba1 = CIM::Instance->new('NWS_HbaPort', [
                  [ Tag         =>  $node1         ],
                  [ PortWWN     =>  $hb->{PortWWN} ],
                  [ NodeWWN     =>  $hb->{NodeWWN} ],
                  [ RegisterName=>  $hb->{RegisterName}],
                  [ LGroup      =>  $hb->{LGroup}  ],
                  [ HbaNumber   =>  $hba           ],
                  [ Path        =>  $hb->{path}    ],
                         ]);

           my($agent) = uc($type) . "Agent";
           $node2 = "";
           eval {
             $node2 = $agent->getPortKey($wwn, $port);
           };

           next if (!$node2);

           my($hba2host) = CIM::Instance->new('NWS_HostHba', [
                  [ Host      => $hostInstance ],
                  [ Hba       => $hba1         ],
                  [ LocationWithinContainer =>  "port.$hba" ],
                    ]);

           my($node2cim) = CIM::Key->new( ['CIM_Card',
                   CreationClassName => 'CIM_Card',
                   Tag               => $node2]);


           $link = CIM::Instance->new('NWS_Link', [
                  [ Link1       => $hba1     ],
                  [ Link2       => $node2cim ],
                    ]);

           $pertains = CIM::Instance->new('NWS_EventPertainsToLink', [
                  [ Event       => $ev  ],
                  [ Link        => $link ],
                    ]);
           push(@all, $hba1,$hba2host, $link, $pertains);
      }
  }
  my %DONE;
  foreach my $sw (@$switches) {
      my($ports) = $sw->port();
      my($dports) = $sw->dport();
      my($port_infos) = $sw->portInfo();
  
      my($p);
      $node1 = $sw->key() ;

      for ($p=0; $p <= $#$dports; $p++) {  # ports in the same switch
          my($dp) =  $dports->[$p];
          next if (!$dp);
          $dp = substr($dp,0,-1) if (substr($dp,-1) eq "-");
          my(@dp2) = split(/\-/, $dp);
          foreach my $dp0 (@dp2) {

            my($switchport) = CIM::Key->new( ['CIM_Card',
                   CreationClassName => 'CIM_Card',
                   Tag               => "$node1.port.$p"]);

            my($switchport2) = CIM::Key->new( ['CIM_Card',
                   CreationClassName => 'CIM_Card',
                   Tag               => "$node1.port.$dp0"]);


           $link = CIM::Instance->new('NWS_Link', [
                  [ Link1       => $switchport   ],
                  [ Link2       => $switchport2  ],
                    ]);

           $pertains = CIM::Instance->new('NWS_EventPertainsToLink', [
                  [ Event       => $ev  ],
                  [ Link        => $link ],
                    ]);

           push(@all, $link, $pertains);
                 
          }
      }
 
      for ($p=0; $p <= $#$ports; $p++) {
           my($type, $wwn, $port) = split(/:/, $ports->[$p]);
           next if (!$type || ($type eq "host"));

           $node1 = $sw->key() . ".port.$p";

           if ($type eq "hub") {
              my($hub) = $to->nodeByName("hub:$wwn");
              my($t0, $wwn0, $port0) = split(/:/, $hub->{info}{from});
              $node2 = "$wwn0.port.$port0";

           } else {  # any storage or switch
              my($agent) = uc($type) . "Agent";
              $node2 = "";
              eval {
                 $node2 = $agent->getPortKey($wwn, $port);
              };
           }
           next if (!$node2);

           my($switchport) = CIM::Key->new( ['CIM_Card',
                   CreationClassName => 'CIM_Card',
                   Tag               => $node1]);

           my($storageport) = CIM::Key->new( ['CIM_Card',
                   CreationClassName => 'CIM_Card',
                   Tag               => $node2]);

           my $done;
           if ($node1 lt $node2) {
               $done = "$node1$node2";
           } else {
               $done = "$node2$node1";
           }
           next if ($DONE{$done});
           $DONE{$done} = 1;

           $link = CIM::Instance->new('NWS_Link', [
                  [ Link1       => $switchport   ],
                  [ Link2       => $storageport  ],
                    ]);

           $pertains = CIM::Instance->new('NWS_EventPertainsToLink', [
                  [ Event       => $ev  ],
                  [ Link        => $link ],
                    ]);

           push(@all, $link, $pertains);
       }
   }
   # does storage to hub (link to what is behind the hub)
   foreach my $st (@$storages) {
      my($ports) = $st->port();
      my($port_infos) = $st->portInfo();
  
      my($p);
      for ($p=0; $p <= $#$ports; $p++) {
           my($type, $wwn, $port) = split(/:/, $ports->[$p]);
           next if ($type ne "hub");

           my($agent) = uc($st->type() ) . "Agent";
           $node1 = "";
           eval {
              $node1 = $agent->getPortKey($st->key(), $p);
           };
           next if (!$node1);

           my($hub) = $to->nodeByName("hub:$wwn");

           my($t0, $wwn0, $port0) = split(/:/, $hub->{info}{from});
           $node2 = "$wwn0.port.$port0";

           my($storageport) = CIM::Key->new( ['CIM_Card',
                   CreationClassName => 'CIM_Card',
                   Tag               => $node1]);

           my($sw_ho_port) = CIM::Key->new( ['CIM_Card',
                   CreationClassName => 'CIM_Card',
                   Tag               => $node2]);

           $link = CIM::Instance->new('NWS_Link', [
                  [ Link1       => $storageport  ],
                  [ Link2       => $sw_ho_port   ],
                    ]);

           $pertains = CIM::Instance->new('NWS_EventPertainsToLink', [
                  [ Event       => $ev  ],
                  [ Link        => $link ],
                    ]);

           push(@all, $link, $pertains);

       }
   }

   $ed = Message->new( { id  => $report->id,
                         instances => \@all, 
                         severity  => Message::SEVERITY_NORMAL });

   $pdm->saveMessage($ed);
}
1;

