package Http;
use IO::Socket;
#  $Name:  $ 
#  $Id: Http.pm,v 1.13 2001/08/03 22:49:17 ccadieux Exp $
#
# $h = Http->new({ log => 0 });
#

$sockaddr = 'S n a4 x8';
$VERSION  = "1.1";
$INVALID_LOGIN = "
   You must provide a username and password to use this resource. Either you
   entered this information incorrectly, or your browser does not know how to
   present the credentials required.";

#$0 = "httpi: handling request";

%content_types =
  ("html" => "text/html",
    "htm" => "text/html",
    "txt" => "text/plain",
    "gif" => "image/gif",
   "jpeg" => "image/jpeg",
    "jpg" => "image/jpeg",
    "js" =>  "text/html"
  );


sub new {
  my($class, $info) = @_;

  bless($info, "Http");
  return $info;
} 

sub sock_to_host {
    local($sock) = getpeername(STDIN);

    return (undef, undef, undef) if (!$sock);
    local($AFC, $port, $thataddr, $zero) = unpack($sockaddr, $sock);
    local($ip) = join('.', unpack("C4", $thataddr));
    return ($ip, $port, $ip);
}

sub ht_response {
   ($currentcode, $currentstring) = (@_);
   return if (0+$httpver < 1);

   $rfcdate = scalar gmtime;
   ($dow, $mon, $dt, $tm, $yr) = ($rfcdate =~ m/(...) (...) (..) (..:..:..) (....)/);
   $dt += 0; $yr += 0;
   $rfcdate = "$dow, $dt $mon $yr $tm GMT";

   my($what) = <<"EOF";
HTTP/$httpver $currentcode $currentstring
Server: HTTPi/$VERSION
MIME-Version: 1.0
Version: $httpver
Date: $rfcdate
EOF
    $what =~ s/\n/\r\n/g;
    $OUT .= $what;
    &hthead("Connection: close") if (0+$httpver > 1);
}

sub hthead {
    my($header, $term) = @_;
    return if (0+$httpver < 1);
    $OUT .=  "$header\r\n" . (($term) ? "\r\n" : "");
}

sub htcontent {
    local($what, $ctype, $mode) = (@_);
    ($contentlength) = $mode || length($what);
    &hthead("Content-Length: $contentlength");
    &hthead("Content-Type: $ctype", 1);
    return if ($method eq 'HEAD' || $mode);
    $OUT .= $what;
}

sub text_header {
  my($http,$text) = @_;
  "HTTP/1.0 200 OK\nContent-type: text/html\n\n$text";
}

sub error {
  my($http, $no, $err, $text) = @_;
  $OUT = "";
  &ht_response($no, $err);
  &hterror("Error", $text);
}

sub debug {
  my($m) = @_;
  open(D, ">>/tmp/httplog"); print D "$m\n"; close(D);
}

sub log {
  my($http) = @_;
  return;

  $date = scalar localtime;
  ($dow, $mon, $dt, $tm, $yr) = ($date =~ m/(...) (...) (..) (..:..:..) (....)/);
  $dt += 0;
  $dt = substr("0$dt", length("0$dt") - 2, 2);
  $date = "$dt/$mon/$yr:$tm +0000"; 

  return if (!$http->{logfile});

  if (open(J, ">>$http->{logfile}")) {
     local $q = $address . (($variables) ? "?$variables" : "");
     $contentlength += 0;
     $contentlength = 0 if ($method eq 'HEAD');
     local ($hostname, $port, $ip) = &sock_to_host();
     $hostname = $hostname || "-";
     $httpuser = $httpuser || "-";
     print J <<"EOF";
$hostname - $httpuser [$date] "$method $q HTTP/$httpver" $currentcode $contentlength "$httpref" "$httpua"
EOF
     close(J); 
  }
}


sub bye { 
  exit; 
}

sub dead {
    &ht_response(500, "Server Error");
    &hterror("Server Error", <<"EOF");
While handling a request for resource $address, the server crashed. Please
attempt to notify the administrators.
<p>Useful(?) debugging information:
<pre>
@_
</pre>
EOF
    &log; 
}

sub http_init {
  $OUT = undef;
  $method = $address = $httpver = $httpref = $httpua = $httpver = undef;
  $httphost = $httpref = $httpua = $httpcl = $httpct = $expect = $httprawu = undef;
  $post_data = undef;
}

sub unix_init {
  my($class, $file) = @_;
  my($PID, $pid);
  $UNIXFILE = $file;
  $PID = System->get_home() . "/DATA/rashttp.pid";

  if (-f $PID) {
    if (open(O, $PID)) {
      $pid = <O>; close(O);
      debug("rashttp already running, killing $pid");
      kill 9, $pid;
    }
  }
  unlink $UNIXFILE;
  open(O, ">$PID"); print O $$; close(O);

  $server = IO::Socket::UNIX->new( Type => SOCK_STREAM,
                         Local => $UNIXFILE,
                         Listen => 1);
  if (!$server) {
    debug("failed on socket ($file): $!");
    exit;
  }
}

sub unix_loop {
  my($http, $util, $master) = @_;
  my($starttime, $client) = 0;
  my($fun, $q, $post_data);
  debug("timeout = $http->{timeout}");
  local $SIG{ALRM} = sub { 
        unlink $UNIXFILE; 
        close($server);  
        unlink(System->get_home() . "/DATA/rashttp.pid");
        exit(0);
       };

  while ($client = $server->accept()) { 
#    break if (time - $starttime > $http->{timeout} +2);  # if alarm does not work
    alarm(0);
    $client->autoflush(1);
    select($client);

    ($fun, $q, $post_data) = Http::sock_process($http, $client, $master);



    eval {
      if ($fun) {
         print $http->text_header;
         if ($master && !$q->{GET} && !$q->{PUT} &&   ($q->{GO} !~ /\:\:/) ) {
           print 
             "<body bgcolor=#F0F0F0>\n\n<center><h3>This is a SLAVE Storage Agent, connect to the MASTER ".
             "with your browser<p> Go  to ".
             "<a href=http://$master:$main'RASPORT target=_top>MASTER Host Agent</a> now.\n";
         } else {
           $ix = rindex($fun, "::");
           if ($ix >= 0) {
              $pk = substr($fun, 0 , $ix);
              $pk =~ s/\:\:/\//g;
              require "$pk.pm";
           } else {
              $fun = "main'$fun";
           }
           if (defined(&$fun)) {
              &$fun($q, $post_data, $util);
           } else {
             print "Function '$fun' not found!<br>";
           }
         }
      }
    };
    if ($@) {
       print "Execution error: $@<br>\n";
    }
    close($client);
    alarm($http->{timeout});
    $starttime = time;
  }
  close($client);
  close($server);

  unlink $UNIXFILE;
}
  

sub sock_process {
   my($http, $sock, $master)  = @_;
   my($post_data, $q, $fun);
   local($SIG{'ALRM'}) = \&bye;
#  $SIG{'__DIE__'} = \&dead;
   
   $address = 0; 
   alarm 60;
   $SAVE = "";
   &http_init;
   while ($line = <$sock>) {
       process1($line);
   }
   if ($method =~ /POST/) {  # get POST data
        $len = $httpcl; $start = 0;
        while ($len > 0) {
            $re = read($sock, $post_data, $len, $start);
            last if (!$re);
            $start += $re;
            $len -= $re;
        }
   }
   ($fun, $q) = &process2($http, $post_data, $master);
   print $OUT if ($OUT);
   return ($fun, $q, $post_data);  
}


sub stdin_process {
   my($http) = @_;
   my($post_data, $q, $fun);

   $SIG{'ALRM'} = \&bye;
#  $SIG{'__DIE__'} = \&dead;
   
   # $sock = getpeername(STDIN);
   
   select(STDOUT); $|=1; $address = 0; 
   alarm 60;
   $SAVE = "";
   &http_init;
   while ($line = <STDIN>) {
       process1($line);
   }
   if ($method =~ /POST/) {  # get POST data
        $len = $httpcl; $start = 0;
         while ($len > 0) {
            $re = read(STDIN, $post_data, $len, $start);
            last if (!$re);
            $start += $re;
            $len -= $re;
        }
   }
   ($fun, $q) = &process2($http, $post_data);
   print $OUT if ($OUT);
   return ($fun, $q, $post_data);  
}

#  $fun =  loadPackage("get_", $q{GET});

sub loadPackage {
  my($prefix, $fun) = @_;
  my( $pk, $name, $pk0);
  my($i) = rindex($fun, "::");
  if ($i > 0) {
    $pk = substr($fun, 0, $i);
    $name = substr($fun, $i+2);
    $pk0 = $pk;
    $pk0 =~ s/\:\:/\//g;
    require "$pk0.pm";
    return $pk . "::" . "$prefix$name";
  } else {
    return $prefix . $fun;
  }
}
  

sub process1 {
   my($line) = @_;

   $SAVE .= $line;
   if($line =~ /^([A-Z]+)\s+([^\s]+)\s+([^\s\r\l\n]*)/) {  # GET / POST
        $method = $1;
        $address = $2; 
        $httpver = $3;
        $httpref = '';
        $httpua = '';
        $httpver = ($httpver =~ m#HTTP/([0-9]\.[0-9]+)#) ?  ($1) : (0.9);
        $address =~ s#^http://[^/]+/#/#;
        next unless ($httpver < 1);
    } else {                                      # ARGS
         $line =~ s/[\r\l\n\s]+$//;
         if ( $line =~ /^Host: (.+)/i) {
            $httphost = $1; $httphost =~ s/:\d+$// ;
         }
         if ($line =~ /^Referer: (.+)/i) { 
            $httpref = $1;
         }
         ($line =~ /^User-agent: (.+)/i) && ($httpua = $1);
         ($line =~ /^Content-length: (\d+)/i) && ($ENV{'CONTENT_LENGTH'} = $httpcl = $1);
         ($line =~ /^Content-type: (.+)/i) && ($ENV{'CONTENT_TYPE'} = $httpct = $1);
         ($line =~ /^Expect: /) && ($expect = 1);
         ($line =~ /^Authorization: Basic (.+)/i) && ($httprawu = $1);

         last if ($line =~ /^$/);  # empty line is delimiter
    }
}


sub process2 {
   my($http, $post_data, $master) = @_;
   if ($expect) {
       &ht_response(417, "Expectation Failed");
       &hterror("Expectation Failed", "The server does not support this method.");
       &log; 
       return (undef);
   }
   if (!$address || (0+$httpver > 1 && !$httphost)) {
       &ht_response(400, "Bad Request");
       &hterror("Bad Request", "The server cannot understand your request.");
       return (undef);
   }
   if ($method !~ /^(GET|HEAD|POST)$/) {
      &ht_response(501, "Illegal Method");
      &hterror("Illegal Method", "Only GET, HEAD and POST are supported.");
      &log; return (undef);
   }
   ($address, $variables) = split(/\?/, $address);
   my(%q);
   &parse($variables, \%q);
   if ($method =~ /POST/) {
      &parse($post_data, \%q);
   }
   my($ctype) = "Content-type: text\html\n\n\n";
   $address=~ s#^/?#/#;
   1 while $address =~ s#/\.(/|$)#\1#;
   1 while $address =~ s#/[^/]*/\.\.(/|$)#\1#;
   1 while $address =~ s#^/\.\.(/|$)#\1#;
   
#------- LOGIN ---------

   if (!$q{GET} && !$q{PUT} && !$master) {
       $httprawu =~ tr#A-Za-z0-9+/##cd;
       $httprawu =~ tr#A-Za-z0-9+/# -_#;
       $httprawu = unpack("u", pack("c", 32+0.75*length($httprawu)) . $httprawu);
       ($httpuser, $httppw) = split(/:/, $httprawu);
       $fail = 1;
       if ($httpuser eq $http->{login} && 
          crypt($httppw, substr($http->{password}, 0, 2)) eq $http->{password}) {
          $fail = 0;
       }
       if ($fail) {
          $httpuser = '';
          &ht_response(401, "Authorization Required");
          &hthead("WWW-Authenticate: Basic realm=\"Network Storage Agent Admin (UserID='ras')\"");
          &hterror("Authorization Required", $INVALID_LOGIN);
          &log; return (undef);
       }
     }
   
    $address =~ s/%([0-9a-fA-F]{2})/pack("H2", $1)/eg;
    $raddress = "$http->{root}$address" ;
    if ($address !~ m#/$# && -d $raddress) {
       &hterror301("http://NSAgent$address/");
    }
    if (-d $raddress) {
       $raddress = "${raddress}index.html";
    }
    alarm 0;
    my($fun);

    if ($q{GET}) {
       $fun =  loadPackage("get_", $q{GET});
    } elsif ($q{PUT}) {
       $fun =  loadPackage("put_", $q{PUT});
    } elsif ($q{GO}) {
       $fun =  loadPackage("", $q{GO});
    } else {
       $fun = $q{GO};
    }
#------- FUNCTION ---------
    if ($fun) {  # functions
       return ($fun,\%q);
    }

#------- STATIC FILE (html, gif etc.. )---------
    if(!open(S, $raddress)) {  # static file
        &hterror404; 
        return (undef);
    } else {
       if (-x $raddress) {      # no CGI supported
         &hterror404; 
         return (undef);
       }
       ($x,$x,$x,$x,$x,$x,$x,$length,$x,$mtime) = stat(S);
       $mtime = scalar gmtime $mtime;
       ($dow, $mon, $dt, $tm, $yr) = ($mtime =~ m/(...) (...) (..) (..:..:..) (....)/);
       $dt += 0; $yr += 0;
       $ctype = 0;
       foreach $el (keys %content_types) {
           if ($raddress =~ /\.$el$/i) {
               $ctype = $content_types{$el};
           }
       }
       $ctype ||= 'text/plain';
       &ht_response(200, "OK");
       &hthead("Last-Modified: $dow, $dt $mon $yr $tm GMT");
       &htcontent("", $ctype, $length);
       unless ($method eq 'HEAD') {
          while(!eof(S)) {
              read(S, $q, 16384);
              $OUT .= $q;
          }
       }
       alarm 0;
       return (undef);
   }
   $http->log;
}

sub hterror {
    local($errstr, $expl) = (@_);
    &htcontent(<<"EOF", "text/html");
<html>
<body>
<h1>$errstr</h1>
$expl
<hr>
<address>httpi/$VERSION</address>
</body>
</html>
EOF
}

sub hterror404 {
    &ht_response(404, "File Not Found");
    &hterror("File Not Found",
        "The resource $address was not found on this system.");
}

sub hterror301 {
    &ht_response(301, "Moved Permanently");
    &hthead("Location: @_");
    &hterror("Resource Moved Permanently",
        "This resource has moved <a href = \"@_\">here</a>.");
    $keep = 0; 
  &log;
}

sub parse {
  my($query_s, $q) = @_;
  my($cnt);

   foreach $el (split(/&/,$query_s)) {
     ($NAME,$VALUE) = split(/=/, $el);
     $NAME =~ s/\+/ /g;
     $NAME =~ s/%([0-9\.|A-F|a-f]{2})/pack(C,hex($1))/eg;

     $VALUE =~ s/\+/ /g;
     $VALUE =~ s/%([0-9|A-F|a-f]{2})/pack(C,hex($1))/eg;
     if ($q->{$NAME}) {
        $q->{$NAME} .= "\t$VALUE";
     } else {
        $q->{$NAME} = $VALUE;
     }

  }
  return %q;
}


1;
