package NWS::Schema;
#  $Name:  $ 
#  $Id: Schema.pm,v 1.5 2001/08/24 21:32:30 ccadieux Exp $

use System;

use strict;
use NWS::SCH_Events;
use NWS::SCH_Storage;
use Data::Dumper;
use Carp;
use CIM::Mof;
sub revision {'$Revision: 1.5 $'}

#  Schema
# 
#  S:String B:Boolean I:uint16 I32:uint32 I64:uint64 K:Key E:Enumeration D:DataTime R:Reference
#  SA:StringArray
#  Q: Qualifiers => [name, "default", type]
#  Element and Association are not declared and are generic.
#           propertyName         => [propType, KeyFlag, Default, Enumeration, Qualifiers ]
#   ClassName => ['Type', 'ParentClass', {
#           stringproperty       => [S, 0, "default value"                    ],
#           dateproperty         => [D, 0,  NOW                               ],
#           enumproperty         => [E, 0, "default", "enumeration values"    ],
#           keystringproperty    => [S, K                                     ],
#           propertywithQuals    => [S, 0 ,default, "enumerationvalues", 
#                                       [unit  => default, type,
#                                        color => default, type]        ],
#                }],
#   Type: E=Event, H=Host, D=StorageDev, C=StorageComponent, 
#         S=Software, A=Association, O=Other, L=Alerts
#         I= !A
#
use vars qw($S %SCHEMAS $COMPILED %SCH);

$COMPILED = 0;
%SCHEMAS = ();
%SCH = ();

sub read {
  my($this, $type, $version) = @_;
  return $SCH{$type . $version};
}

sub init {
  my($class) = @_;
  my($E);
  opendir(O, System->get_home() . "/System");
  my(@a) = readdir(O); closedir(O);
  foreach my $f (@a) {
     next if (substr($f,-4) ne ".mof");
     my($name) = substr($f,0,-4);

     my($mofFile) = System->get_home() . "/System/$name.mof";
     $name =~ /\w+_(\d+)\.(\d+)$/;
     my($version) = "$1.$2";
     my($schFile) = System->get_home() . "/DATA/$name.sch";
     my($mofAge)  = (stat($mofFile))[9];
     my($schAge)  = 0;
     $schAge = (stat($schFile))[9] if (-f $schFile);
     if ($schAge < $mofAge) {
        Debug->print1("Reading mof $mofFile");
        eval {
          $E = CIM::Mof->fromMof($mofFile);
        };
        if ($@) {
           croak("ERROR: Failed to read mof file $mofFile: $@");
        }
        open(O, ">$schFile");
        print O dumper($E);
        $Data::Dumper::Indent = 1;
#        print O Dumper($E);
        close(O);
     } else {
        open(O, $schFile);
        my(@l) = <O>; close(O);
        $E = eval substr("@l", 8);
     }
     croak("Invalid Schema!") if (ref($E) ne "HASH");
     if ($E) { # combine
       $SCH{substr($name,0,1) . $version} = $E;
       if ($SCHEMAS{$version}) {
         my($F) = $SCHEMAS{$version};
         foreach my $k (keys %$E) {
             if ($F->{$k}) {
               Debug->print1("Class is in two schemas: $f : $F->{$k}");
             }
             $F->{$k} = $E->{$k};
         }
       } else {
         $SCHEMAS{$version} = $E;
       }
     }
  }
  my($done) = 1;
}

sub versions {
  my($class) = @_;
  my(@v) = keys %SCHEMAS;
  return @v;
}

sub dumper {
  my($E) = @_;
  my($o);
  $o = "\$VAR1 = { \n";
  foreach my $k (sort keys %$E) {
      my($h) = $E->{$k}[0];
      my($parent) = $h->{parent}? "'$h->{parent}'": $h->{parent};
      $o .= " $k => [ { type => '$h->{type}', parent => $parent }, [ \n";

      my($pro) = $E->{$k}[1];
      foreach my $p (@$pro) {
          my($key) = $p->[2]? "'$p->[2]'": 0;
          my($def) = defined($p->[3])? "'$p->[3]'": "undef";
          my($ln) =  length($p->[0]); my($sp);
          $sp  = ($ln < 20)? " " x (20 - $ln) : " ";
          my($quals) = "";
          my($q) = $p->[4];
          if ($#$q >= 0) {
             $quals = ",[";
             my($x);
             for ($x=0; $x < $#$q; $x+= 3) {
               $quals .= " $q->[0] => '$q->[1]', '$q->[2]',";
             }
             chop($quals); 
             $quals .= "]";
          }
          $o .= "   [ $p->[0]$sp => '$p->[1]', $key, $def,undef $quals ],\n";
      }
      $o .= "   ]],\n\n";
   } 
   $o .= "};\n";
}

# do not compile

sub _database {
  my($class, $version) = @_;

  $version = "1.1" if (!$version);
  return $SCHEMAS{$version};
}

  
# compile and returns static pointer

sub database {
  my($class,$version) = @_;
  my($i);
  $version = "1.1" if (!$version);
  
  if ($COMPILED) {
    return $SCHEMAS{$version};
  } else {
    my($name, $type, @compile, $cnt, $parent, %find, $e);
    foreach my $v (keys %SCHEMAS) {
      foreach my $class (keys %{$SCHEMAS{$v}} ) {
        $e = $SCHEMAS{$v}{$class};
        my($isa) = ":$class:";
        %find = (); @compile = (); $cnt = 0; 
        my($level) = -1;
        while (1) {
           $level++;
           foreach my $db_el (@{$e->[1]}) {
              $name = $db_el->[0];
              $type = $db_el->[1];
              if ( defined($i = $find{$name}) ) {
                 if ($type ne "R") {  # nothing to change on references
                   print STDERR "PropertyRepeat $name in ($parent/$class) \n" if (Debug->level> 3);
                   my($c) = $compile[$i];
                   $compile[$i] = [$name, $type,
                                $c->[2] || $db_el->[2],   # key
                                $c->[3] || $db_el->[3],   # default
                                $c->[4] || $db_el->[4],   # enum list
                                $c->[5] || $db_el->[5],   # quals
                                $c->[6],
                               ];
                 }
              } else {
                $compile[$cnt] =  $db_el;
                $compile[$cnt][6] = $level;
                $find{$name} = $cnt;
                $cnt++;
              }
           }
           last if ($e->[2]);
           $parent = $e->[0]{parent};
           last if (!$parent);
           $isa .= "$parent:";
           $e = $SCHEMAS{$v}{$parent};
        }
        $SCHEMAS{$v}{$class}->[0]{isa} = $isa;
        $SCHEMAS{$v}{$class}->[1] = [@compile];
        $SCHEMAS{$v}{$class}->[2] = 1; # compiled flag
      }
    }
    $COMPILED = 1;
    return $SCHEMAS{$version};
  }
}


sub verify {
  my($class) = @_;
  my(%P, $cnt);

  foreach my $v (keys %SCHEMAS) {
    %P = ();
    foreach my $x1 (keys %{$SCHEMAS{$v}} ) {
       $P{$x1} = $SCHEMAS{$v}{$x1}->[0]{parent};
    }
    $cnt = 0;
    foreach my $x (keys %P) {
       next if ($P{$x} eq "0");
       if (!exists($P{$P{$x}})) {
          print "parent $P{$x} does not exists!\n"; $cnt++;
       }
    }
  }
  print "Sckema OK!\n" if (!$cnt);
}

#  print Schema->tree({ version => "1.1", details => 1, compiled => 1, node => "startNode", html => 1});
sub tree {
  my($class,  $arg) = @_;
  my($parent);
  my($details ) = $arg->{details};
  my($compiled) = $arg->{compiled};
  my($html)     = $arg->{html};
  my($version)  = $arg->{version};
  $version = "1.1" if (!$version);

  my($db);
  if ($compiled) {
    $db = NWS::Schema->database($version);
  } else {
    $db = NWS::Schema->_database($version);
  }

  my(%CHILD);

  foreach my $x (keys %$db) {
     $parent = $db->{$x}[0]{parent};  
     $parent = "ROOT" if (!$parent);
     $CHILD{$parent} .= "$x,";
  }
  foreach my $x (keys %CHILD) {
    my(@a) = split(/,/, $CHILD{$x});
    $CHILD{$x} = join(",", sort @a);
  }
  return &_visit($arg->{node} || "ROOT", \%CHILD, 0, $details, $db, $html);
}

sub _visit {
  my($node, $p, $level, $details, $db, $html) = @_;
  my($o);
  my($n1) = ($html) ? "<b>$node</b>": $node;
  $o =  "    " x $level . "$level $n1";
  $o .= " [type=$db->{$node}->[0]{type}, parent=$db->{$node}->[0]{parent}]";
  $o .= &_displayClass($node, $level,$db) if ($details && $node ne "ROOT");
  $o .= "\n";
  my(@childs) = split(/,/, $p->{$node});
  foreach my $child (@childs) {
     $o .= &_visit($child, $p, $level+1, $details, $db, $html);
  }
  return $o;
}

sub _displayClass {
  my($node, $level, $db) = @_;
  my($o, $x, $f2);
  my($n) = $db->{$node};
  $o .= " \n";
  foreach $x (@{$n->[1]}) {
     if ($x->[1] eq "R") {
       $f2 = $x->[2];
     } else {
       $f2 = $x->[2]? "K":"";
     }
     my($sign) = ($x->[6])? "-":"+";
     $o .= "    " x ($level+1) . "$sign$x->[0],$x->[1],$f2,$x->[3] \n";
  }
#  chop ($o);
  return $o;
}
          
1;


__END__

=head1 NAME

NWS::Schema.pm - Schema Database Class


=head3 SYNOPSIS

 use NWS::Schema;

 $db = NWS::Schema->_database();  # not compiled.
 $db = NWS::Schema->database();   # compiled.
 print NWS::Schema->verify();
 print NWS::Schema->tree();



=head3 DESCRIPTION

This module handles CIM Schemas. It is used primarily by CIM::Instance and can also
be used to verify and display a schema in indented fashion. When compiled, all attributes
from the parents are shown in each class and property that are repeated are merged 
properly.


=head3 CONSTRUCTOR

=over 4

=item database();

 Example:
   $db = NWS::Schema->database;

=back 4


=head3 METHODS

=over 4

=item verify();

verify all schemas. Will complain about missing parents and repeated classes.

   Example:
     NWS::Schema->verify();

=item tree();

Will display all classes as a tree structure from the ROOT.

   Example:
     print NWS::Schema->tree({details => 1, compiled => 1, node => "start", html => 1});

=back 4

=head3 SEE ALSO




=head3 AUTHOR

 Christian Cadieux (ccadieux@central.sun)



=head3 COPYRIGHT

Copyright (c) 2000 Sun Microsystems

=cut
