package PDM;
#  $Name:  $ 
#  $Id: PDM.pm,v 1.39 2001/09/28 16:00:21 ccadieux Exp $

use Storable;
use Data::Dumper;
use PDM::Thresholds;
use Carp;
use strict;
use Net::Ping;
use Report;
use Debug;
use NWS::Schema;
use PDM::Map;
use State;
use Catalog;

#
#  Interface:
#   new
#   serialize : called by the displatcher.
#   saveReport:
#   getDocSequence:
#   getLastReport:
#   saveReport:
#   saveMessage:
#   getCacheHandle;
#   inCache :  boolean
#      
use vars qw($Error %V @Reports_CB %NEW_REPORTS $OLD_REPORTS $CACHE @EVENT_DOCS $MAP %GLOBAL);

$Error      = 0;
%V          = ();
@Reports_CB = ();
@EVENT_DOCS = ();
%NEW_REPORTS = ();

sub error {
  return $Error;
}


#  my($pdm) = PDM->new({  dir => "$HOME/DATA",
#                      renv => $renv ,
#                   devices => $devices,
#                     hosts => $hosts,
#                    notifs => $notifs, http => 1});

sub new {
  my($class, $arg) = @_;
  my( $N, %T);
  my($pdm) = $arg;
  my($renv) = System->get_renv;

  $pdm->{dir} = System->get_home() . "/DATA" if (!$pdm->{dir});

  if ($arg->{http}) {
    $N = "$pdm->{dir}/CACHE";
    $CACHE = &_read($N);
 
  } else {
    $N = "$pdm->{dir}/OLD_REPORTS";
    $OLD_REPORTS = &_read($N, ":");

    $N = "$pdm->{dir}/CACHE";
    $CACHE = &_read($N);
    my($th) = PDM::Thresholds->new;
    $th->init;
    $pdm->{_th} = $th;
    State->init(); 
    NWS::Schema->init();
  }

    my $master = Util->findMaster();  # erase master catalog if config was pushed recently.
    if ($master) {
      my $config_data_age  = (stat(System->get_home() . "/DATA/rasagent.conf.push"))[9];

      if ($config_data_age > Catalog->catalogAge($master) ) {
         Debug->print2("Catalog is old: erasing");
         Catalog->clearCatalog($master);
      }
      TO->resetIncomplete();
    }

    my($devs) = System->get_configDevices();
    foreach my $d (@$devs) {
       next if (!Util->isMineToMonitor($d));
       $T{lc($d->{type})}{$d->{name}} = 1;
    }
    my($D) = System->get_home() . "/DATA/OLD_REPORTS";
    opendir(O1, $D) ;
    my(@a) = readdir(O1); closedir(O1);
    my($VAR1);

    foreach my $f (@a) {
       next if ($f !~ /.+:.+/);
       open(O1, "$D/$f");
       my(@b) = <O1>; close(O1);
       eval "@b";
       my($name) = $VAR1->{_id}{name};
       my($type) = lc($VAR1->{_id}{category});

       if ( (index("message,san,topo,host,mpxio", $type) < 0) &&
               $name && $type && !$T{$type}{$name}) {
           unlink("$D/$f");
           State->clearComponent($f);
           Debug->print2("Report $f not in config file, removed.");
       }
    }

  bless($pdm, "PDM");
  return $pdm;
}

sub threshold {
  my($pdm, @parms) = @_;
  return $pdm->{_th};
}


sub serialize {
  my($pdm) = @_;

# Each reports has it's own file in the OLD_REPORTS dir.
  $pdm->_write(\%NEW_REPORTS, "OLD_REPORTS");
#  Storable::store(\%NEW_REPORTS, "$pdm->{dir}/NEW_REPORTS");

  my($th) = $pdm->{_th};
  $th->serialize;
  $DB::single = 1;
  $pdm->_write($CACHE, "CACHE");
#  Storable::store($CACHE, "$pdm->{dir}/CACHE");
  Catalog->write();
  State->write();
}

sub _write {
  my($pdm, $x, $dir) = @_;
  my($k);
  my($f) = "$pdm->{dir}/$dir";
  mkdir $f, 0777 if (!-d $f);
  $Data::Dumper::Indent = 1;
  foreach $k (keys %$x) {
     open(O, ">$f/$k");
     print O Dumper($x->{$k});
     close(O);
  }
}

sub old_reports_exist {
  my($class) = @_;
  my($rep);
  opendir(O, System->get_home() . "/DATA/OLD_REPORTS");

  while ($rep = readdir(O)) {
     if ($rep =~ /.+:.+/) {
        return 1;
     }
  }
  return 0;
}


sub _read {
  my($dir, $pat) = @_;
  my(%O) = ();
  my($file, $v, $l1);
  my($f) = $dir;
  opendir(O, $f);
  my(@f) = readdir(O); closedir(O);

  foreach $file (@f) {
    next if (substr($file, 0,1) eq ".");
    next if ($pat && $file !~ /$pat/);
    open(O, "$f/$file");
    my($VAR1) = undef;
    my($l) = "";
    while ($l1 = <O>) { $l .= $l1}
    close(O);
    eval "$l";
    $O{$file} = $VAR1;
  }
  return \%O; 
    
}


sub readCache {
  my($class, $name) = @_;
  my($VAR1);
  my($file) = System->get_home() . "/DATA/CACHE/$name";
  if (open(O, $file)) {
    my(@L) = <O>; close(O);
    eval "@L";
    return $VAR1;
  } else {
    return undef;
  }
}

sub writeCache {
  my($class, $cache, $file) = @_;
  open(O, ">" . System->get_home() . "/DATA/CACHE/$file");
  print O Dumper($cache);
  close(O);
}

sub writeCacheObject {
  my($class, $file) = @_;
  open(O, ">" . System->get_home() . "/DATA/CACHE/$file");
  print O Dumper($CACHE->{$file});
  close(O);
}

use vars qw(%Stale);

sub setStaleCatalog {
  my($class ,$host) = @_;

  $Stale{$host} = 1;
}

sub getStaleCatalog {
  my($class) = @_;
  my(@a);
  foreach my $k (keys %Stale) {
    push(@a, $k);
  }
  return @a;
}


###########################################################
#  DEVICE STATE
###########################################################
$MAP = undef;

# static method

sub getDeviceStateMap {
  my($pdm, $section) = @_;
  my($l, %MAP);
  return $MAP->section($section) if ($MAP);

  my($F) = System->get_home() . "/System/Map";
  opendir(O, $F);
  my @L = readdir(O); closedir(O);

  foreach my $d (@L) {
     next if (substr($d,0,1) eq ".");
     open(O, "$F/$d");
     my $ix = index($d, ".");
     my $fname = substr($d, 0, $ix);
     my($sec) = "";
     while ($l = <O>) {
        chop($l);
        next if (substr($l,0,1) eq "#");
        if (substr($l,0,1) eq "[") {
           my $v = substr($l,1,-1);
           $ix = index($v, ".");
           $v = substr($v,$ix+1) if ($ix > 0);
           $sec = "$fname.$v";
        } else {
           my(@a) = split(/\s*\=\s*/, $l);
           next if ($#a < 1);
           $MAP{$sec}{$a[0]} = $a[1];
        }
     }
     close(O);
  }
  $MAP = PDM::Map->new(\%MAP);
  return $MAP->section($section);
}


###########################################################
#  REPORTS STUFF
###########################################################


sub getReportList {
  my($pdm) = @_;
  my(@list);
  foreach my $k (keys %NEW_REPORTS) {
      push(@list, $k);
  }
  foreach my $k (keys %$OLD_REPORTS) {
      if (!$NEW_REPORTS{$k}) {
         push(@list, $k);
      }
  }
  return \@list;
}


sub getReport {
  my($pdm, $key) = @_;
  return $NEW_REPORTS{$key} || $OLD_REPORTS->{$key};
}

sub getLastReport {
  my($pdm, $key) = @_;
  return  $OLD_REPORTS->{$key};

}

#  name, category, ip, key_type/key_value, report_type, report_value => {};
# Save a new report.
# $arg = {skip_health => 1}

sub _saveReport {
  my($pdm, $report) = @_;

# Only update reports if you got a good one.

  if ($report->status eq Report->STATUS_OK) {
      $NEW_REPORTS{$report->fileKey} = $report ;
  } else {  # update report
      $report->updateStatus(); # change the status of the last good report.
  }
  return;
}

sub saveReport {
  my($pdm,  $report, $arg) = @_;
  my($i, $hm, $filter, $cb, $rcb);
  $DB::single = 1;

  if ($arg->{skip_health}) {
    $pdm->_saveReport($report);
    return;
  }

# read all callback, check the filter, call if good.

  for ($i=0; $i <= $#Reports_CB; $i++) {  
      $rcb    = $Reports_CB[$i];
      $hm     = $rcb->[0];
      $cb     = $rcb->[1];
      $filter = $rcb->[2];
      next if ($filter->{INITIAL} || $filter->{FINAL});
      next if ($filter->{category} && $filter->{category} ne $report->category);
      next if ($filter->{report_type} && $filter->{report_type} ne $report->get_report_type);
      $hm->$cb($report);  # call callback with this report.
  }
  $pdm->_saveReport($report);

}

sub initialHealth {
  my($pdm) = @_;
  my($i, $rcb, $hm, $cb, $filter);

  for ($i=0; $i <= $#Reports_CB; $i++) {  
      $rcb    = $Reports_CB[$i];
      $hm     = $rcb->[0];
      $cb     = $rcb->[1];
      $filter = $rcb->[2];
      if ($filter->{INITIAL}) {
         $hm->$cb(undef) ;
      }
  }

}

sub finalHealth {
  my($pdm) = @_;
  my($i, $rcb, $hm, $cb, $filter);

  for ($i=0; $i <= $#Reports_CB; $i++) {  
      $rcb    = $Reports_CB[$i];
      $hm     = $rcb->[0];
      $cb     = $rcb->[1];
      $filter = $rcb->[2];
      if ($filter->{FINAL}) {
         $hm->$cb(undef);
      }
  }

}


###########################################################
#  STORE & FORWARD subs
###########################################################

# StoreAndForward
# Used by shuttle and the http to 
#  $pdm->SAFwrite({dir => "Shuttle", keep => $days_to_keep}) ;

sub SAFwrite {
  my($pdm , $arg) = @_;
  my($dir) = System->get_home . "/DATA/";
  my($f);
  if ($arg->{dir}) {
    $dir .= $arg->{dir};
  } else {
    croak("SAF requires a 'dir argument");
  }
  my($keep) = $arg->{keep} || 10;
  my($dc) = int(time/60/60);
  my($file) = "$dir/$dc" ;
  my($old) = $dc - $keep * 24;
  opendir(O, $dir);
  while ($f = readdir(O)) {
     next if (substr($f,0,1) eq ".");
     if ($f < $old) {
        unlink "$dir/$f";
        Debug->print2("Removing old file $dir/$f");
     }
  }
  closedir(O);
  if (!open(O, ">>$file")) {
     Debug->err(CANNOT_WRITE => "to $file");
     return undef;
  }
  my($ed);
  my($mess) = $pdm->getMessages;
  Debug->print2("Saving $#$mess messages");
  foreach $ed (@$mess) {
     print O $ed->toC;
  }
  close(O);
  return 1;
}

#
#  $pdm->SAFread({ dir => "Shuttle" });
#  Static method.
sub SAFread {
  my($pdm, $arg) = @_;
  my($f, $l, @E);
  my($dir) = System->get_home . "/DATA/";
  if ($arg->{dir}) {
    $dir .= $arg->{dir};
  } else {
    croak("SAF requires a 'dir argument");
  }
  my($last) = int(time/60/60) - 1;  # last hour
  opendir(O, $dir);
  my($ed);
  while ($f = readdir(O)) {
     next if (substr($f,0,1) eq ".");
     if ($f <= $last) {
        open(O, "$dir/$f");
        my($r);
        while ($l = <O>) {
           if (substr($l,0,2) eq "-E") {
              $ed =  Message->fromC($r);
              $ed->fileNo($f);
              push(@E, $ed);
              $r = "";
           } else {
              $r .= $l;
           }
        }
     }
  }
  closedir(O);
  return \@E; # a list of Messages
}

###########################################################
#  EVENT STUFF
###########################################################
# 
# Executed on the slave, appends to the SLAVE_DATA file all new events found.
#
sub storeMessages {
  my($pdm) = @_;
  my($x, $ed);

  if (open(O, ">>" . System->get_home() . "/DATA/SLAVE_DATA")) {
    for ($x = 0; $x <= $#EVENT_DOCS; $x++) {
       $ed = $EVENT_DOCS[$x];    
       print O $ed->toC;
    }
    close(O);
  } else {
    Debug->err(CANNOT_READ => "SLAVE_DATA $!");
  }
}
#
# store toString as html file
#
sub docMessage {
  my($pdm, $edoc) = @_;
  my($etype, $desc);
  my($ev) = $edoc->instances(0);
  my($cat) = $edoc->id('category');
  if ($edoc->type() eq 'E') {
    $etype = $ev->propertyByName('EventType')->value;
    $desc = $ev->propertyByName('Caption')->value;
    $etype .= ".$desc" if ($desc);
  } else {
    $etype = $ev->className();
    $desc = $ev->propertyByName('AlertType')->value;
    $etype .= ".$desc" if ($desc);
  }
  my($renv) = System->get_renv;
  my $F = "/tmp/$etype.toc";
  print "Saving toc to $F \n";
  open(O, ">$F");
  print O $edoc->toC;
  close(O);
}

sub saveMessage {
  my($pdm, $edoc, $fromSlave) = @_;
  my($xx);
  if (!$edoc->instances(0)->type()) {
     Debug->err(CIM_ERR => "Event has no type");
     return;
  }
  croak("Invalid type for event: " . ref($edoc)) if (ref($edoc) ne "Message");
  if (Debug->level() > 2) {
    $pdm->docMessage($edoc) 
  }
  if (!$fromSlave) {
    if ($edoc->type eq "E" && !$fromSlave) {  # log only local events
       my($ev) = $edoc->instances(0);
       my($time) = $ev->value("EventTime");
       my($data) = $time . "\t" . 
              $ev->value("EventType") . "\t" .
              $ev->value("Description") . "\t" . $edoc->id('deviceName') ;
       if (substr($data,-1) eq ":") {  # get more
         my @d1 = split(/\n/, $ev->value("Data"));
         for ($xx = 0; $xx < $#d1; $xx++) {
           $data .= "L$d1[$xx]\n";
         }
       }
       $pdm->writeTruncate(System->get_home() . "/DATA/Events.log", $data);

    } elsif ($edoc->type eq "A") {
       my($ev) = $edoc->instances(0);
       my($time) = $ev->value("EventTime");
       my($sev) = $ev->value("Severity");
       my($data) = $time . "\t" . 
              $ev->value("AlertType") . "\t" .
              $ev->value("Description") . "\t" . "Sev:$sev" . "\t" . 
              $edoc->id('deviceName');

       $pdm->writeTruncate(System->get_home() . "/DATA/Alerts.log", $data);
    }
  }
  
  Debug->print1("==> Saving Message: " . $edoc->instances(0)->className . ":" .
                 $edoc->instances(0)->propertyByName("Description")->value);
  push(@EVENT_DOCS, $edoc);
}


sub writeTruncate {
  my($pdm, $file, $data) = @_;
  my($chmod);
  if ($data) {
     my($size) = (stat($file))[7];
     if ($size > 1 * 1000000) {
        rename $file, $file . ".2";
     }
     $chmod = 1 if (!-f $file);
     if (open(O,">>$file")) {
       print O "$data\n";
       close(O);
     } else {
       Debug->err(CANNOT_WRITE => $file);
     }
     chmod 0666, $file if ($chmod);
  }
}


#  Request event-docs from the PDM, executed by providers.
#  $edocs = $pdm->getMessages({ ClassName => "className" });
#  $edocs = $pdm->getMessages({ ClassName => "className" });
#  $edocs = $pdm->getMessages({ ClassName => [<set of classes>] });
#  foreach $ed (@$edocs) {
#    ...
#
sub getMessages {
  my($pdm, $arg) = @_;
  my($x, $ed);
  my(@found);
  return \@EVENT_DOCS if (!$arg->{ClassName});

  for ($x=0; $x <= $#EVENT_DOCS; $x++) {
     $ed = $EVENT_DOCS[$x];
     next if ($arg->{ClassName} && 
             ! &set_equals($ed->instances->[0]->className, $arg->{ClassName}));
     push(@found, $ed);
  }
  return \@found;
} 

sub getEventCount {
  my($pdm) = @_;
  return $#EVENT_DOCS + 1;
}

#
# Reports_CB = [health_monitor, $function, $filter]
# filter can be on category, report_type or 'INITIAL'
# INITIAL : will execute automatically at the request of the dispatcher.
#

sub reportRequest {
  my($pdm, $hm, $callback, $filter) = @_;

  push(@Reports_CB, [$hm, $callback, $filter]);

}



%GLOBAL= ();

sub getGlobal {
  my($pdm, $key) = @_;
  my($obj) = {};

  if (!defined($GLOBAL{$key})) {
    $GLOBAL{$key} = $obj;
    return $obj;
  } else {
    return $GLOBAL{$key};
  }
}

  
sub dumpCache {
  my($pdm) = @_;
  $CACHE = undef;
}

# always returns a handle to an object or a new empty object.
# static method;
#
sub getCacheHandle {
  my($pdm, $key) = @_;
  my($obj) = {};

  if (!$CACHE) {
    my($D) = System->get_home() . "/DATA/CACHE";
    $CACHE = &_read($D);
  }

  if (!defined($CACHE->{$key})) {
    $CACHE->{$key} = $obj;
    return $obj;
  } else {
    return $CACHE->{$key};
  }
}

sub inCache {
  my($pdm, $key) = @_;
  if ($CACHE->{$key}) {
    return 1;
  } else {
    return 0;
  }
}

###########################################################
#  Timers
#  reset: reset the timer <T>, use 'reset' when events are transmitted.
#  get  : returns the #minutes since last reset. use 'get' to check the time since last
#  reset.
#       : if undefined, init the timer <T>, returns 0
###########################################################

sub resetTimer {
  my($pdm, $timer) = @_;
  croak("No timer defined") if (!$timer);
  my($h) = $pdm->getCacheHandle('timers');
  $h->{$timer} = int(time/60);
  return 0;
}


# returns the # of minutes of the timer.
sub getTimer {
  my($pdm, $timer) = @_;
  my($now) = int(time/60);
  croak("No timer defined") if (!$timer);
  my($h) = $pdm->getCacheHandle('timers');
  if ($h->{$timer}) {
    return $now - $h->{$timer}; # return elapsed minutes
  } else {
    $h->{$timer} = $now;
    return 0;                   # return 0 minutes, NEW timer
  }
}




# Answer the question: Has it been X days since  a certain event occured:
# - If the question was never asked, the event is initialized and
#     the answer is 'NEW'.
# - If it's been that long, answer 'YES', reset the event.
#   else answer undef
#   Uses the cache system to store the memory for all events.
# Usage:
#   if (PDM->isXdays('send_switch_counts')) {
#       'send_swith_stats in message...
#
sub isXdays {
 my($pdm, $event_name, $days) = @_;
 my(@a,$year, $yday, $day);

 $days = 1 if (!$days);
 my($mins) = $days * 60 * 24;
 my($t) = $pdm->getTimer($event_name);

 if ($t == 0) {
    return "NEW";
 } elsif ($t >= $mins) {
    Debug->print2("Timer '$event_name' is " . int($t/60) . " hours old");
    $pdm->resetTimer($event_name);
    return "YES";
 } else {
    return undef;
 }
}


sub isXhours {
 my($pdm, $event_name, $hours) = @_;
 my(@a,$year, $yday, $day);

 $hours = 1 if (!$hours);
 my($mins) = $hours * 60;

 my($t) = $pdm->getTimer($event_name);

 if ($t == 0) {
    return "NEW";
 } elsif ($t >= $mins) {
    Debug->print2("Timer '$event_name' is " . int($t/60) . " hours old");
    $pdm->resetTimer($event_name);
    return "YES";
 } else {
    return undef;
 }
}

sub isXreset {
 my($pdm, $event_name) = @_;

 my($h) = $pdm->getCacheHandle('timers');
 $h->{$event_name} = 0;
}



###########################################################
# MISC.
###########################################################
use vars qw($SEQ);

sub getEventSequence {
  my($pdm) = @_;
  my($s);
 
  $SEQ++;
  return System->hostid() . "." . time . ".$SEQ";
  
  my($file) = "EventSequence";
  if (open(O, System->get_home() . "/System/$file") ) {
    $s = <O>;
    $s++;
    close(O);
  } else {
    $s = 1;
  }
  open(O , ">" . System->get_home() . "/System/$file");
  print O $s;
  close(O);
  return System->hostid . "." . $s;
}



1;

__END__

=head1 NAME

PDM.pm - Persistent Data Manager


=head3 SYNOPSIS

 use PDM;

 $rep = PDM->new( {...} );


=head3 DESCRIPTION

This module include persistence functions.


=head3 CONSTRUCTOR

=over 1

=item new()


 $pdm => PDM->new( {dir => "cache dir", renv => , $devices => , hosts => , notifs => });

=back 1

=head3 METHODS

=over 4

=item serialize();

Serialize the PDM data stores, called by the dispatcher.

  $pdm->serialize();


=item saveReport();

Saves a report, called by instrumentation

=item getLastReport();

returns an old report.

=item initialHealth();

Execute initial health modules, created with the INITIAL flag.

=item finalHealth();

Execute final health modules, created with the FINAL flag.


=item SAFwrite();

Store&Forward write: writes to a separate file.

=item SAFread();

Store&Forward read: reads a set of files.


=item saveMessage();

Store one Message in memory.

=item storeMessages();

Serialize to the disk all eventDocs, called by slaves dispatchers.

=item getMessages();

returns a list of eventDocs available.

=item reportRequest();

called at the creation of a health monitor to register a callback.

=item getCacheHandle();

Returns a handle to an object given a cache key. If the object does not
exist, it will create an empty one and return a handle so that the caller
can still use the handle to store stuff. This will be serialized by the
dispatcher at the end of the rasagent run.

  Example:
    $h1 = PDM->getCacheHandle('savethis');
    $h1->{'here'} = "some stuff";
    $h1->{'there'} = "more stuff";
    $h1->{'count'}++;

    # This information will be available on the next run of rasagent.>

=item inCache();

returns TRUE if the key has an object in the cache.

=item getTimer($timer);

Will initialize a timer or return the number of minutes since the last reset
of the timer. By default, the timer is 'heartbeat' unless 'timer' is defined.

=item resetTimer($timer);

Will reset a timer to zero. Called manually and by getTimer when no timer is
found.

=item getEventSequence();

Get a unique event sequence.

=item getDocSequence();

Get a unique Document sequence for a set of events.

=back 4

=head3 SEE ALSO




=head3 AUTHOR

 Christian Cadieux (ccadieux@central.sun)



=head3 COPYRIGHT

Copyright (c) 2000 Sun Microsystems

=cut
