package PDM::Index;
#  $Name:  $ 
#  $Id: Index.pm,v 1.3 2001/03/01 16:17:15 ccadieux Exp $

use Debug;
use strict;
use vars qw($VALID);
use Data::Dumper;
use Carp;
sub revision {'$Revision: 1.3 $'}

$VALID = "wwn,serial,ip,ref,name";

#                  KEY       
#   $idx = Index->new('wwn=123');
#
sub new {
  my($idx, $key) = @_;
  my(@keys, $x, $val);
  my($per) =  PDM->getCacheHandle("PDMIndex");

  return $per->{$key} if ($per->{$key});
  my(@a) = split(/=/, $key);

  croak("Invalid key name : $a[0]") if (index($VALID, $a[0]) < 0);
  $val =  [$key, []];
  bless($val, "PDM::Index");
  $per->{$key} = $val;

  return $val;
}

sub dump {
  my($obj) = @_;
  my($per) =  PDM->getCacheHandle("PDMIndex");
  my($o);
  foreach my $k (keys %$per) {
     $o  .= "KEY=$k : ";
     foreach my $k1 (@{$per->{$k}->[1]}) {
         $o .= "$k1,";
     }
     chop($o);
     $o .= "\n";
  }
  return $o;
}

sub delete {
  my($obj, $key) = @_;

  my($per) =  PDM->getCacheHandle("PDMIndex");
  $key = $obj->key if ($obj ne "PDM::Index");

  if ($per->{$key}) {
     delete($per->{$key});
  } else {
     Debug->print2("Index::delete: '$key' does not exist, cannot delete");
     return undef;
  }
}

  

#
#  will replace if already there, else add.
#  $idx->addKey(@add_keys);                 # dynamic call
#   or
#  PDM::Index->addKey($primary, @add_keys); # static call
#
sub addKey {
  my($idx, @keys) = @_;
  my($h, $k, $obj, $key);

  if ($idx eq "PDM::Index") {
     my($per) = $idx->{_data};
     $key = shift(@keys);
     $obj = $per->{$key};
  } else {
     $obj = $idx;
  }

  if ($obj) {
    $h = $obj->toHash;
    foreach $k (@keys) {
       my(@a) = split(/=/, $k);
       if (index($VALID, $a[0]) < 0) {
          croak("Invalid key name : $a[0]");
       }
       Debug->print2("addKey:key exist: Key=$a[0], value=($a[1],$h->{$a[0]})") if ($h->{$a[0]});
       if ($a[1] eq "NULL") {
         delete($h->{$a[0]});
       } else {
         $h->{$a[0]} = $a[1];
       }
    }
    $obj->toArray($h);
    return $obj;
  } else {
    Debug->print2("PDM::Index::addKey: Cannot find key: $key");
    return undef;
  }
}


sub isPrimary {
  my($idx, $key) = @_;

  my($per) = PDM->getCacheHandle("PDMIndex");
  return ($per->{$key})? 1:0;
}

#
#  print $idx->find("wwn=123")->value("ip");
#  @found = $idx->find("ref=ref1");
#
#
sub find {
  my($i , $key) = @_;
  my($per) =  PDM->getCacheHandle("PDMIndex");
  my(@f, $k, $o, $ak);
  return $per->{$key} if ($per->{$key});

  foreach $k (keys %$per) {
     $o = $per->{$k}->[1];
     foreach $ak (@$o) {
        if ($ak eq $key) {
          if (wantarray) {
            push(@f, $per->{$k});
            last;
          } else {
            return $per->{$k};
          }
        }
     }
     return @f if (wantarray);
  }
  my($undef) =  [];
  bless($undef , "PDM::Index");
  return $undef;
}

sub toHash {
  my($o) = @_;
  my(%h, $k) ;
  foreach $k (@{$o->[1]}) {
      my(@a) = split(/=/, $k);
      $h{$a[0]} = $a[1];
  }
  return \%h;
}

sub toArray {
  my($o, $hash) = @_;
  my($k, @x);

  foreach $k (keys %$hash) {
     push(@x, "$k=$hash->{$k}");
  }
  $o->[1] = \@x;
}


#  print $o->value("wwn");
#
sub value {
  my($o, $key) = @_;
  my($k);
  my($v) = $o->[0];
  my(@a) = split(/=/, $v);
  return $a[1] if ($a[0] eq $key);
  
  $v = $o->[1];
  foreach $k (@$v) {
    @a = split(/=/, $k);
    if ($a[0] eq $key) {
       return $a[1];
    }
  }
  return undef;
}

sub key {
  my($o) = @_;
  return $o->[0];
}


1;


# 1;
#


__END__

=head1 NAME

PDM::Index.pm  - Used to store permanently information about device keys.


=head3 SYNOPSIS

 use PDM::Index;

 $idx = PDM::Index->new("wwn=123");
 print $idx->find("wwn=123")->value('name');
 print $idx->find("name=hello")->value('wwn');

 $idx->addKey("ref=ref123");
 PDM::Index->addKey("wwn=123", "ip=1.1.1.1");


=head3 DESCRIPTION

This module keeps track of primary and alternate keys for object (devices).
Object can be found using the primary of alternate keys. The primary key
is a hask key and must be unique.


=head3 CONSTRUCTOR

=over 4

=item new( $primary);

Create an index , will reuse an existing one if it exist.

 Example:
   $idx = PDM::Index->new("wwn=123");
   $idx->addKey('ip=123');
   print $idx->value("ip");

=back 4


=head3 METHODS

=over 4

=item addKey();

Allow to add keys to an existing index object. delete entries with value 'NULL'.

  Example:
    $idx->addKey("ref=ref1");
    $idx->addKey("ref=NULL"); # remove entry
    $idx->addKey("ref=");     # entry still there but empty.
    or
    PDM::Index->addKey("www=123", "ref=ref1");  # static, must specify the pri.

=item delete();

Delelte Index.
   
   Example:
     $idx->delete();
      or
     PDM::Index->delete('www=123'); # static, need primary key.


=item dump();

Dump all keys in a string for debugging.

  Example:
     print PDM::Index->dump();

=item delete();

Delete an Index.

  Example:
      $idx->delete();                  # delete itself.
      PDM::Index->delete('www=123');   # static version

=item isPrimary();

Is this a primary key.

  Example:
    if ($idx->isPrimary('wwn=123')) {...

=item find();

Returns an object (or list) with that primary or alt key.

  Example:
    $idx2 = $idx->find("wwn=123");  # will return one match.
    @list = $idx->find("ref=ref1");  # will return all matches
     or
    $idx2 = PDM::Index->find("wwn=123");  # works static or dynamic

=item value();

Returns the value of a key-name.

  Example:
   print $idx->value("ref");  # returns "ref1"

=item key();

returns the primary key.

  Example:
    print $idx->key();   # returns "wwn=123"

=back 4

=head3 SEE ALSO

=head3 AUTHOR

 Christian Cadieux (ccadieux@central.sun)



=head3 COPYRIGHT

Copyright (c) 2000 Sun Microsystems

=cut
  
  
