package Report;
#  $Name:  $ 
#  $Id: Report.pm,v 1.19 2001/07/29 22:00:01 ccadieux Exp $

use Carp;
use Util::Http;
use ReportContent;
use strict;
use Data::Dumper;

use vars qw($ERROR_LOG $WARNING_LOG $NOTICE_LOG  $DEVICE_ERROR $HOST_CONFIG $HOST_DISK_INQ $HOST_DATA_PING $SYSTEM_ERROR );

#
#  $rep = Report->new( { $type, $id, $value, $log_value , $status });
#  Interface:
#   $rep->name
#   $rep->type
#   $rep->category
#   $rep->content
#   $rep->id
#   $rep->created
#   $rep->key


sub STATUS_CANNOT_CONNECT  {'CC'};
sub STATUS_OK              {'OK'};

sub CAT_A3500FC          {'a3500fc'};
sub CAT_A1000            {'a1000'};
sub CAT_A2000            {'a2000'};
sub CAT_A3000            {'a3000'};
sub CAT_T3               {'t3'};
sub CAT_A5K              {'a5k'};
sub CAT_SWITCH           {'switch'};
sub CAT_BROCADE          {'brocade'};
sub CAT_HOST             {'host'};
sub CAT_ST4              {'st4'};
sub CAT_MESSAGE          {'message'};
sub CAT_T3MESSAGE        {'t3message'};
sub CAT_SAN              {'san'};
sub CAT_TOPO             {'topo'};
sub CAT_MPXIO            {'mpxio'};
sub CAT_TAPE             {'tape'};
sub CAT_WHAT             {'what'};
sub CAT_D2               {'d2'};


$ERROR_LOG       = 'L1';
$WARNING_LOG     = 'L2';
$NOTICE_LOG      = 'L3';
$DEVICE_ERROR    = 'x';
$HOST_CONFIG     = 'y';
$HOST_DISK_INQ   = 'h';
$HOST_DATA_PING  = 'p';
$SYSTEM_ERROR    = 'e';

#$id = { accessName => $device->{ip},
#        deviceName => $wwn,
#        name        => $device->{name},
#        category    => Report::CAT_T300,
#        ip          => $device->{ip},
#      };



sub new {
  my($class, $id, $report, $log, $error) = @_;

  my(%id2);
  $id2{_id}         = $id;
  croak("Id need accessName and deviceName") 
      if (!exists($id->{accessName}) || !exists($id->{deviceName}));
  $id2{_created}    = Util->get_today;
  $id2{_value}      = $report || {};
  $id2{_log}        = $log;
  $id2{_status}     = $error || 'OK';

  my($r) = \%id2;
  bless($r, "Report");
  return $r;
  
}



# 
# PUSH ReportsList to Master
#
sub pushReportsList {
  my($class) = @_;
  my($url, $err, $ans, $list);
  my($master) = Util->findMaster();
  my($renv) = System->get_renv;

  if ($master) { # you are a slave
    my($hostname) = $renv->{hostname};
    if (!$class->isSameReports() ) {
      $url =  Util->makeUrl($master, "?PUT=saveF&file=REMOTE_LISTS/$hostname");
      $list = Report->reportList();
      ($err, $ans)  = Util::Http->post($url, $list, 20);
      if ($err) {
        Debug->print2("Cannot push reportList: $err");
      }
    }
  }
}

sub isSameReports {
  my($class) = @_;
  my($new, $old);
  my($r);
  foreach $r (sort keys %PDM::NEW_REPORTS) {
     $new .= "$r,";
  }
  foreach $r (sort keys %PDM::OLD_REPORTS) {
     $old .= "$r,";
  }
  return ($new eq $old);
}




#   $r = Report->fromFile({ file => 'file.txt'});
#
sub fromFile {
  my($class, $arg) = @_;
  my($report) = {};
  my(%r, $l);
  croak("Missing 'file' argument!") if (!$arg->{file});
  if (!open(O, "$arg->{file}")) {
     croak("Cannot open $arg->{file}: $!");
  }
  while ($l = <O>) {
     chop($l);
     next if (substr($l,0,1) eq "#");
     my(@a) = split(/ *= */, $l);
     if ($l =~ /=/) {
        $r{$a[0]} = $a[1];
     }
  }
  close(O);
  $report->{_id} = "not defined";
  $report->{_value} = \%r;
  bless($report, "Report");
  return $report;
}

#  filename=date=accessName=name

sub reportList {
  my($report) = @_;
  my($VAR1);
  my($D) = System->get_home() . "/DATA/OLD_REPORTS";
  opendir(O, $D);
  my(@dirs) = readdir(O); closedir(O);
  my($out);

  foreach my $d (@dirs) {
     my @bb = split(/:/, $d);
     next if ($#bb != 1 || (index("host,san,topo", $bb[0]) >= 0));
     open(O, "$D/$d");
     my(@a) = <O>; close(O);
     eval "@a";
     if ($VAR1) {
       my($access) = $VAR1->{_id}{accessName};
       my($name)   = $VAR1->{_id}{name};
       my($created) = $VAR1->{_created};
       $out .= "$d=$created=$access=$name\n";
     }
  }
  return $out;
}

#  read report from a file
#  $rep1 = Report->read("a5k:12321321");

sub read {
  my($class, $file) = @_;
  my($VAR1);
  my($D) = System->get_home() . "/DATA/OLD_REPORTS";

  if (open(O, "$D/$file")) {
    my(@a) = <O>; close(O);
    eval "@a";
    return $VAR1;
  } else {
    return undef;
  }
}

#
# looks in OLD_REPORTS and REMOTE_LISTS for the host of a report.
#
sub findReportHost {
  my($class, $key) = @_;
  my($l);
  if (-f System->get_home() . "/DATA/OLD_REPORTS/$key") {
     return "";
  }
 
  my $D =  System->get_home() . "/DATA/REMOTE_LISTS";

  opendir(O, $D); my @files = readdir(O); closedir(O);
  foreach my $f (@files) {
    next if (substr($f,0,1) eq ".");
    open(O, "$D/$f");
    while ($l = <O>) {
      chop($l);
      my @a = split(/=/, $l);
      if ($a[0] eq $key) {
         close(O);
         return $f;
      }
    }
    close(O);
  }
  return undef;
}


sub readAllReports {
  my($report) = @_;
  my($D) = System->get_home() . "/DATA/OLD_REPORTS";
  opendir(O, $D);
  my(@dirs) = readdir(O); closedir(O);
  my($out);
  my(@reps);
  foreach my $d (@dirs) {
     my($VAR1);
     next if ($d !~ /\:/);
     open(O, "$D/$d");
     my(@a) = <O>; close(O);
     eval "@a";
     push(@reps, $VAR1);
  }
  return \@reps;
}
  


sub readReport {
  my($report, $key) = @_;
  my($VAR1);
  $key = $report->fileKey() if (!$key);

  open(O, System->get_home() . "/DATA/OLD_REPORTS/$key");
  my(@a) = <O>; close(O);
  eval "@a";
  return $VAR1;
}

# static
# use host to call remote machine
# use fileName, wwn or enc-name in this order

#  Report->readNetReport({ host     => 'ccdev',  
#                           timeout => 20,
#                          file     => 'asdsad',
#                          wwn      => '123213',
#                          name     => 'diag189'}
#                        );

sub readNetReport {
  my($report, $arg) = @_;
  my($host) = $arg->{host};
  my($data, $q);
  my($to) = $arg->{timeout} || $main::TO;
  my($CD) = System->get_home() . "/DATA/remote";
  my($url);

  if (!$arg->{name} && !$arg->{file} && !$arg->{wwn}) {
     return "ERR No argument (name, wwn, filename) ";
  }
  if ($host) {
      if ($arg->{file}) {
        $url = Util->makeUrl($host, "?GET=FILER&file=OLD_REPORTS/$arg->{file}");
      } else {
        $url = Util->makeUrl($host,
         "?GET=Report::instr_report&file=$arg->{file}&name=$arg->{name}&wwn=$arg->{wwn}");
      }
      $data = Util::Http->get($url, $to);
      if ($arg->{file}) {
        my($cache) = $CD . "/$arg->{file}";
        mkdir $CD, 0666 if (!-d $CD);
        open(O, ">$cache");
        print O $data; close(O);
      }
  } else {
      $q->{GET} = undef;
      $q->{file} = $arg->{file};
      $q->{wwn} = $arg->{wwn};
      $q->{name} = $arg->{name};
      $data = &get_instr_report($q);
  }

  if (!$data) {
    return ("Cannot retrieve $arg->{name} $arg->{file} from $host ($Util::Http::ERROR)<br>",undef)
  }
  if (substr($data,0,3) =~ "ER") {
    return ($data, undef);
  }
  my($VAR1);
  eval $data;

  if (!$VAR1) {
    return ("$arg->{name} $arg->{file} not found on $host!<br>", undef);
  }
  if ($host && !$arg->{file}) {
    my($id) = $VAR1->{_id};
    my($cache) = "$CD/" . $id->{category} . ":" . $id->{deviceName};
    mkdir $CD, 0666 if (!-d $CD);
    open(O, ">$cache");
    print O $data; close(O);

  }
  return (undef, $VAR1);

}

# called locally or remotely 

sub get_instr_report {
  my($q) = @_;
  my($l);
  my($P) = System->get_home() . "/DATA/OLD_REPORTS";
  my($out);

  if ($q->{file}) {   #got a file name
    if (open(O, "$P/$q->{file}")) {
      while ($l = <O>) {
        $out .= $l;
      }
      close(O);
    } else {
      $out = "ERR File $q->{file} does not exist!";
    }
  } elsif ($q->{name} || $q->{wwn}) { # got an enclosure name
     my($wwn);
     if (!$q->{wwn}) {
        $wwn = PDM->name2wwn($q->{name});
     } else {
        $wwn = $q->{wwn};
     }
     $out = "ERR $q->{name}/$q->{wwn} not found";
     opendir(O, $P);
     my(@dirs) = readdir(O); closedir(O);
     foreach my $d (@dirs) {
        if ($d =~ /$wwn/) {
           $out = Util->readf("$P/$d");
           last;
        }
     }
  }

  if ($q->{GET}) {
    print $out;
  } else {
    return $out;
  }
}
  

sub writeReport {
  my($report, $r) = @_;
  $Data::Dumper::Indent = 1;
  my($key) = $report->fileKey();
  open(O, ">" . System->get_home() . "/DATA/OLD_REPORTS/$key");
  print O Dumper($r);
  close(O);
}

# used when communication is lost and no instr-report was generated
# will merge information from rasagent with last good instr-report.

sub updateStatus {
  my($report) = @_;
  
  my($r) = $report->readReport();
  if ($r) {
    $r->{_status} = $report->{_status};
    my($val) = $r->{_value};
    my($new)  = $report->{_value};
    foreach my $v (keys %$new) {    # update all information from the new report
       $val->{$v} = $new->{$v};
    }
    $report->writeReport($r);
  } else {
    Debug->err(CANNOT_READ => $report->fileKey());
  }
}


sub status {
  my($report) = @_;
  return $report->{_status};
}

  
sub accessName {
  my($report) = @_;
  return $report->{_id}{accessName};
}

sub deviceName {
  my($r) = @_;
  return  $r->{_id}{deviceName};
}

#
# used to store this report in the file-system

sub fileKey {
  my($r) = @_;
  return  lc($r->{_id}->{category}) . ":" . $r->{_id}{deviceName};
}

sub makeFileKey {
  my($r, $cat, $deviceName) = @_;
  return lc($cat) . ":$deviceName";
}
  

sub name {
  my($report) = @_;
  return $report->{_id}{name};
}


sub category {
  my($report) = @_;

  return $report->{_id}->{category};
}

sub value {
  my($report) = @_;
  return $report->{_value};
}

sub content {
  my($report) = @_;
  
  return ReportContent->new($report->{_value});
}

sub log {
  my($report) = @_;

  return $report->{_log};
}

sub id {
  my($r, $key) = @_;
  if ($key) {
     return $r->{_id}{$key};
  }
  return $r->{_id};
}

sub created {
  my($r) = @_;
  return $r->{_created};
}




                          
1;



# 1;
#


__END__

=head1 NAME

Report.pm - Class that contains instrumentation reports


=head3 SYNOPSIS

 use Report;

 $rep = Report->new({ ... });

 $rep = Report->fromFile({ ... });

 $content = Report->fromFile(..)->content();



=head3 DESCRIPTION

This module create reports from instrumention or from saved files.



=head3 CONSTRUCTOR

=over 4

=item new( <type>, <device_id>, <report_hash> );

 Example:
   $id = {
           acceName     => $dev_name,
           deviceName   => $dev_name,
           category     => Report::CAT_A3500FC,
           ip           => 'NOIP',
           key_type     => "ctrl_id",
           key_value    => $internal_name,
         };

   $report = Report->new($id, $sys_report);

=item fromFile('file.txt');

    Used for testing;

=back 4


=head3 METHODS

=over 4

=item status();

return the status of the Report.

=item name();

returns the name of the device report.

=item category();

returns the category of the report.

=item content();

returns a ReportContent object.

 Example:
  $content = Report->fromFile('x')->content;
  print $content->get('disk.u1d1.state');

=item log();

returns loglines from the message files.


=item created();

returns creation date of the report.

=item deviceKey();

returns category . ":" . name ;

=item key();

returns type . ":" . category . ":" . name ;


=back 4

=head3 SEE ALSO




=head3 AUTHOR

 Christian Cadieux (ccadieux@central.sun)



=head3 COPYRIGHT

Copyright (c) 2000 Sun Microsystems

=cut
