package Scheduler;

use Util;
use strict;
use Debug;
use Data::Dumper;
use System;
use Solaris::Procfs;
use Solaris::Procfs::Filesystem;

#

sub run {
  my($class, $host, $program, $opts, $to) = @_;
  my($pid, $rc);
  my $to = 30 if (!$to);

  my($renv) = System->get_renv();
  $host = $renv->{hostname} if (!$host);

  if ($host eq $renv->{hostname}) {
     $rc = &get_run({ program => $program, opts => $opts, to => $to});
  } else {
     my $url = Util->makeUrl($host, "?GET=Scheduler::run&program=$program&opts=$opts&HTTP=1");
     $rc = Util::Http->get($url, 10);
     my $CD = System->get_home() . "/DATA/Proc/$host";
     mkdir $CD, 0777 if (!-d $CD);
     unlink "$CD/list";
  }
  return $rc;
}

#
#  S_$$ : script that runs the command
#  O_$$ : output of the command
#  E_$$ : stderr of the command
#  P_$$ : start and stop line for the command
#
sub get_run {
  my($q) = @_;
  my ($pid);
  my $program = $q->{program};
  my $opts = $q->{opts};
  my($renv) = System->get_renv();
  my $host = $renv->{hostname};

  my $D = System->get_home() . "/DATA/Proc";

  if ( ($pid = fork()) == 0) {
      if(ref($program) eq "CODE") {
         &$program($opts); exit 0;     # Start perl subroutine
      } else {
         close(STDIN);
         close(STDOUT);
         open(O,">$D/P_$$");
          
         print O "$$\tSTART\t" . Util->get_today() . "\t$program\t$opts\t$host\n";
         close(O);
         open(O, ">$D/S_$$");
         print O "$program $opts > $D/O_$$ 2>$D/E_$$\n";
         my $format = "+\%Y-\%m-\%d %H:%M:%S";
         print O "if test -s \"$D/E_$$\"\nthen\n";
         print O " echo \"$$\tSTOP\t`/usr/bin/date \\\"$format\\\"`\tERR\" >> $D/P_$$\n";
         print O "else\n";
         print O " echo \"$$\tSTOP\t`/usr/bin/date \\\"$format\\\"`\" >> $D/P_$$\n";
         print O "fi\n";
         close(O);
         chmod 0777, "$D/S_$$";
         exec "$D/S_$$";               # Start shell process
         if ($!) {
            open(O, ">>$D/P_$$");
            print O "Error: $!\n";
            close(O);
         }
         exit 0;                       # In case something goes wrong
      }
  } elsif($pid > 0) {      # Parent:
      $SIG{CHLD} = 'IGNORE';
      
  } else {                           # Fork Error:
      Debug->print2("Cannot start process $program ");
      $pid = 0;
  }
  if ($q->{HTTP}) {
     print $pid;
  } else {
     return $pid;                      #   return OK
  }
}

sub delete {
  my($class, $pid, $host) = @_;
  my($renv) = System->get_renv();
  $host = $renv->{hostname} if (!$host);
  if  ($host eq $renv->{hostname}) {
     my $d = System->get_home() . "/DATA/Proc";
     unlink "$d/P_$pid";
     unlink "$d/E_$pid";
     unlink "$d/O_$pid";
     unlink "$d/S_$pid";
  } else {
     my $url = Util->makeUrl($host, "?GET=DELET&file=Proc/&filter=??$pid");
     my $rc = Util::Http->get($url, 10);
  }
  return "OK";
}

sub exist {
  my($class, $pid, $host) = @_;
  my $rc;
  my($renv) = System->get_renv();
  $host = $renv->{hostname} if (!$host);

  if  ($host eq $renv->{hostname}) {
      $rc = &get_exist({pid => $pid});
  } else {
     my $url = Util->makeUrl($host, "?GET=Scheduler::exist&pid=$pid&HTTP=1");
     $rc = Util::Http->get($url, 10);
  }
  return $rc;
}

sub get_exist {
  my($q) = @_;
  my $pid = $q->{pid};
  my $p = new Solaris::Procfs::Filesystem;
      
  my $rc = ($p->{$pid})? 1:0;

  if ($q->{HTTP}) {
     print $rc;
  } else {
     return $rc;
  }
}

#pid | command | arg | host | start | end | err

sub processList {
  my($class, $host) = @_;
  my($renv) = System->get_renv();
  my($rc, @out, $x);
  $host = $renv->{hostname} if (!$host);

  if ($host eq $renv->{hostname}) { # local list
     my($D) = System->get_home() . "/DATA/Proc";
     opendir(O, $D);
     my @d = readdir(O); closedir(O);
     for ($x=0; $x <= $#d; $x++) {
         if ($d[$x] =~ /^P_/) {
            open(O, "$D/$d[$x]");
            my @l = <O>; close(O);
            my @start = split(/\t/, $l[0]);
            my @end = split(/\t/, $l[1]);
            push(@out, "$start[0]\t$start[3]\t$start[4]\t$start[5]\t$start[2]\t$end[2]\t$end[3]");
         }
     }
     return \@out;

  } else {
     my $CD = System->get_home() . "/DATA/Proc/$host/list";
     if (-f $CD) {
        open(O, $CD);
        my $l;
        while ($l = <O>) {
           chop($l);
           push(@out, $l);
        }
        close(O);
        return \@out;
     }
     
     my $url = Util->makeUrl($host, "?GET=FILER&file=/Proc/&filter=P_");
     my $data = Util::Http->get($url, 15);

     my @s = split(/\n/, $data);
     my %P = ();
     my $done = 0;
     for ($x=0; $x <= $#s; $x++ ){
         my @l = split(/\t/, $s[$x]);
         if ($l[1] eq "START") {
            $P{$l[0]} = "$l[0]\t$l[3]\t$l[4]\t$l[5]\t$l[2]";
            $done++;
         } else {
            $P{$l[0]} .= "\t$l[2]\t$l[3]";
            $done--;
         }
     }

     foreach my $v (reverse sort keys %P) {
         push(@out, $P{$v});
     }
     if ($done == 0) {  # all processed are done
        open(O, ">$CD");
        print O join("\n", @out) . "\n";
        close(O);
     }
     return \@out;
  }
}
     

sub read {
  my($class, $host, $pid) = @_;
  my(@p1, @p2, @a, @out, @err, $rc, $x);

  my($renv) = System->get_renv();
  $host = $renv->{hostname} if (!$host);
  if ($host eq $renv->{hostname}) {  # local
     open(O, System->get_home() . "/DATA/Proc/P_$pid");
     @a = <O> ; close(O); 
     @p1 = split(/\t/, $a[0]);
     @p2 = split(/\t/, $a[1]);

     open(O, System->get_home() . "/DATA/Proc/O_$pid");
     @out = <O> ; close(O);

     open(O, System->get_home() . "/DATA/Proc/E_$pid");
     @err = <O> ; close(O);
  } else {
     my $url = Util->makeUrl($host, "?GET=FILER&file=Proc/&filter=??$pid");
     $rc = Util::Http->get($url, 10);
     my $in;
     @a = split(/\n/, $rc);
     for ($x=0; $x <= $#a; $x++) {
         if (substr($a[$x],0,8) eq "#FILE P_") {
            @p1 = split(/\t/, $a[$x+1]);
            @p2 = split(/\t/, $a[$x+2]);
         } elsif (substr($a[$x],0,8) eq "#FILE O_") {
            $in = "O";
         } elsif (substr($a[$x],0,8) eq "#FILE E_") {
            $in = "E";
         } elsif ($in eq "O") {
            push(@out, "$a[$x]\n");
         } elsif ($in eq "E") {
            push(@err, "$a[$x]\n");
         }
     }
  }
  my @pp = ($p1[0], $p1[3], $p1[4], $p1[5], $p1[2], $p2[2], $p2[3]);
  return (\@pp, \@out, \@err);
 
}

  


1;
