package State;

use Util;
use strict;
use System;
use Util::Http;
use Data::Dumper;
use Debug;

use vars qw($STATE $state_changed);

# read state file and push to master, must write first.
#
sub push {
  my($class, $master) = @_;

  $master = Util->findMaster;

  return if (!$master);
  return if (!$state_changed);

  my($renv) = System->get_renv();
  my($F) = System->get_home() . "/DATA/state/" . $renv->{hostname};

  my($url) = Util->makeUrl($master, "?PUT=saveA&file=state/$renv->{hostname}.push");

  open(O, $F);
  my(@a)  = <O>; close(O);
  Debug->print2("  State: push State to master");
  my($err, $ans)  = Util::Http->post($url, join("", @a), 20);
  if ($ans !~ /OK/) {
     Debug->err(HTTP_ERROR => "Failed to push state info: $err");
  }

}

#
# will merge from a list of hosts and returned a merged state object.
#
sub mergeC {
  my($class, $STATE, $key, $sev1, $date, $mess, $name) = @_;

  if (exists $STATE->{C}{$key}) {
     my($sev0) = $STATE->{C}{$key}[0];

     if ($sev1 == 0 || $sev1 > $sev0) {
        $STATE->{C}{$key} = [$sev1, "$date $mess", $name];
     }
  } else {
     $STATE->{C}{$key} = [$sev1, "$date $mess", $name];
  }
}

sub mergeL {
  my($class, $STATE, $host, $key, $sev1, $date, $desc, $pc, $ra) = @_;

  if (exists $STATE->{L}{$key}) {
     my($o) = $STATE->{L}{$key};

     $STATE->{L}{$key} = [$sev1, "$date $desc", [$pc, $ra], $o->[3] + 1, $host];
  } else {
     $STATE->{L}{$key} = [$sev1, "$date $desc", [$pc, $ra], 1, $host];
  }
}


sub init {
  my($class) = @_;
  my($master) = Util->findMaster;
  my($D)      = System->get_home() . "/DATA/state";
  my($renv)   = System->get_renv();
  my $host    = $renv->{hostname};

  if ($master) {
     unlink "$D/$host";
  }
}



sub linkLog {
  my($class, $host, $key) = @_;
  my($renv) = System->get_renv();
  my($q);
  if ($renv->{hostname} eq $host) {
     $q = {key => $key};
     return &get_linkLog($q);
  } else {
     print "accessing $host..<br>\n";
     my $url = Util->makeUrl($host, "?GET=State::linkLog&key=$key&HTML=1");
     my $data = Util::Http->get($url, 30);
     return $data;
  }
}

sub get_linkLog {
  my($q) = @_;
  my($key)  = $q->{key};
  my($out, $l, $x);
  my(@A);
  my $MAX = 3;
  open(O, System->get_home() . "/DATA/LinkStateLog");
  my $in = 0;
  my $cnt = -1;
  while ($l = <O>) {
    chop($l);
    if (substr($l,0,1) eq "L") {
      my @a = split(/\t/, $l);
      $in = ($a[1] eq $key);
      if ($in) {
         my $ii = index($a[4], ")");
         $cnt++;
         $cnt = 0 if ($cnt > $MAX);
         if ($ii > 0) {
           $A[$cnt] = "<b>$a[3]</b> " . substr($a[4],0,$ii+1) . "\n";
         } else {
           $A[$cnt] = "<b>$a[3]</b> $a[4]\n";
         }
      }
    } elsif ($in) {
      $A[$cnt] .= "$l\n";
    }
  }
  for ($x = $cnt; $x >= 0; $x--) {
    $out .= $A[$x] . "\n";
  }
  for ($x = $MAX ; $x > $cnt; $x--) {
    $out .= $A[$x] . "\n";
  }

  if ($q->{HTML}) {
     print $out;
  } else {
     return $out;
  }
}



#
#  read state for a master/slave host
#
sub read {
  my($class, $host) = @_;
  my($l);
  my($D) = System->get_home() . "/DATA/state";
  my($master) = Util->findMaster;
  if ($master) {
     $STATE = {};
     return $STATE;  
  }

# MASTER: merge all state files into one, the master's 

  my($renv) = System->get_renv();
  $host = $renv->{hostname};
  my($VAR1);
  open(O, "$D/$host");
  my(@a) = <O>; close(O);
  eval "@a";
  $STATE = $VAR1 || {};

  opendir(O, $D);
  my @states = readdir(O); close(O);

  my $in = 0;
  my ($last);
  foreach my $st (@states) {
    next if ($st !~ /\.push$/);
    rename "$D/$st", "$D/$st.2";
    open(O, "$D/$st.2");
    my $host = substr($st, 0, -5);
    while ($l = <O>) {
       chop($l);
       $in = 1;
       my(@x) = split(/\t/, $l);
       # "C\t$cat:$keyval:$comp\t$sev\t$date\t$mess\t$name\n";
       if ($x[0] eq "C") {
          $class->mergeC($STATE, $x[1], $x[2], $x[3], $x[4], $x[5]);

       } elsif ($x[0] eq "L") {
       # "L\t$key\t$sev\t$date\t$desc\t$pc\t$ra\n";
          $class->mergeL($STATE, $host, $x[1], $x[2], $x[3], $x[4], $x[5], $x[6]);

       } elsif ($x[0] eq "R" || $x[0] eq "W") {
       }
      
    }
    close(O);
  }

  $class->write() if ($in);
  bless($STATE, 'State');
  return $STATE;
}


sub clearComponent {
  my($class, $obj) = @_;

  if (!$STATE) {
     $class->read();
  }
  my $C = $STATE->{C};
  foreach my $k (keys %$C) {
    if (substr($k,0,length($obj)) eq $obj) {
       $STATE->{C}{$k}[0] = -2;
    }
  }
}



sub components {
  my($class) = @_;
 
  return $class->{C};
}

sub links {
  my($class) = @_;
 
  return $class->{L};
}

sub clear {
  my($class, $obj) = @_;

  $class->{C}{$obj}[0] = -2;
}

sub clearLink {
  my($class, $obj) = @_;

  if (defined($class->{L}{$obj})) {
    $class->{L}{$obj}[0] = -2;
    if (defined( $class->{L}{$obj}[3])) {
      $class->{L}{$obj}[3] = 0;
    }
  }
}

  

#
# only writes local state table.
#
sub write {
  my($class, $host) = @_;
  my($master) = Util->findMaster;
  return if ($master); 

  return if (!$STATE);
  my($renv) = System->get_renv();
  my($F) = System->get_home() . "/DATA/state/" . $renv->{hostname};
  open(O, ">$F");
  $Data::Dumper::Indent = 1;
  print O Dumper($STATE);
  close(O);
}


# this runs on the host that has data-path access.

# State->saveLinkState($type1, $key1, $type2, $key2,
#         Message::SEVERITY_WARNING, $desc, $reads, $writes, $pc, $ra);
# probable cause, recomm actions

sub saveLinkState {
  my($class, $cat1, $node1, $cat2, $node2, $sev, $desc, $reads, $writes,
     $pc, $ra) = @_;

  $cat1 = lc($cat1);
  $cat2 = lc($cat2);
  my($key) = "$node1|$node2";
  my($data, $x);

  my($master) = Util->findMaster;
  my($D)      = System->get_home() . "/DATA/state/";
  my($renv)   = System->get_renv();
  my $host    = $renv->{hostname};
  my $date    = Util->today();
  $desc       =~ s/[\|,]/ /g;
  $pc         =~ s/\n/||/g;
  $ra         =~ s/\n/||/g;
  $state_changed = 1;

# 
# SAVE LOCAL LINK LOG INFO
#
  my $stateLog = System->get_home() . "/DATA/LinkStateLog";
  open(W, ">>$stateLog");
  print W "L\t$node1|$node2\t$sev\t$date\t$desc\n";

  for ($x=0; $x <= $#$reads && $x < 10; $x++) {
    print W "R\t$reads->[$x]\n";
  }
  for ($x=0; $x <= $#$writes && $x < 10; $x++) {
    print W "W\t$writes->[$x]\n";
  }
  close(O);
  Debug->truncate($stateLog, 1);

#
# SLAVE
#
  if ($master) {
     open(O, ">>$D/$host");
    
     print O "L\t$key\t$sev\t$date\t$desc\t$pc\t$ra\n";
     close(O);
     return;
  }

#
# MASTER
#
  if (!$STATE) {
     $class->read();
  }
  my($sev0)  = $STATE->{L}{$key}[0];
  my($cnt)   = $STATE->{L}{$key}[3];


  $STATE->{L}{$key} =
     [$sev,"$date $desc", [$pc,$ra], $cnt+1, $host];  # 0=ok, 1=w, 2=err,3=down
}


# ->saveState($cat, $name, $keyval,
#         Message::SEVERITY_DOWN, "Lost Communication");
#            4=notpresent, 3=down, 2=error,1=warn,0=ok
#  this runs on the host that has instrumentation agent running.

sub saveState {
  my($class, $cat, $keyval, $comp, $name, $sev, $mess) = @_;

  $cat = lc($cat);
  $name = $keyval if (!$name);
  $comp = "e" if (!$comp);  # enclosure

  my($master) = Util->findMaster;
  my($D)      = System->get_home() . "/DATA/state";
  my($renv)   = System->get_renv();
  my $host    = $renv->{hostname};
  my $date    = Util->today();

  if ($master) { # a slave
     $state_changed = 1;
     open(O, ">>$D/$host");
     print O "C\t$cat:$keyval:$comp\t$sev\t$date\t$mess\t$name\n";
     close(O);
     return;
  }
     

  if (!$STATE) {
     $class->read();
  }
  my($sev0) = $STATE->{C}{"$cat:$keyval:$comp"}[0];

  if ($sev == 0 || $sev >= $sev0) {
    $state_changed = 1;
    $mess =~ s/[\|,]/ /g;
                                        # 0=ok, 1=w, 2=err,3=d own
    $STATE->{C}{"$cat:$keyval:$comp"} = [$sev,"$date $mess", $name];  
  }
  $comp = $comp;
}

sub isChanged {
  return $state_changed;
}



1;



