package TO;
use Util;
use System;
use PDM::ConfigFile;
use strict;
use Data::Dumper;
use FC;
use Catalog;
use Agent::SWITCHAgent;

use vars qw($NEWTOPO);

##################################################################
#
# TO->clearTopo();
# $to->get_NewTopo();                    # print topo data structure.
# $to = TO->readExistingTopo();
# $to = TO->readTopo();                 # will discover if ned be.
# print $to->toString;                  # print the whole topology
# $to->saveTopo();                      # save a topology in DATA/topo/<hostname>

# $list = $to->hostList();              # return a list of nodes.
# $list = $to->switchList();
# $list = $to->storageList();
# $list = $to->hubList();

# $list->[0]->name();                   # get the name of the node ('a5k:12321312');
# TO->nodeName($node);                  # get the name of a node (static);
# $node->name();                        # get the name of a node (dynamic)
# $node->type();                        # get the type of a node.

# $node = $to->nodeByName('a5k:123123'); # get the node for a specified key
# $node->toString();                    # print one node of the topology
# $portList  = $node->portInfo();       # info about each port
# $volList   = $node->volInfo();        # info about each volume (T3)
# $linkList  = $node->port();           # where does each port goes ('a5k:12312:0');
# $diskList  = $node->diskInfo();       # info about each disks
#
##################################################################


# hosts hbas start at 0
# Hosts->[0]{info}        = {hostname => ... }
# Hosts->[0]{port}[0]      = linkkey   Ex: s:0:1
# Hosts->[0]{portInfo}[0] = {physicalpath => ...}

# switch ports start at 1, 0 is empty;
# Switches->[0]{info}     = {name => , ip => ...}
# Switches->[0]{port}[1]  = linkKey
# Switches->[0]{portInfo}[1]  = { type => ,...}

# hosts port start at 0
# Storages->[0]{info}      = { type => "A/T", info => 'A|T|...'}
# Storages->[0]{port}[0]      =  linkKey   
# Storages->[0]{portInfo}[0] =  { port info ... }
# Storages->[0]{diskInfo}{f1,f2,f3,r1,r2,r3...} =  { disk info ... }
#
# Names   ->{'a:DPL1'} = 1;    means that a5000 with name DPL1 is at Storages[1]
# 

#
#  Hosts    0
#  Switches 1
#  Storages 2
#  Hubs     3
#  Names    4

sub push {
  my($class, $master) = @_;

  return if (!$master);
  return if (!$NEWTOPO);

  Debug->print2("  Topology: push new topo to master");

  my($renv) = System->get_renv();
  my($F) = System->get_home() . "/DATA/topo/" . $renv->{hostname};


  if (open(O, $F)) {
    my($url) = Util->makeUrl($master, "?PUT=saveF&file=topo/$renv->{hostname}");
    my(@a)  = <O>; close(O);

    my($err, $ans)  = Util::Http->post($url, "@a", 20);
    if ($ans !~ /OK/) {
      Debug->err(HTTP_ERROR => "failed to push topo info: $err");
    }
  } 

}

sub discmanPath {
  my($class) = @_;

  return "/opt/SUNWvtsst/bin/" . 
         ((System->bitMode() eq "64") ? "sparcv9/" : "") . "discman";
}

sub available {
  my($class) = @_;

  if (-f $class->discmanPath() ) {
     return 1;
  } 
  return 0 if (System->os_version() lt "5.8");

  my($err, $ans) = Util->run_command(System->get_home() . "/bin/install_discman","test",30);

  if (-f $class->discmanPath() ) {
     return 1;
  } 

  return 0;
}

# returns cached discman output if available
# called by rasagent locally

sub getDiscman {
  my($class, $host, $save) = @_;
  my($command, $com, @a);

  $command = $class->discmanPath();
  my($renv) = System->get_renv();

  $host = $renv->{hostname} if (!$host);
  my($file) = System->get_home() . "/DATA/discman/$host";
  if ($save && -r $file) {
     open(O, $file); 
     my(@x) = <O>; close(O);
     return  \@x;
  }
  return undef if (!$class->available() );

  my($err,$com, $stderr) = Util->run_command( $command, "discman.test", 300);
  Debug->print2("discman returned $#$com lines");
  if ($stderr) {
    Debug->err(DISCMAN => $stderr);
  }
  if (open(O, ">$file")) {
    print O join("\n", @$com);
    close(O);
  } else {
    Debug->err(TEXT => "$file: $!");
  }
  return $com;
}

sub clearDiscman {
  my($class) = @_;

  my($renv) = System->get_renv();
  my($file) = System->get_home() . "/DATA/discman/" . $renv->{hostname};
  unlink $file;
}

#
# return one or a list of host if it's a merge-topo

sub host2list {
  my($class, $host) = @_;

  if (substr($host,0,6) eq "MERGE-"){
    my($file1) = System->get_home() . "/DATA/topo_merge";
    open(O, $file1);
    my(@hlist, $l);
    while ($l = <O>) {
       chop($l);
       my(@c) = split(/\|/, $l);
       if ($c[0] eq substr($host,6)) {
          @hlist = split(/\t/, $c[2]);
          last;
       }
    }
    close(O);
    return @hlist;
  } else {
    return ($host);
  }
}

#
#  reads the current topology
#  if topo/hostname is missing, get a new topology and save it
#
#  host may be 'nscc-test or 'MERGE-san

sub readExistingTopo {
  my($class, $host) = @_;
  my(@lines, $VAR1);

  my $D    = System->get_home() . "/DATA/topo";
  my $renv = System->get_renv();

  if (!$host) {
    $host = $renv->{hostname} ;
  }
  if (open(O, "$D/$host")) {
    @lines = <O>; close(O);
    eval "@lines";
    if ($VAR1) {
      return $VAR1;
    }
  }
  return undef;
}

#
# runs locally, gets a new local or remote topo;
# 
sub getNewTopo {
  my($class, $host, $to) = @_;
  my $VAR1;
  my($renv) = System->get_renv();
  $to = 120 if (!$to);

  if (!$host) {
    $host = $renv->{hostname} ;
  }
  if ($host eq $renv->{hostname}) {
    TO->clearTopo();
    return TO->readTopo();
  }
  
  my $url  = Util->makeUrl($host, "?GET=TO::NewTopo&host=$host");
  my $data = Util::Http->get($url, $to);

  eval $data;
  if ($VAR1) {
     $VAR1->saveTopo($host);
     return $VAR1;
  }
}

# execute on the remote machine
sub get_NewTopo {
  my($q) = @_;
  
  TO->clearTopo();
  my $to = TO->readTopo();
    
  if ($to) {
    $Data::Dumper::Indent = 1;
    print Dumper($to);
  }
}


sub readTopo {
  my($class, $host, $save) = @_;
  my(@lines, $VAR1);
  my($D) = System->get_home() . "/DATA/topo";

  my($renv) = System->get_renv();
  if (!$host) {
    $host = $renv->{hostname} ;

  } elsif (substr($host,0,6) eq "MERGE-"){
    my(@hlist) = $class->host2list($host);

    my($age, $d1, $host0) ;
    foreach $host0 (@hlist) {
       $d1 = (stat("$D/$host0"))[9];
       $age = $d1 if ($d1 > $age);
    }
    $d1 = (stat("$D/$host"))[9];

    if ($age < $d1) {   # merged topo is still current
      if (open(O, "$D/$host")) {
        @lines = <O>; close(O);
        eval "@lines";
        if ($VAR1) {
          return $VAR1;
        }
      }
      Debug->print0("Cannot read $host topo");
      return;
    }
    
    close(O);
    my($topoM, $to1);
    foreach $host0 (@hlist) { # list of host from config-file
      if (open(O, "$D/$host0")) {
        @lines = <O>; close(O);
        eval "@lines";
        if (!$VAR1) {
           Debug->print0("Cannot read topo $host0");
           next;
        }
        $to1  = $VAR1;
        if ($topoM) {
           $class->merge($topoM, $to1);
        } else {
           $topoM = $to1;
        } 
      } else {
        Debug->print0("Cannot find topo $host0");
      }
    }
    
    $topoM->saveTopo($host) if ($topoM);
    return $topoM;
    
  }

#
# regular topo
#
  if (open(O, "$D/$host")) {
    @lines = <O>; close(O);
    eval "@lines";
    if ($VAR1) {
      return $VAR1;
    }
  }
  my($d) = $class->getDiscman($host, $save);
  $DB::single = 1;
  return undef if ($#$d == -1);

  my($topo, $trace) = $class->parseDiscman($d);
  return undef if (!$topo);
  if ($renv->{showoob} eq "Y") {
     $topo->addOOB();
  }
  $topo->saveTopo($host);

  return $topo;
}

sub merge0 {
  my($s0, $s1, $desc) = @_;
  my($x, $err);
  for ($x=0; $x <= $#$s1; $x++) {
    next if (!$s1->[$x]);
    if ($s0->[$x]) {
       $err .= "$desc $x already exist, not merged<br>";
    } else {
       $s0->[$x] = $s1->[$x];
    }
  }
  return $err;
}

sub topoList {
  my($class) = @_;
  
  my($l, @list);
  opendir(O, System->get_home() . "/DATA/topo");
  while ($l = readdir(O)) {
    next if (substr($l,0,1) eq ".");
    push(@list,$l);
  }
  return \@list;
}


sub merge {
  my($class, $TO, $to) = @_;
  my $err;
  my($HO) = $TO->[0];
  my($SW) = $TO->[1];
  my($HU) = $TO->[3];
  my($ST) = $TO->[2];

  my($ho) = $to->[0];
  my($sw) = $to->[1];
  my($hu) = $to->[3];
  my($st) = $to->[2];

  foreach my $key (keys %$ho) {  # new hosts
     $HO->{$key} = $ho->{$key};
  }

  foreach my $key (keys %$sw) {
     if (exists($SW->{$key}))  {
       $err .= &merge0($SW->{$key}{dport},    $sw->{$key}{dport},    "$key:dport");
       $err .= &merge0($SW->{$key}{portInfo}, $sw->{$key}{portInfo}, "$key:portInfo");
       $err .= &merge0($SW->{$key}{port},     $sw->{$key}{port},     "$key:port");
     } else {
       $SW->{$key} = $sw->{$key};
     }
  }

  foreach my $key (keys %$st) {
     if (exists($ST->{$key}))  {
       $err .= &merge0($ST->{$key}{portInfo}, $st->{$key}{portInfo}, "$key:portInfo");
       $err .= &merge0($ST->{$key}{port},     $st->{$key}{port},     "$key:port");

     } else {
       $ST->{$key} = $st->{$key};
     }
  }

  foreach my $key (keys %$hu) {  # new hubs
     $HU->{$key} = $hu->{$key};
  }
}

sub diff {
  my($class, $to1, $to2) = @_;
  my($err, $err2, $err3, $d, $d2, $d3);

  ($err, $d) = $class->_diff($to1->[0], $to2->[0], "HOST");
  return "$err on $d" if ($err);

  ($err, $d) = $class->_diff($to1->[1], $to2->[1], "SWITCH");
  return "$err on $d" if ($err);

  ($err, $d) = $class->_diff($to1->[2], $to2->[2], "STORAGE");
  return "$err on $d" if ($err);
  return undef;
}

sub _diff {
  my($class, $e, $e2, $desc) = @_;
  my($x);
  my @size1 = (keys %$e);
  my @size2 = (keys %$e2);
  if ($#size1 != $#size2) {
     return ("SIZE_CHANGE", $desc);
  }

  foreach my $key (keys %$e) {    # each host, switch, storage
      my $p = $e->{$key};
      my $p2 = $e2->{$key};
#      my($type, $key0) = split(/\:/, $key);
      foreach my $section (keys %$p) { # each section (portInfo, info, port, dport ..
           my $pA = $p->{$section};
           my $pA2 = $p2->{$section};
           if ($section eq "info" && $pA->{name} ne $pA2->{name}) {
              return ("NAME_CHANGE", "$desc $key");

           } elsif (ref($pA) eq "ARRAY") {
              if ($#$pA != $#$pA2) {
                 return ("SIZE_CHANGE", "$desc $section");
              }
              for ($x=0; $x <= $#$pA; $x++) {
                  my $pB = $pA->[$x];
                  my $pB2 = $pA2->[$x];
                  if (ref($pB) eq "HASH") {
                    foreach my $value (keys %$pB) { # each value
                       if ($pB->{$value} ne $pB2->{$value}) {
                          return ("VALUE_CHANGE", "$desc:$key FIELD:$section:$x:$value");
                       }
                    }
                  } elsif ($pB ne $pB2) {
                    return ("VALUE_CHANGE", "$desc:$key FIELD:$section:$x");
                  }
              }
           } elsif (ref($pA) eq "HASH") {
              my @size1 = (keys %$pA);
              my @size2 = (keys %$pA2);
              if ($#size1 != $#size2) {
                  return ("SIZE_CHANGE","$desc:$key FIELD:$section");
              }
              foreach my $ael (keys %$pA) {
                  my $pB = $pA->{$ael};
                  my $pB2 = $pA2->{$ael};
                  if (ref($pB) eq "HASH") {
                    foreach my $value (keys %$pB) { # each value
                       if ($pB->{$value} ne $pB2->{$value}) {
                          return ("VALUE_CHANGE", "$desc:$key FIELD:$section:$ael:$value")
;
                       }
                    }

                  } elsif ($pA->{$ael} ne $pA2->{$ael}) {
                     return ("VALUE_CHANGE", "$desc:$key FIELD:$section:$ael");
                  }
              }
           }
       }
   }
   return (undef,$desc);
}


# instance method

sub saveTopo {  # save a topology
  my($Topo, $file) = @_;

  open(O, ">" .  System->get_home() . "/DATA/topo/$file"); 
  $Data::Dumper::Indent = 1;
  my($data) = Dumper($Topo);
  $data =~ s/^        /\t\t/gm;     
  print O $data;
  close(O);
}

# returns the date of the last topology generated

sub lastTopo {
  my($class, $host) = @_;
  my($renv) = System->get_renv();
  $host = $renv->{hostname} if (!$host);

  my($file) = System->get_home() . "/DATA/topo/" . $host;
  if (-f $file) {
     return Util->get_file_created($file);
  } else {
     return undef;
  }
}

sub topoAge {
  my($class, $host) = @_;
  my($renv) = System->get_renv();
  $host = $renv->{hostname} if (!$host);
  my $topo_file = System->get_home() . "/DATA/topo/$host";
  my $age = (stat($topo_file))[9];
  return $age;
}

sub clearTopo {
  my($class, $host) = @_;

  my($renv) = System->get_renv();
  $host = $renv->{hostname} if (!$host);
  $NEWTOPO = 1;
  my($file) = System->get_home() . "/DATA/topo/" . $host;
  unlink $file;
  FC->clearCache();
}

#
# if topo/name  is gone, reset the incomplete cache.
#
sub resetIncomplete {
  my($class) = @_;
  my($renv) = System->get_renv();
  my $host = $renv->{hostname};
  my($file) = System->get_home() . "/DATA/topo/" . $host;
  if (!-f $file) {
    my $h = PDM->getCacheHandle("incomplete_topo");
    $h->{try} = undef;
  }
}

sub setIncomplete {
  my($class) = @_;
  my $h = PDM->getCacheHandle("incomplete_topo");
  $h->{try} = 2;
}



sub toString {
  my($Topo) = @_;
  my($x);

  my($hosts) = $Topo->hostList();
  foreach my $ho (@{$Topo->hostList()}) {
     print $ho->toString();
     print "\n";
  }
  foreach my $sw (@{$Topo->switchList()}) {
     print $sw->toString();
     print "\n";
  }
  foreach my $st (@{$Topo->storageList()}) {
     print $st->toString();
     print "\n";
  }
}


#
# save XY coord of a topo, done by applet
#
sub put_topo_xy {
 my($q, $post_data) = @_;
 my($topoFile) = $q->{topo} || "data1";
 my($F) = System->get_home() . "/DATA/tmp/topoxy_$topoFile";
 open(O, ">>/tmp/log"); print O "$F\n"; close(O);

  if (open(O, ">$F")) {
    print O "$post_data\n";
    close(O);
    chmod 0666, $F;
    print "\Saved (" . length($post_data) . " bytes)\n";
  } else {
    print "\nERR: $!\n";
  }
}


sub addOOB {
 my($to) = @_;
 my($x, $p, $obj);
 my($switches) = $to->switchList();
 my($storages) = $to->storageList();

 for ($x=0; $x <= $#$switches; $x++) {
    my($sw) = $switches->[$x];
    my $pi = $sw->{port};
    my $V = SWITCHAgent->getNodesInfo($sw->{info}{sw_ipAddr},20);

    for ($p=1; $p <= $#$pi; $p++) {
       next if ($pi->[$p]);  # already connected
       next if ($p > $#$V);
       my $wwn = $V->[$p][0];
       next if (!$wwn);
       my($ci, $port) = Catalog->deviceRead('t3',Catalog::WWN, $wwn);
       next if (!$ci);

       my $key = $ci->[Catalog::Key];  # connect this t3 to the switch
       my $t3 = $to->nodeByName("t3:$key");
       if (!$t3) {  
          $t3 = $to->newT3($wwn, $ci );
       }
       # link port of switch to t3
       $sw->{port}[$p] = "t3:$key:$port";
       $t3->{port}[$port-1] = "switch:" . TO->nodeName($sw) . ":$p";
    }
 }

}

sub newT3 {
  my($to, $wwn, $ci) = @_;
  my $t3;
  my $storages  =  $to->[2];
  my $key = $ci->[Catalog::Key];

  my $obj = {};
  $obj->{info}{type}    = 't3';
  $obj->{info}{name}    = "t3:$key";
  $obj->{info}{BoxName} = $ci->[Catalog::Name] || $wwn;
  $obj->{info}{host}    = System->hostname();
  $obj->{info}{PortWWN} = $wwn;
  $obj->{info}{NodeWWN} = $wwn;
  $obj->{info}{PG}      =  ($ci->[Catalog::WWN2])? 1:0;
  bless($obj, 'TO_Storage');
  $t3 = $storages->{"t3:$key"} = $obj;
  return $t3;
}

use Agent::SWITCHAgent;

sub newSW {
  my($to, $ci, $dev) = @_;
  my($sw,$x);
  my $switches =  $to->[1];
  my $key = $ci->[Catalog::Key];
  my($err, $version) = SWITCHAgent->chassisInfo($dev);
  "@$version" =~ /Ports: (\d+)/;
  my $ports = $1;

  my $obj = {};
  $obj->{info}{type}    = 'switch';
  $obj->{info}{name}    = "switch:$key";
  $obj->{info}{ProductID}  = "Switch";
  $obj->{info}{sw_ipAddr}  = $ci->[Catalog::IP];
  $obj->{info}{BoxName} = $ci->[Catalog::Name];
  $obj->{info}{host}    = System->hostname();
  for ($x=1; $x <= $ports; $x++) {
    $obj->{port}[$x] = 1;
  }
  bless($obj, 'TO_Switch');
  $sw = $switches->{"switch:$key"} = $obj;
  return $sw;
}



sub parseDiscman {
  my($class, $com) = @_;

  my(%hosts, %hbas, %devices);
  my($Hosts)  = {};
  my($Switches) = {};
  my($Storages) = {};
  my($Hubs) = {};
  my($Syns) = {};
  my($SW)   = {};
  my($ST)   = {};

  my($Topo) = [$Hosts, $Switches, $Storages, $Hubs , $Syns];
  bless($Topo, 'TO');

  my($out, $l, $x, $hostname, $in, $curr_hba, $curr_dev_cnt, $curr_dev, @a );

#  my(@lux) = `/usr/sbin/luxadm probe -p`;  # needs run_command
  my($log);
#  $DB::single= 1;
  my $dev_cnt  = 0;
  for ($x=0; $x <= $#$com; $x++ ){
      $l = $com->[$x];
       next if ($l =~ /^\s*$/);
       $l = Util->trim($l);
       if ($l =~ /^\s*Hostname:\s*(.*)/) {
          $hostname = "$1";
          if (index($hostname, ".") < 0) {
             my $do = lc(System->domain());
             $hostname = "$hostname.$do" if ($do);
          } else {
             my $ix2 = index($hostname, ".");
             $hostname = substr($hostname,0,$ix2) . lc(substr($hostname,$ix2));
          }

          $hosts{$hostname}{type}    = "Sun";
          $hosts{$hostname}{name}    = "host:$hostname";
          $hosts{$hostname}{hostid}  = System->hostid();
          $hosts{$hostname}{model}   = System->model();
          $hosts{$hostname}{release} = System->os_version();

       } elsif ($l =~ /^\s*Detected.*HBA port\(s\)/) {
          $in = "hbasumm";

       } elsif ($l =~ /^\s*Detected.*FCAL Device\(s\)/) {
          $in = "devicesumm";

       } elsif ($l =~ /^\s*HBA Port (\d+) Path:(.*)/) {
          $in = "hba"; $curr_hba = $1; #sprintf("%2.2d", $1);
          $hbas{$hostname}{$curr_hba}{path} = $2;
          $curr_dev_cnt = -1;

       } elsif ($l =~/^\s*Device # (\d+):\s*$/) {
          $in = "device";
          $curr_dev_cnt++; $dev_cnt++;
          $curr_dev = sprintf("%3.3d", $curr_dev_cnt);

       } elsif ($in eq "hbasumm") {     #  SOCAL HBA port(s):      0

       } elsif ($in eq "devicesumm") {
          @a = split(/\s*:\s*/, $l,2);
          $hosts{$hostname}{$a[0]} .= $a[1] . ",";

       } elsif ($in eq "hba") {
          @a = split(/\s*:\s*/, $l,2);
          $a[1] =~ s/[,\|]/ /g;
          $hbas{$hostname}{$curr_hba}{$a[0]} = $a[1];

       } elsif ($in eq "device") {
          @a = split(/\s*:\s*/, $l,2);
          if (length($a[1]) > 0) {
             $a[1] =~ s/[,\|]/ /g;
             $devices{$hostname}{$curr_hba}{$curr_dev}{$a[0]} = $a[1];
          }
       }
    }
  return undef if (!$dev_cnt);

  my(%attached, $dinfo, $alpa, $sw_type, $portno, $current_sw, $devs);
  my($dp_port, @DP2OBJ, $last_sl_port, $hba);
  my(%PG, %PGM) ;
  my($host, $hostObj, $h, $hinfo , $dp_switch, $current_st, $st_type, 
    $current_hub , $st_port, $new);
  my($ip_port) = -1;
  my(%MAP, $from , $fromK);
  my $incomplete = {flag => 0};

  foreach $host (sort keys %hosts) {
    $hostObj = TO->newHost($Topo, $host, $hosts{$host});
    
    $out .= "Host: $host $hosts{$host}{type}\n";
    foreach $hba (sort keys %{$hbas{$host}}) {  # number
       $dp_port = 0; $dp_switch = -1;
       $out .= "\n[hba-$hba ";
       $hinfo = $hbas{$host}{$hba};
       $hostObj->{portInfo}[$hba] = $hinfo;
       $dp_port = 0; $dp_switch = -1; %attached = (); $last_sl_port = "";

       if ($hinfo->{al_pa} == -2) {    # hba not attached
           $hostObj->{port}[$hba] = 0;
           $out .= "not connected]\n";
           next;
       }
       $MAP{"HBA$hba"} = [$hostObj, $hba];
       my %A5;
       my($p2) = $hinfo->{path};
       my($i2) = rindex($p2, ":");
       $p2 = substr($p2, 1, $i2-1);
       
       $out .= "$hinfo->{LGroup}, Port:$hinfo->{PortWWN}, alpa=$hinfo->{al_pa}]\n";
       $out . "[       $hinfo->{path}  ]\n";
       $devs = $devices{$host}{$hba};
       foreach my $d (sort keys %$devs) {
           $dinfo = $devs->{$d};
           # SWITCH
           if ($dinfo->{ProductID}  =~ /Switch/) {
               ($new, $current_sw) = TO->newSwitch($SW, $Topo, $host, $dinfo);
               $portno  = $dinfo->{sw_PortNum};
               $sw_type = TO->switchPortType($dinfo);

               $MAP{"DEVICE".$dinfo->{DeviceID}} = [$current_sw, $portno];

               $current_sw->{portInfo}[$portno] = {
                       sw_PortNum   => $dinfo->{sw_PortNum},
                       sw_PortType  => $dinfo->{sw_PortType},
                       sw_PortState => $dinfo->{sw_PortState},
                       sw_ipAddr_remote=> $dinfo->{sw_ipAddr_remote},
                       sw_remote_fcaddr=> $dinfo->{sw_remote_fcaddr},
                       sw_FabricID  => $dinfo->{sw_FabricID},
                       Status       => $dinfo->{Status},
                       RegisterName => $dinfo->{RegisterName},
                       Parent       => $dinfo->{Parent},
                       portType     => $sw_type,
                       'Parent type' => $dinfo->{'Parent type'},
                       };
               $alpa = $dinfo->{sw_DevAlpas};
               $alpa = substr($alpa,0,11) . "..." if (length($alpa) > 11);

               $dp_port = 0; $dp_switch = -1;
               if ($dinfo->{NodeWWN} =~ /^0+$/) {
                   $current_sw->{port}[$portno] = 0;
               } else {
                  $fromK = $dinfo->{"Parent type"} . $dinfo->{Parent};
                  $from  = $MAP{$fromK};
                  if (!$from) {
                     Debug->err(TEXT => "TOPO: In sw-device $dinfo->{DeviceID}: Cannot find the parent $fromK");
                     next;
                  }
                  if ($from->[0] == $current_sw) { # port2port in same switch
                    $current_sw->{dport}[$from->[1]] .= $portno . "-";

                  } else {
                    $current_sw->{port}[$portno] = TO->nodeName($from->[0]) .":". $from->[1];
                    $from->[0]->{port}[$from->[1]] = TO->nodeName($current_sw) . ":$portno";
                  }
               }
           # STORAGE
           } else {
              next if ($dinfo->{LGroup} =~ /-Internal-/); # internal disk
              # next if ($dinfo->{"Path State"} =~ /STATE_STANDBY/); # MPX

              ($new, $st_port, $st_type, $current_st) = 
                         TO->newStorage($ST, $Topo, $host, $dinfo ,$hba, 
                         $p2, $incomplete, \%attached);
              next if (!$current_st);

              $MAP{"DEVICE".$dinfo->{DeviceID}} = [$current_st, $st_port] ;

              my($vol) = 0;
              my($disk) = $dinfo->{RegisterName};

              if ($st_type eq "t3") {
                 $current_st->{volInfo}{$dinfo->{SerialNo}} = $dinfo;
                 if ($dinfo->{PortMode} =~ /Alternate/) {
                   $current_st->{info}{PG} = 1 ;
                   next;
                 }
              } elsif ($st_type eq "a5k") {
                 if ($dinfo->{RegisterName} !~ /\-ses(\d+)/) {  # disks device
                    my($i) = index($disk, "-");                 # c5t34d0-f2
                    $disk = substr($disk, $i+1);
                    $current_st->{diskInfo}{$disk} = {
                        PortWWN => $dinfo->{PortWWN},
                        NodeWWN => $dinfo->{NodeWWN},
                        LogicalPath => $dinfo->{LogicalPath},
                        FirmwareRev => $dinfo->{FirmwareRev},
                        sf_al_pa => $dinfo->{sf_al_pa},
                        VendorID => $dinfo->{VendorID},
                        SerialNo => $dinfo->{SerialNo},
                        RegisterName => $dinfo->{RegisterName},
                    };
                    next;
                 }
                 next if ($current_st->{info}{"ses_cnt$hba"} > 0);
                 $current_st->{info}{"ses_cnt$hba"}++;
              } else {
                 $current_st->{diskInfo}{$disk} = $dinfo;
              }

              next if ($attached{TO->nodeName($current_st).$st_port}); 
              $attached{TO->nodeName($current_st).$st_port} = 1;

              $fromK = $dinfo->{"Parent type"} . $dinfo->{Parent};
              $from  = $MAP{$fromK};
              if (!$from) {
                 Debug->err(TEXT => "TOPO: In st-device $dinfo->{DeviceID}: Cannot find the parent $fromK");
                 next;
              }
              &addNode($host, $Topo, $from->[0], $from->[1], $st_type, $current_st, $st_port);
           }
       }
    }
  }
  my $ic = PDM->getCacheHandle("incomplete_topo");
  if ($incomplete->{flag}) {
     if (!defined($ic->{try})) {
        $ic->{try} = 3 ;    # start the counter
        $ic->{start} = time;
        Debug->err(TEXT => "Incomplete Topo (new):\n$incomplete->{err}");
     } elsif ($ic->{try} > 0) {
        my $t = int( (time - $ic->{start}) / 60);
        Debug->err(TEXT => "Incomplete Topo (started $t mins. ago):\n$incomplete->{err}");
     }
  } else {
     $ic->{try} = undef;
  }

  return ($Topo, $out);  
}


sub addNode {
  my($host, $Topo, $from, $from_port, $st_type, $current_st, $st_port) = @_;

  my($current_hub);
  my $a1 = $from->{port}[$from_port];

  if (0 && $a1 && ($a1 ne TO->nodeName($current_st) . ":$st_port")) {
      my(@a11) = split(/:/, $a1);
      my($h) = $Topo->nodeByName("$a11[0]:$a11[1]");
      if ($a11[0] eq "hub") { # already a hub, use it
          my($no) =  $h->{port};
          $no = $#$no + 1;
          $h->{port}[$no] =  "$st_type:" . TO->nodeName($current_st) . ":$st_port";
          $current_st->{port}[$st_port] = "hub:$a11[1]:$no";
      } else {              # create a hub.
          $current_hub = TO->newHub($Topo, $host, $from, $from_port);
          $from->{port}[$from_port]    = TO->nodeName($current_hub) . ":0";
          $current_hub->{port}[0]       = TO->nodeName($from) . ":$from_port";

          $current_hub->{port}[1]       = $a1;
          $h->{port}[$a11[2]]           = TO->nodeName($current_hub) . ":1";

          $current_hub->{port}[2]       = TO->nodeName($current_st) . ":$st_port";
          $current_st->{port}[$st_port] = TO->nodeName($current_hub) . ":2";
      }
  } else {
      $from->{port}[$from_port]    = TO->nodeName($current_st) . ":$st_port";
      $current_st->{port}[$st_port] = TO->nodeName($from) . ":$from_port";
  }
}


  

sub storageType {
  my($class, $dinfo) = @_;
  if ($dinfo->{LGroup} =~ /A5/) {
     return 'a5k';
  } elsif ($dinfo->{LGroup} =~ /\-T3\-/) {
     return 't3';
  } elsif ($dinfo->{LGroup} =~ /\-Internal\-/) {
     return 'i';
  } elsif ($dinfo->{LGroup} =~ /\-A35/) {
     return 'a3500fc';
  } else {
     return 'x';
  }
}

# new means first device of this enclusure or 
# for A5K, new is for each ses device.

use vars qw(%TMP);

sub newStorage {
  my($class, $ST, $Topo, $host, $dinfo, $hba, $hba_path, $incomplete,
      $attached) = @_;
  my($type) = $class->storageType($dinfo);
  my($size, $port, $ix, $new, $key, $ci);
  my($name, $name2);

  my($storages) = $Topo->[2];
  my($syns)     = $Topo->[4];

  if ($type eq "a3500fc") {
     ($key, $port) = Catalog->deviceRead('a3500fc',Catalog::WWN, 
                           $dinfo->{SerialNo}, Catalog::Key);
     if (!$key) {
        my $sn  = $dinfo->{SerialNo};
        $name = "a3500fc:$sn";
        $dinfo->{BoxName} = $sn;
        $incomplete->{flag} = 1;
        if (!$TMP{$sn}) {
          $incomplete->{err} .= "Topology: Cannot identify this A3500FC ($sn), it may not be monitored\n";
          $TMP{$sn} = 1;
        }
     } else {
        $name = "a3500fc:$key";
        $dinfo->{BoxName} = substr($key,3);
     }
     $port--;

  } elsif ($type eq "t3") {
     ($ci, $port) = Catalog->deviceRead('t3',Catalog::WWN, $dinfo->{PortWWN});
     $key = $ci->[Catalog::Key];
     $dinfo->{BoxName} = $ci->[Catalog::Name] || $dinfo->{PortWWN};

     if (!$key) {
       $incomplete->{flag} = 1;
       my $wwn2 = $dinfo->{PortWWN};
       if (!$TMP{$wwn2}) {
         $incomplete->{err} .= "Topology: Cannot identify this T3 ($wwn2), it may not be monitored.\n";
         $TMP{$wwn2} = 1;
       }
       $name = "t3:$dinfo->{PortWWN}";
       $port = 1;
     } else {
       $name = "t3:$key";
     }
     $port--;

  } elsif ($type eq "a5k") {
     if ($dinfo->{LogicalPath} =~ /\/ses/) {
         $ST->{$dinfo->{BoxName}} = $dinfo->{NodeWWN};
     }
     $name = "a5k:" . $ST->{$dinfo->{BoxName}};
     $name2= $type . ":" . $dinfo->{BoxName} if ($dinfo->{BoxName});
  }

  $new = 0;

  if ( !defined($storages->{$name})) {
     my($obj) = {};
     $obj->{"_hba" . $hba} = 1;
     $obj->{info}{type} = $type;
     $obj->{info}{name} = $name;
     $obj->{info}{BoxName} = $dinfo->{BoxName};
     $obj->{info}{host} = $host;
     $obj->{info}{PortWWN} = $dinfo->{PortWWN};
     $obj->{info}{NodeWWN} = $dinfo->{NodeWWN};
     $obj->{info}{LogicalPath} = $dinfo->{LogicalPath};
     $obj->{info}{LGroup} = $dinfo->{LGroup};
     $obj->{info}{VendorID} = $dinfo->{VendorID};

     my($p) =  $dinfo->{LGroup};
     my(@a) = split(/[\-\(]/, $p); 
     $p = substr($a[1],0,-1) + 0;
     $obj->{info}{sesCount} = $p;
     bless($obj, 'TO_Storage');
     $storages->{$name} = $obj;
     $syns->{$name2} = $obj if ($name2);
     $new = 1;
  }
  $ix = $storages->{$name};

#
#  FIGURE-OUT WHICH PORT
#
  my($new_p);
  if ($type eq "a5k") {
     if ($dinfo->{RegisterName} =~ /\-ses(\d+)/ && 
              ($ix->{info}{"ses_cnt$hba"} == 0) ) {   # first ses
       my($p) = $dinfo->{PortWWN};
       my($p0) = substr($p,-1);
       $port = (index("129a", $p0) >= 0) ? 0:1;
       if (!$ix->{portInfo}[$port]) {
         $ix->{portInfo}[$port] = $dinfo;
         $ix->{portInfo}[$port]{hba_path} = $hba_path; # store hba-path to the port
       } else { 
         $port = ($port==0) ? 1:0; # use the other port
         $ix->{portInfo}[$port] = $dinfo;
         $ix->{portInfo}[$port]{hba_path} = $hba_path; # store hba-path to the port
       }
     }
  } elsif ($type eq "t3") {
     if (index($ix->{info}{vols}, "$dinfo->{SerialNo},")  < 0) {
       $ix->{info}{vols} .= "$dinfo->{SerialNo},";
     }
     $ix->{info}{PG} = 1 if ($port == 1);
     if (! $ix->{portInfo}[$port]) {
        $ix->{portInfo}[$port] = $dinfo;
        $ix->{portInfo}[$port]{hba_path} = $hba_path; # store hba-path to the port
     }

  } else {
     ($port, $new_p) = &find_port($ix, $hba_path);

     if ($new_p) {
       $ix->{portInfo}[$port] = $dinfo;
       $ix->{portInfo}[$port]{hba_path} = $hba_path; # store hba-path to the port
     }
  }
  
  return ($new, $port, $type,$ix);
}

sub find_port {
  my($ix, $hba_path) = @_;
  my($port, $new);
  if ($ix->{portInfo}[0]{hba_path} eq $hba_path) {
     $port = 0;
  } elsif ($ix->{portInfo}[1]{hba_path} eq $hba_path) {
     $port = 1;
  } elsif ($ix->{portInfo}[0]{hba_path}) {
     $port = 1; $new = 1;
  } else {
     $port = 0; $new = 1;
  }
  return ($port, $new);
}

  
sub newHost {
  my($class, $Topo, $hostname, $info) = @_;
  my($size);
  my($name) = "host:$hostname";
  my($hosts) = $Topo->[0];
  if (!defined($hosts->{$name}) ) {
     my($obj) = {};
     $obj->{info}       = $info;
     $obj->{info}{type} = "host";
     $obj->{info}{name} = $name;
     bless($obj, 'TO_Host');
     $hosts->{$name} = $obj;
  }
  return $hosts->{$name};
}

  


sub newHub {
  my($class, $Topo, $host, $from, $from_port) = @_;
  my($size);
  my($hubs) = $Topo->[3];
  my(@x) = (keys %$hubs);
  my($name) = "hub:H" . ($#x +1);

  if (!defined($hubs->{$name})) {
     my($obj) = {};
     $obj->{info}{type} = 'hub';
     $obj->{info}{name} = $name;
     $obj->{info}{host} = $host;
     $obj->{info}{from} = $from->type() . ":" . $from->key() . ":$from_port"; # for CIM event
     bless($obj, 'TO_Hub');
     $hubs->{$name}   = $obj;
  }
  return $hubs->{$name};
}


#
#  LIST OF OBJECTS
#
sub hostList {
  my($Topo) = @_;
  my($hosts) = $Topo->[0];
  my(@list) = (values %$hosts);
  return \@list;
}

sub switchList {
  my($Topo) = @_;
  my($hosts) = $Topo->[1];
  my(@list) = (values %$hosts);
  return \@list;
}

sub storageList {
  my($Topo) = @_;
  my($hosts) = $Topo->[2];
  my(@list) = (sort values %$hosts);
  return \@list;
}

sub hubList {
  my($Topo) = @_;
  my($hosts) = $Topo->[3];
  my(@list) = (values %$hosts);
  return \@list;
}

sub nodeByName {
  my($Topo, $name) = @_;
  my($x);

  my($type, $n, $port) = split(/:/, $name);
  my($name) = "$type:$n";

  for ($x=0; $x <= 3; $x++) {
    my($n) = $Topo->[$x];
    if ($n->{$name}) {
       if (wantarray) {
          return ($n->{$name}, $port);
       } else {
          return $n->{$name};
       }
    }
  }
  return undef;
}

#
#  $To->nodeName($node);

sub nodeName {
  my($Topo, $node) = @_;
  
  return $node->{info}{name};
}

sub newSwitch {
  my($class, $SW, $Topo, $host, $dinfo) = @_;
  my( $size, $name );
  my($new) = 0;

  if (!$SW->{$dinfo->{sw_ipAddr}} ) {
     if ($dinfo->{sw_WWN}) {
        $SW->{$dinfo->{sw_ipAddr}} = $dinfo->{sw_WWN};
     } else {
        $SW->{$dinfo->{sw_ipAddr}} = $dinfo->{sw_ipAddr};
     }
  }
  
  $name = "switch:" . $SW->{$dinfo->{sw_ipAddr}};

  my($switches) = $Topo->[1];
  my($syns)     = $Topo->[4];

  my($name2) = "switch:" . $dinfo->{sw_ipName};
  my($name3) = "switch:" . $dinfo->{sw_ipAddr};


  if (!defined($switches->{$name})) {
     $new= 1;
     my($obj) = {};

     $obj->{info}{name} = $name;
     $obj->{info}{type} = "switch";
     $obj->{info}{host} = $host;
     foreach my $k (keys %$dinfo) {
       $obj->{info}{$k} = $dinfo->{$k};
     }
     $obj->{info}{portCount} = $dinfo->{sw_MaxPorts};
     bless($obj, 'TO_Switch');
     $switches->{$name}  = $obj;
     $syns->{$name2} = $obj;
     $syns->{$name3} = $obj;
  }
  return ($new ,$switches->{$name});
}

sub switchPortType {
  my($class, $dinfo) = @_;

  my($p) =  $dinfo->{RegisterName};
  my($p2)=  $dinfo->{sw_PortType};

  if ($p2 eq "E_Port") {
     return "e";
  } elsif ($p2 eq "FL_Port") {
     return "ip";
  } elsif ($p2 eq "TL_Port") {
     return "dp";
  }
  my(@a) = split(/[\-\(\_]/, $p); 
  return $a[3];
}
  


sub display {
  my($class,$devs,  $enc) = @_;
  my($type, $name, $comp) = split(/:/, $enc);

  if ($type eq "a5k" || $type eq "t3" || $type eq "a3500fc") {
    return TO->storageToString(-1, "$type:$name");
  } elsif ($type eq "switch") {
     return TO->switchToString(-1, "s:$name");

  } elsif ($type eq "h") {
     return TO->hostToString(-1,"h:$name");
  } else {
    print "Nothing on $enc! \n";
  }
}

sub cleanN {
  my($s) = @_;
  my(@a) = split(/:/, $s);
  if ($a[0] eq "hub") {  
    return "u:$a[1]";
  }
  return substr($a[0],0,1) . ":" . $a[1];
}

#
# ->appletStream({ status => \&function,  clear => 1, fibreLog => $errorlist });
#

sub appletStream {
 my($to, $arg) = @_;
 my($l, %XY, @a);
 my($encInfo, $portInfo, $diskInfo, $otherInfo) ;

 my($statusFunc) = $arg->{status};

 my($writeFlag) = 0;
 my($fibrelog) = $arg->{fibreLog};
 my($renv, $devs, $hosts, $notifs) = PDM::ConfigFile->read;
 my(%SW);
 foreach my $sw (@$devs) {
    if ($sw->{type} eq "switch") {
       $SW{$sw->{ipno}} = $sw->{name};
    }
 }
 my($out) = "";

 if ($renv->{showoob} eq "Y") {
   foreach my $dev (@$devs) {
      next if ($dev->{host}); 
      next if ($dev->{type} ne "t3" && $dev->{type} ne "switch");
      my $ci = Catalog->deviceRead($dev->{type},Catalog::IPno, $dev->{ipno});
      next if (!$ci);
      my $key = $ci->[Catalog::Key];  
      next if ($to->nodeByName("$dev->{type}:$key"));
      # found a t3 or switch monitored by master, add it to the topology temporarily.
      if ($dev->{type} eq "t3") {
        $to->newT3($ci->[Catalog::WWN1], $ci);
      } else {
        $to->newSW($ci, $dev);
      }
   }
 }

 my($clear)      = $arg->{clear};
 my($l, @a, %XY);

 my($F) = System->get_home() . "/DATA/tmp/topoxy_$arg->{name}";
 my $zoom = -1;
 if (!$clear) {
   open(O, $F);
   while ($l = <O>) {
     chop($l);
     @a = split(/\|/, $l);
     if (substr($a[0] ,0,2) eq "z=") {
        $zoom = substr($a[0],2);
     } else {
       $XY{$a[3]} = [$a[4],$a[5]];
     }
   }
   close(O);
 }

 my($cnt) = 1;
 my($curr_x, $curr_y, $x, $s, $ports, $name, $x0, $y0, $h, $label2, $from, $to2);
 $curr_x = 200; $curr_y = 20; 
 my($save_cnt, $y, @b);

 my($switches) = $to->switchList();
 my($hubs) = $to->hubList();
 my($hosts) = $to->hostList();
 my($storages) = $to->storageList();

 my(%MAP);
 for ($x=0; $x <= $#$switches; $x++) {
    my($sw) = $switches->[$x];
    $MAP{$sw->{info}{name}} = "s$x";
 }
 for ($x=0; $x <= $#$hubs; $x++) {
    my($sw) = $hubs->[$x];
    $MAP{$sw->{info}{name}} = "u$x";
 }
 for ($x=0; $x <= $#$hosts; $x++) {
    my($sw) = $hosts->[$x];
    $MAP{$sw->{info}{name}} = "h$x";
 }
 for ($x=0; $x <= $#$storages; $x++) {
    my($sw) = $storages->[$x];
    $MAP{$sw->{info}{name}} = "a$x";    # a5k:wwn => 2
 }

#
# SWITCHES
#
 my($hook_cnt) = 0;
 for ($x=0; $x <= $#$switches; $x++) {
    $s     = $switches->[$x];

    my($dports) = $switches->[$x]{dport};
    my($zones , $zc);
    for ($y=0; $y <= $#$dports; $y++) {  # iport => dport1,dport2 ...
       if ($dports->[$y]) {
          my $v2 = $dports->[$y];
          chop($v2) if (substr($v2,-1) eq "-");
          $zones .= "$y-$v2/";
          $zc++;
       }
    }
    if ($#$dports < 0) {   # fake the dport to draw ports in the switch
        my($dports) = $switches->[$x]{port};
       my $cnt = 0;
       for ($y=1; $y <= $#$dports; $y++) {
           next if (!$dports->[$y]);
           $zones .= "$y-";
           $cnt++;
           if ($cnt %2 == 0) {
              chop($zones) if (substr($zones,-1) eq "-");
              $zones .= "/";
           }
       }
       $zones .= " ";
    }
    chop($zones);
    my($max2) = ($zc > 8) ? 16:8;

    $name   = $s->{info}{name};
    $label2 = $s->{info}{ProductID} . ",";

    my($dName);
    $dName = $SW{$s->{info}{sw_ipAddr}} || $s->{info}{sw_ipAddr};

    $x0 = $XY{$name}[0] ;
    $y0 = $XY{$name}[1] ;
    if (!$x0) {
      $x0 = $curr_x; $y0 = $curr_y;
      $curr_y += 130;
    }
    my($encStatus, $portStatus, $diskStatus, $otherStatus);
    $encInfo= $portInfo= $diskInfo= $otherInfo = "";
    if ($statusFunc) {
      ($encStatus, $portStatus, $diskStatus, $otherStatus,
       $encInfo, $portInfo, $diskInfo, $otherInfo) =  
                &$statusFunc("switch", $name, $arg);
    }
    $out .= "s$x|v|s,XXX,$zones,$max2,F8F8E0|$name=$dName,$label2|$x0|$y0|$encStatus|$portStatus|$diskStatus|$otherStatus|$encInfo|$portInfo|diskinfo|otherinfo\n";

    $ports = $switches->[$x]{port};

    for ($y=1; $y <= $#$ports; $y++) {
        if ($ports->[$y]) {
           @b = split(/:/, $ports->[$y]);
           next if ($b[0] ne "switch"); # do switch2switch only

           $from =  $MAP{"$b[0]:$b[1]"} . ",$b[2]";
           $to2   = "s$x,$y";
           $out .= &add_link($from, $to2, $hook_cnt, $x0- 20, $y0+ $y*20+ 15);
           $hook_cnt++;
           $cnt++;
        }
    }

    for ($y=0; $y <= $#$dports; $y++) {
        if ($dports->[$y]) {
           @b = split(/\-/, $dports->[$y]);
           for my $z (@b) {
             $out .= "00|e|s||s$x,$y|s$x,$z\n";
           }
        }
    }
 }
#
# HUBS
#
 my($hbas, $hInfo, $portStatus, $portInfo, $map);
 $curr_x = 300; $curr_y = 20;

 for ($x=0; $x <= $#$hubs; $x++) {
    $s = $hubs->[$x];
    $name = $s->{info}{name};
    $ports = $hubs->[$x]{port};

    $x0 = $XY{$name}[0] ;
    $y0 = $XY{$name}[1] ;
    if (!$x0) {
      $x0 = $curr_x; $y0 = $curr_y;
      $curr_y += 100;
    }
    $out .= "u$x|v|u,Hub|$name|$x0|$y0||||||portinfo|diskInfo|encinfo\n";

    for ($y=0; $y <= $#$ports; $y++) {
        if ($ports->[$y]) {
           @b = split(/:/, $ports->[$y]);

           $name = $hubs->[$x]{info}{name};
           $from = "u$x,$y";
           $to2   =  $MAP{"$b[0]:$b[1]"} . ",$b[2]";

           $out .= "00|e|f||$from|$to2\n";
        }
    }
 }


  

#
# HOSTS
#
 $curr_x = 20; $curr_y = 20;
 $encInfo = "";

 for ($x=0; $x <= $#$hosts; $x++) {
    $h     = $hosts->[$x];
    $hbas  = $hosts->[$x]{port};
    $hInfo = $hosts->[$x]{portInfo};
    $ports = $#$hbas + 1;
#    $ports = 8 if ($ports < 8);
    $name  = $h->{info}{name};

    my $name0 = substr($name, 5);
    my $in1 = index($name0, ".");
    $name0 = substr($name0, 0, $in1) if ($in1 > 0);

    $x0 = $XY{$name}[0] ;
    $y0 = $XY{$name}[1] ;
    if (!$x0) {
      $x0 = $curr_x; $y0 = $curr_y;
      $curr_y += 110;
    }
    $portStatus = ""; $portInfo = "";
    for ($y=0; $y <= $#$hbas; $y++) {
       my($in) =  $hInfo->[$y]{path};
       $in = substr($in,9) if (substr($in,0,9) eq " /devices");
       my($ii) = rindex($in, "/");
       $in = substr($in, 0, $ii);
       $in =~ s/,/_/g;
       if ($hbas->[$y]) {
         $portStatus .= ",";
         $portInfo .= "$in,";
       } else {
         $portStatus .=  "-1,";
         $portInfo .= "$in,";
       }
    }
    chop($portStatus);
    chop($portInfo) if ($portInfo);
    $out .= "h$x|v|h,Sun,$ports|$name=$name0,|$x0|$y0||$portStatus|||$encInfo|$portInfo||otherInfo\n";

#07|e|f|0|0|0|cable|06,1|03,6||

    for ($y=0; $y <= $#$hbas; $y++) {
        if ($hbas->[$y]) {
           @b = split(/:/, $hbas->[$y]);
           $name = $hInfo->[$y]{RegisterName} ;
           $from = "h$x,$y";
           $to2   = $MAP{"$b[0]:$b[1]"} . ",$b[2]";
           next if (substr($to2,0,1) eq "a") ;  # a=storage array
           if (substr($name, 0, 6) eq "pseudo") {
#             $out .= "00|e|M|2/pl,pseudo-link: This link is just a place-holder|$from|$to2\n";
           } else {
             $out .= &add_link($from, $to2, $hook_cnt, $x0+60, $y0+$y*9);
           }
           $hook_cnt++;
        }
    }
 }
#
# STORAGES
#
 my($type, $diskStatus, $dInfo, $diskInfo, $pInfo);
 $curr_x = 360; $curr_y = 20;
 my(%PG);

 my($part1, $part2, $pg);
 for ($x=0; $x <= $#$storages; $x++) {
    $pg = "";
    $s = $storages->[$x];
    $type = $s->{info}{type};
    $name = $s->{info}{name};
    my $dName = $s->{info}{BoxName};
    my $ix2 = index($dName,".");
    $dName = substr($dName,0,$ix2) if ($ix2 > 0);
    my($ses) = 2;
    my($cat);
    $label2 = "";

    if ($type eq "a3500fc") {
       $label2 = "A35";
       $cat = "a,A35,2,10,,L,Luns,E0F0E0";

    } elsif ($type eq "a5k") {
       $cat = "a,A5K,2/AB,11,f/r,FBP,Fan/BackPlane/Power,F0D0D0";
       my(@b) = split(/\-/, $s->{info}{LGroup});
       $label2 = "A5K";

    } elsif ($type eq "t3") {
       if ($s->{info}{PG}) {
         $cat = "a,T3,2/12,9,1/2,LMPV,Luns/Midplane/Power/Volumes,D0D0F0";
         $label2 = "T3Pg";
       } else {
         $cat = "a,T3,1/1,9,1,LMPV,Luns/Midplane/Power/Volumes,D0D0F0";
         $label2 = "T3";
       }
       $ses = 1;
    }
    $x0 = $XY{$name}[0] ; # arra
    $y0 = $XY{$name}[1] ;

    $label2 .= ",";

    if (!$x0) {
      $x0 = $curr_x; $y0 = $curr_y;
      if ($x % 3 == 0) {
         $curr_x += 80;
         $curr_y += 50;
      } elsif ($x % 3 == 1) {
         $curr_x += 80;
         $curr_y += 50;
      } else {
         $curr_x -= 160;
         $curr_y += 10;
      }
    }

    $dInfo  = $TO::Storages->[$x]{diskInfo};

    my($encStatus, $portStatus, $diskStatus, $otherStatus);
    $encInfo= $portInfo= $diskInfo= $otherInfo = "";
    if ($statusFunc) {
      ($encStatus, $portStatus, $diskStatus, $otherStatus,
       $encInfo, $portInfo, $diskInfo, $otherInfo) = 
            &$statusFunc($type, $name, $arg);
    }

    $out .= "a$x|v|$cat|$name=$dName,$label2|$x0|$y0|$encStatus|$portStatus|$diskStatus|$otherStatus|$encInfo|$portInfo|$diskInfo|$otherInfo\n";

    $ports = $storages->[$x]{port};
    $pInfo = $storages->[$x]{portInfo};
    for ($y=0; $y <= $#$ports; $y++) {
        if ($ports->[$y]) {
           @b = split(/:/, $ports->[$y]);

           my($ix) = rindex($pInfo->[$y]{LGroup} , "(");
           $name = substr($pInfo->[$y]{LGroup}, $ix+1,-1);
           $from =  $MAP{"$b[0]:$b[1]"} . ",$b[2]";
           $to2   = "a$x," . ($y+1) ;

           $out .= &add_link($from, $to2, $hook_cnt, $x0-20, $y0+$y*20+15);
           $hook_cnt++;
           $cnt++;
        }
    }
 }


 

#
#  linkState
#
 my($state) = $arg->{state}->{'L'};
 my($map1, $map2, %LNK);

 foreach my $l (keys %$state) {
   my($node1, $node2) = split(/\|/, $l);  # source -> destination.

   if ($node1 le $node2) {
      my($lnk) = $LNK{"$node1|$node2"};
      $LNK{"$node1|$node2"} = [$lnk->[0]+$state->{$l}[3], $lnk->[1]+0, 
            $state->{$l}[1], $state->{$l}[0]];
   } else {
      my($lnk) = $LNK{"$node2|$node1"};
      $LNK{"$node2|$node1"} = [$lnk->[0]+0, $lnk->[1]+$state->{$l}[3], 
            $state->{$l}[1], $state->{$l}[0]];
   }
 }
 foreach $l (keys %LNK) {
   my($node1, $node2) = split(/\|/, $l);

   my($type1, $k1, $p1) = split(/:/, $node1);
   my($type2, $k2, $p2) = split(/:/, $node2);

   my $info2 = $LNK{$l}[2];
   my($sev, $cnt0);
   if ($LNK{$l}[3] == 1) {
      $sev = "E";
      $cnt0 = $LNK{$l}[0] . "/" . $LNK{$l}[1];
   } else {
      $sev = "M"; # link-Error or Mpxio-error
      $cnt0 = "$LNK{$l}[3]/mpxio";
   }
   $info2 =~ s/\n/ /g;

   my($m1) = $MAP{"$type1:$k1"};
   my($m2) = $MAP{"$type2:$k2"};
   next if (!$m1 || !$m2);
   next if ($LNK{$l}[0] + $LNK{$l}[1] <= 0);

   $out .= "$cnt|e|$sev|$cnt0,$info2,$node1/$node2|$m1,$p1|$m2,$p2\n";
   $cnt++;
 }
   
    
  open(O, ">/tmp/appletStream");
  print O $out; close(O);
  return ($zoom, $out);

}


sub add_link {
  my($from, $to, $hook, $x, $y) = @_;
  my($out);
#  $out .= "x$hook|v|x||$x|$y|\n";

  $out .= "00|e|f||$from|$to\n";
#  $out .= "00|e|f||x$hook,0|$to\n";
  return $out;
}

########################################################################
#
#    Host/Switch/Storage/Hub/ PACKAGES 
#
########################################################################
package TO_Host;

sub port      { my($h) = @_; return $h->{port} }
sub portInfo { my($h) = @_; return $h->{portInfo} }
sub info     { my($h) = @_; return $h->{info} }


sub toString {
  my($h) = @_;
  my($x, $out, $hbasI);

  $out = "HOST  : '$h->{info}{name}'\n";
 
  my($ports) = $h->port();
  my($hbas) = $h->portInfo();
  my($hba, $port);

  for ($x=0; $x <= $#$hbas; $x++) {
     $hba = $hbas->[$x];
     $port = $ports->[$x];
     next if ($hba->{al_pa} == -1);
     $out .= "      HBA-$x: NodeWWN=$hba->{NodeWWN} \n".
             "             Path   =$hba->{path}  \n".
             "             LGroup =$hba->{LGroup} \n".
             "             Mode   =$hba->{path}, al_pa=$hba->{al_pa},\n";
     $out .= "             LINK_TO-> $port \n";
  }
  return $out;
}


sub type { "host"}

sub hostid {
  my($obj) = @_;
  return $obj->{info}{hostid};
}

sub name {
  my($obj) = @_;

  return $obj->{info}{name};
}

sub key {
  my($obj) = @_;
  my(@xx) = split(/:/, $obj->{info}{name});
  return $xx[1];
}

  

########################################################################

package TO_Switch;

sub port      { my($h) = @_; return $h->{port} }
sub dport      { my($h) = @_; return $h->{dport} }
sub info      { my($h) = @_; return $h->{info} }
sub portInfo { my($h) = @_; return $h->{portInfo} }


sub iPort {
  my($sw, $port) = @_;
  my ($x,$d);
  my $dp = $sw->{dport}; # internal links
  for ($x=1; $x <= $#$dp; $x++) {
     $d = $dp->[$x];
     if (index("-$d", "-$port-") >= 0) {
         return $x;
     }
  }
  return undef;


}

sub toString {
  my($s) = @_;
  my($x);

  my($out) = "SWITCH: '$s->{info}{name}' ip=$s->{info}{ip}, ipName=$s->{info}{ipName}\n";

  my($ports)  = $s->port();
  my($portsI) = $s->portInfo();

  for ($x=1; $x <= $#$ports; $x++) {
     my($p) = $portsI->[$x];
     my($pi) = $ports->[$x];
     next if (!$p);
     $out .= "      PORT-$x: sw_PortNum=$p->{sw_PortNum}, NodeWWN=$p->{NodeWWN} \n";
     $out .= "              LINK_TO-> $pi \n";
  }
  return $out;
}

sub type { "switch"};
sub name {
  my($obj) = @_;

  return $obj->{info}{name};
}

sub id {
  my($obj) = @_;

  my($id) = "switch '";
  if ($obj->{info}{sw_ipName} !~ /unknown/) {
    $id .= $obj->{info}{sw_ipName} . "/";
  }
  $id .= $obj->{info}{sw_ipAddr} if ($obj->{info}{sw_ipAddr});
  $id .= "'";

  return $id;
}

# second part of name
sub key {
  my($obj) = @_;
  my(@xx) = split(/:/, $obj->{info}{name});
  return $xx[1];
}
 

########################################################################

package TO_Storage;

sub port      { my($h) = @_; return $h->{port} }
sub portInfo { my($h) = @_; return $h->{portInfo} }
sub diskInfo { my($h) = @_; return $h->{diskInfo} }
sub volInfo  { my($h) = @_; return $h->{volInfo} }
sub info     { my($h) = @_; return $h->{info} }



sub toString {
  my($s) = @_;
  my($disks, $ports, $x, $portI, $volI);

  my($name) = $s->name();
  my($type) = $s->type();
  my($info) = $s->{info};
  my($utype) = sprintf("%-6.6s", uc($type));

  my($out) = "$utype: '$name', BoxName=$info->{BoxName}, LogicalPath=$info->{LogicalPath} \n";

  $ports = $s->port();
  $portI = $s->portInfo();

  for ($x=0; $x <= $#$portI; $x++) {
     my($p) = $portI->[$x];
     $out .= "      PORT-$x: NodeWWN=$p->{NodeWWN}, \n" . 
             "              LogicalPath=$p->{LogicalPath}, \n".
             "               LINK_TO-> $ports->[$x]\n";
  }

  my($disks) = $s->diskInfo();
  foreach my $disk (sort keys %$disks) {
     my($di) = sprintf("%3.3s", $disk);
     my($d) = $disks->{$disk};
     $out .= "     DISK-$di : NodeWWN=$d->{NodeWWN},  RegisterName=$d->{RegisterName}, $d->{SerialNo} \n";
  }
  my($vols) = $s->volInfo();
  if ($vols) {
    foreach my $v1 (keys %$vols) {
       print "     VOL-$v1\n";
    }
  }
  return $out;
}

sub type {   
  my($obj) = @_;
  return $obj->{info}{type};
}

sub name {
  my($obj) = @_;

  return $obj->{info}{name};
}

sub id {
  my($obj) = @_;
  my($id) = $obj->{info}{type} . " '" ;
  $id .= $obj->{info}{BoxName} || $obj->{info}{NodeWWN};
  $id .= "'";
  return $id;
}
  

sub key {
  my($obj) = @_;
  my(@xx) = split(/:/, $obj->{info}{name});
  return $xx[1];
}


########################################################################

package TO_Hub;

sub port      { my($h) = @_; return $h->{port} }
sub portInfo { my($h) = @_; return $h->{portInfo} }

sub toString {

}

sub type { "hub"};
sub name {
  my($obj) = @_;

  return $obj->{info}{name};
}

sub key {
  my($obj) = @_;
  my(@xx) = split(/:/, $obj->{info}{name});
  return $xx[1];
}


1;
