#
# Scheduled Down Time Formats
#
sub sdt_format {
  my($f) = @_;
  my($d, $dashes, $days);
  my($d1, $d2, $h1, $h2, $m1, $m2, @now, $today, $f1, $y1, $y2);
  my($today_year, $today_dow, $today_time, $dow1, $dow2, $range1, $range2);
  my($today_dom, $mth1, $mth2, $today_month);

  @now = localtime; $now[4]++;
  $today_month = $now[4];
  $today_dow = $now[6];
  $today_dom = $now[3];
  $today_time = sprintf("%2.2d:%2.2d", $now[2], $now[1]);
  $today_year = $now[5] - 100;

  $days = ",sun,mon,tue,wed,thu,fri,sat,";
  $f = lc($f);
  $f =~ s/ +/ /g;
  $d = substr($f, 0,3);
  $f1 = $f;
  $f1 =~ s/\-/$dashes++/eg;

# sdt1=Sun 23:30 - Mon 02:30            # day range, every week

  if ($dashes == 1) {
    $f =~ /(...) (\d+):(\d+) *- *(...) (\d+):(\d+)/;
    $d1 = $1; $h1 = $2; $m1 = $3;
    $d2 = $4; $h2 = $5; $m2 = $6;
    $dow1 = index($days,  ",$d1,");
    $dow2 = index($days,  ",$d2,");
    if ($dow1 < 0 || $dow2 < 0) {
       return ("Invalid DayOfWeek format", 0);
    }
    if (!$h1 || !$h2 || !$m1 || !$m2) {
       return ("Invalid Hours/Mins", 0);
    }
    if ($h1 > 23 || $h2 > 23 || $m1 > 59 || $m2 > 59) {
       return ("Invalid Hours/Mins", 0);
    }
    my($r1) = int($dow1/4);
    $range1 = sprintf("%d %2.2d:%2.2d", $r1, $h1, $m1);
    my($r2) = int($dow2/4);
    $r2 += 7 if ($r2 < $r1);
    $range2 = sprintf("%d %2.2d:%2.2d", $r2 , $h2, $m2);
       
    $today  = "$today_dow $today_time";

    if ($today ge $range1 && $today le $range2) {
      return (0, 1);
    } else {
      return (0, 0);
    }
  
# std2=DD-YY 8:30 - DD-YY 10:30         # day range, every month

  } elsif ($dashes == 3) {
    $f =~ /(\d+)-(\d+) (\d+):(\d+) *- *(\d+)-(\d+) (\d+):(\d+)/;
    $d1 = $1; $y1 = $2; $h1 = $3; $m1 = $4;
    $d2 = $5; $y2 = $6; $h2 = $7; $m2 = $8;

    if (!$h1 || !$h2 || !$m1 || !$m2 || !$d1 || !$y1 || !$d2 || !$y2) {
       return ("Invalid format", 0);
    }
    if ($h1 > 23 || $h2 > 23 || $m1 > 59 || $m2 > 59) {
       return ("Invalid Hours/Mins", 0);
    }
    $y1 -= 2000 if ($y1 >= 2000);
    $y2 -= 2000 if ($y2 >= 2000);

    if ($d1 > 31 || $d2 > 31 || $y1 > 5 || $y2 > 5) {
       return ("Invalid Day/Year, Format is DD-YY HH:MM", 0);
    }
    $range1 = sprintf("%2.2d-%2.2d %2.2d:%2.2d",$y1, $d1, $h1, $m1);
    $d2 += 31 if ($d2 < $d1);
    $range2 = sprintf("%2.2d-%2.2d %2.2d:%2.2d",$y2, $d2, $h2, $m2);
    $today = sprintf("%2.2d-%2.2d %s",$today_year, $today_dom, $today_time);

    if ($today ge $range1 && $today le $range2) {
      return (0, 1);
    } else {
      return (0, 0);
    }

#  sdt3=MM-DD-YY 8:30 - MM-DD-YY 10:30  # date range, specific date

  } elsif ($dashes == 5) {
    $f =~ /(\d+)-(\d+)-(\d+) (\d+):(\d+) *- *(\d+)-(\d+)-(\d+) (\d+):(\d+)/;
    $mth1 = $1; $d1 = $2; $y1 = $3; $h1 = $4; $m1 = $5;
    $mth2 = $6; $d2 = $7; $y2 = $8; $h2 = $9; $m2 = $10;

    if (!$h1 || !$h2 || !$m1 || !$m2 || !$d1 || !$y1 || !$d2 || !$y2 ||!$mth1) {
       return ("Invalid format", 0);
    }
    if ($h1 > 23 || $h2 > 23 || $m1 > 59 || $m2 > 59) {
       return ("Invalid Hours/Mins", 0);
    }
    $y1 -= 2000 if ($y1 >= 2000);
    $y2 -= 2000 if ($y2 >= 2000);

    if ($d1 > 31 || $d2 > 31 || $y1 > 5 || $y2 > 5 || $mth1 > 12 || $mth2 > 12) {
       return ("Invalid Month/Day/Year, Format is MM-DD-YY HH:MM", 0);
    }
    $range1 = sprintf("%2.2d-%2.2d-%2.2d %2.2d:%2.2d",$y1, $mth1, $d1, $h1, $m1);
    $range2 = sprintf("%2.2d-%2.2d-%2.2d %2.2d:%2.2d",$y2, $mth2, $d2, $h2, $m2);
    $today = sprintf("%2.2d-%2.2d-%2.2d %s",
                $today_year, $today_month, $today_dom, $today_time);

    if ($today ge $range1 && $today le $range2) {
      return (0, 1);
    } else {
      return (0, 0);
    }

  
  } else {

    return "Invalid Format in : $f";
  }
}

1;
